@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: dev_asc.c,v 1.58 2005/01/09 01:55:24 debug Exp $
+ *  $Id: dev_asc.c,v 1.67 2005/01/30 13:14:11 debug Exp $
  *
  *  'asc' SCSI controller for some DECstation/DECsystem models, and
  *  for PICA-61.
@@ -56,10 +56,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "memory.h"
-#include "misc.h"
+#include "cpu.h"
 #include "devices.h"
 #include "diskimage.h"
+#include "machine.h"
+#include "memory.h"
+#include "misc.h"
 
 #include "ncr53c9xreg.h"
 
@@ -314,8 +316,10 @@ fatal("TODO..............\n");
 				if (len + (d->dma_address_reg & ((sizeof(d->dma)-1))) > sizeof(d->dma))
 					len = sizeof(d->dma) - (d->dma_address_reg & ((sizeof(d->dma)-1)));
 
-				if (len2 > len)
+				if (len2 > len) {
 					memset(d->dma + (d->dma_address_reg & ((sizeof(d->dma)-1))), 0, len2);
+					len2 = len;
+				}
 
 #ifdef ASC_DEBUG
 				if (!quiet_mode) {
@@ -345,7 +349,9 @@ fatal("TODO..............\n");
 					unsigned char *n;
 
 if (d->dma_controller != NULL)
-	printf("WARNING!!!!!!!!! BUG!!!! Unexpected stuff...\n");
+	printf("WARNING!!!!!!!!! BUG!!!! Unexpected stuff..."
+	    "len2=%i d->xferp->data_in_len=%i\n", (int)len2,
+	    (int)d->xferp->data_in_len);
 
 					all_done = 0;
 					/*  fatal("{ asc: multi-transfer data_in, len=%i len2=%i }\n", len, len2);  */
@@ -1024,7 +1030,8 @@ int dev_asc_access(struct cpu *cpu, struct memory *mem,
 			}
 
 			/*  TODO: not just disk, but some generic SCSI device  */
-			target_exists = diskimage_exist(d->reg_wo[NCR_SELID] & 7);
+			target_exists = diskimage_exist(cpu->machine,
+			    d->reg_wo[NCR_SELID] & 7);
 
 			if (target_exists) {
 				/*
@@ -1188,7 +1195,7 @@ break;
  *
  *  Register an 'asc' device.
  */
-void dev_asc_init(struct cpu *cpu, struct memory *mem,
+void dev_asc_init(struct machine *machine, struct memory *mem,
 	uint64_t baseaddr, int irq_nr, void *turbochannel,
 	int mode,
 	size_t (*dma_controller)(void *dma_controller_data,
@@ -1227,6 +1234,6 @@ void dev_asc_init(struct cpu *cpu, struct memory *mem,
 		    MEM_BINTRANS_OK | MEM_BINTRANS_WRITE_OK, d->dma);
 	}
 
-	cpu_add_tickfunction(cpu, dev_asc_tick, d, 15);
+	machine_add_tickfunction(machine, dev_asc_tick, d, 15);
 }
 
