@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_bt459.c,v 1.51 2005/01/09 01:55:24 debug Exp $
+ *  $Id: dev_bt459.c,v 1.58 2005/01/30 13:14:11 debug Exp $
  *  
  *  Brooktree 459 vdac, used by TURBOchannel graphics cards.
  */
@@ -34,10 +34,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "cpu.h"
 #include "devices.h"
-#include "emul.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
+#include "x11.h"
 
 #include "bt459.h"
 
@@ -205,7 +207,7 @@ static void bt459_update_X_cursor(struct cpu *cpu, struct bt459_data *d)
 	 */
 
 #ifdef WITH_X11
-	if (cpu->emul->use_x11 && d->vfb_data->fb_window != NULL) {
+	if (cpu->machine->use_x11 && d->vfb_data->fb_window != NULL) {
 		for (y=0; y<=ymax; y++) {
 			for (x=0; x<=xmax; x+=4) {
 				struct fb_window *win = d->vfb_data->fb_window;
@@ -570,9 +572,9 @@ int dev_bt459_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr
 /*
  *  dev_bt459_init():
  */
-void dev_bt459_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	uint64_t baseaddr_irq, struct vfb_data *vfb_data, int planes,
-	int irq_nr, int type)
+void dev_bt459_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, uint64_t baseaddr_irq, struct vfb_data *vfb_data,
+	int planes, int irq_nr, int type)
 {
 	struct bt459_data *d = malloc(sizeof(struct bt459_data));
 	if (d == NULL) {
@@ -628,7 +630,7 @@ void dev_bt459_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 		memory_device_register(mem, "bt459_irq", baseaddr_irq, 0x10000,
 		    dev_bt459_irq_access, (void *)d, MEM_DEFAULT, NULL);
 
-	cpu_add_tickfunction(cpu, dev_bt459_tick, d, BT459_TICK_SHIFT);
+	machine_add_tickfunction(machine, dev_bt459_tick, d, BT459_TICK_SHIFT);
 
 	memory_device_register_statefunction(mem, d, bt459_state);
 }
