@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_dc7085.c,v 1.39 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_dc7085.c,v 1.48 2005/02/06 15:39:37 debug Exp $
  *  
  *  DC7085 serial controller, used in some DECstation models.
  */
@@ -35,8 +35,9 @@
 #include <string.h>
 
 #include "console.h"
+#include "cpu.h"
 #include "devices.h"
-#include "emul.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
 
@@ -50,6 +51,8 @@
 struct dc_data {
 	struct dc7085regs	regs;
 
+	int			console_handle;
+
 	/*  For slow_serial_interrupts_hack_for_linux:  */
 	int			just_transmitted_something;
 
@@ -104,7 +107,7 @@ void dev_dc7085_tick(struct cpu *cpu, void *extra)
 	struct dc_data *d = extra;
 	int avail;
 
-	if (cpu->emul->slow_serial_interrupts_hack_for_linux) {
+	if (cpu->machine->slow_serial_interrupts_hack_for_linux) {
 		/*
 		 *  Special hack to prevent Linux from Oopsing. (This makes
 		 *  interrupts not come as fast as possible.)
@@ -280,7 +283,7 @@ do_return:
  *  if a framebuffer device is used. Channel 0 will then be treated as a
  *  DECstation keyboard, instead of a plain serial console.
  */
-void dev_dc7085_init(struct cpu *cpu, struct memory *mem,
+int dev_dc7085_init(struct machine *machine, struct memory *mem,
 	uint64_t baseaddr, int irq_nr, int use_fb)
 {
 	struct dc_data *d;
@@ -297,10 +300,14 @@ void dev_dc7085_init(struct cpu *cpu, struct memory *mem,
 	d->regs.dc_csr = CSR_TRDY | CSR_MSE;
 	d->regs.dc_tcr = 0x00;
 
-	lk201_init(&d->lk201, use_fb, add_to_rx_queue, d);
+	d->console_handle = console_start_slave(machine, "DC7085");
+
+	lk201_init(&d->lk201, use_fb, add_to_rx_queue, d->console_handle, d);
 
 	memory_device_register(mem, "dc7085", baseaddr, DEV_DC7085_LENGTH,
 	    dev_dc7085_access, d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_dc7085_tick, d, DC_TICK_SHIFT);
+	machine_add_tickfunction(machine, dev_dc7085_tick, d, DC_TICK_SHIFT);
+
+	return d->console_handle;
 }
 
