@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_le.c,v 1.29 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_le.c,v 1.38 2005/02/18 06:51:54 debug Exp $
  *  
  *  LANCE ethernet, as used in DECstations.
  *
@@ -44,8 +44,6 @@
  *  tries to read TURBOchannel rom data from 0x1c03f0, and that is provided
  *  by the turbochannel device, not this device.
  *
- *  TODO:  Make sure that this device works with both NetBSD and Ultrix
- *         and on as many DECstation models as possible.
  *
  *  TODO:  Error conditions (such as when there are not enough receive
  *	   buffers) are not emulated yet.
@@ -55,7 +53,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "cpu.h"
 #include "devices.h"
+#include "emul.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
 #include "net.h"
@@ -124,7 +125,7 @@ struct le_data {
  *
  *  Read a 16-bit word from the SRAM.
  */
-uint64_t le_read_16bit(struct le_data *d, int addr)
+static uint64_t le_read_16bit(struct le_data *d, int addr)
 {
 	/*  TODO: This is for little endian only  */
 	int x = d->sram[addr & (SRAM_SIZE-1)] +
@@ -138,7 +139,7 @@ uint64_t le_read_16bit(struct le_data *d, int addr)
  *
  *  Write a 16-bit word to the SRAM.
  */
-void le_write_16bit(struct le_data *d, int addr, uint16_t x)
+static void le_write_16bit(struct le_data *d, int addr, uint16_t x)
 {
 	/*  TODO: This is for little endian only  */
 	d->sram[addr & (SRAM_SIZE-1)] = x & 0xff;
@@ -152,11 +153,12 @@ void le_write_16bit(struct le_data *d, int addr, uint16_t x)
  *  Initialize data structures by reading an 'initialization block' from the
  *  SRAM.
  */
-void le_chip_init(struct le_data *d)
+static void le_chip_init(struct le_data *d)
 {
 	d->init_block_addr = (d->reg[1] & 0xffff) + ((d->reg[2] & 0xff) << 16);
 	if (d->init_block_addr & 1)
-		fatal("[ le: WARNING! initialization block address not word aligned? ]\n");
+		fatal("[ le: WARNING! initialization block address "
+		    "not word aligned? ]\n");
 
 	debug("[ le: d->init_block_addr = 0x%06x ]\n", d->init_block_addr);
 
@@ -224,13 +226,16 @@ void le_chip_init(struct le_data *d)
  *
  *  This routine should only be called if TXON is enabled.
  */
-void le_tx(struct le_data *d)
+static void le_tx(struct net *net, struct le_data *d)
 {
 	int start_txp = d->txp;
 	uint16_t tx_descr[4];
 	int stp, enp, i, cur_packet_offset;
 	uint32_t bufaddr, buflen;
 
+	/*  TODO: This is just a guess:  */
+	d->reg[0] &= ~LE_TDMD;
+
 	do {
 		/*  Load the 8 descriptor bytes:  */
 		tx_descr[0] = le_read_16bit(d, d->tdra + d->txp*8 + 0);
@@ -252,20 +257,23 @@ void le_tx(struct le_data *d)
 			return;
 		if ((tx_descr[2] & 0xf000) != 0xf000)
 			return;
-		if (buflen < 12 || buflen > 1900)
+		if (buflen < 12 || buflen > 1900) {
+			fatal("[ le_tx(): buflen = %i ]\n", buflen);
 			return;
+		}
 
-		debug("[ le: tx descr %3i DUMP: 0x%04x 0x%04x 0x%04x 0x%04x => addr=0x%06x, len=%i bytes, STP=%i ENP=%i ]\n",
-		    d->txp, tx_descr[0], tx_descr[1], tx_descr[2], tx_descr[3],
+		debug("[ le_tx(): descr %3i DUMP: 0x%04x 0x%04x 0x%04x 0x%04x "
+		    "=> addr=0x%06x, len=%i bytes, STP=%i ENP=%i ]\n", d->txp,
+		    tx_descr[0], tx_descr[1], tx_descr[2], tx_descr[3],
 		    bufaddr, buflen, stp, enp);
 
 		if (d->tx_packet == NULL && !stp) {
-			fatal("[ le: le_tx(): !stp but tx_packet == NULL ]\n");
+			fatal("[ le_tx(): !stp but tx_packet == NULL ]\n");
 			return;
 		}
 
 		if (d->tx_packet != NULL && stp) {
-			fatal("[ le: le_tx(): stp but tx_packet != NULL ]\n");
+			fatal("[ le_tx(): stp but tx_packet != NULL ]\n");
 			free(d->tx_packet);
 			d->tx_packet = NULL;
 			d->tx_packet_len = 0;
@@ -279,14 +287,16 @@ void le_tx(struct le_data *d)
 			d->tx_packet_len = buflen;
 			d->tx_packet = malloc(buflen);
 			if (d->tx_packet == NULL) {
-				fprintf(stderr, "out of memory (1) in le_tx()\n");
+				fprintf(stderr, "out of memory (1) in "
+				    "le_tx()\n");
 				exit(1);
 			}
 		} else {
 			d->tx_packet_len += buflen;
 			d->tx_packet = realloc(d->tx_packet, d->tx_packet_len);
 			if (d->tx_packet == NULL) {
-				fprintf(stderr, "out of memory (2) in le_tx()\n");
+				fprintf(stderr, "out of memory (2) in"
+				    " le_tx()\n");
 				exit(1);
 			}
 		}
@@ -304,7 +314,7 @@ void le_tx(struct le_data *d)
 		 *  the packet.
 		 */
 		if (enp) {
-			net_ethernet_tx(d, d->tx_packet, d->tx_packet_len);
+			net_ethernet_tx(net, d, d->tx_packet, d->tx_packet_len);
 
 			free(d->tx_packet);
 			d->tx_packet = NULL;
@@ -328,7 +338,7 @@ void le_tx(struct le_data *d)
 	} while (d->txp != start_txp);
 
 	/*  We are here if all descriptors were taken care of.  */
-	fatal("[ le: le_tx(): all TX descriptors used up? ]\n");
+	fatal("[ le_tx(): all TX descriptors used up? ]\n");
 }
 
 
@@ -337,7 +347,7 @@ void le_tx(struct le_data *d)
  *
  *  This routine should only be called if RXON is enabled.
  */
-void le_rx(struct le_data *d)
+static void le_rx(struct net *net, struct le_data *d)
 {
 	int i, start_rxp = d->rxp;
 	uint16_t rx_descr[4];
@@ -365,11 +375,14 @@ void le_rx(struct le_data *d)
 			return;
 		if ((rx_descr[2] & 0xf000) != 0xf000)
 			return;
-		if (buflen < 12 || buflen > 1900)
+		if (buflen < 12 || buflen > 1900) {
+			fatal("[ le_rx(): buflen = %i ]\n", buflen);
 			return;
+		}
 
-		debug("[ le: rx descr %3i DUMP: 0x%04x 0x%04x 0x%04x 0x%04x => addr=0x%06x, len=%i bytes ]\n",
-		    d->rxp, rx_descr[0], rx_descr[1], rx_descr[2], rx_descr[3],
+		debug("[ le_rx(): descr %3i DUMP: 0x%04x 0x%04x 0x%04x 0x%04x "
+		    "=> addr=0x%06x, len=%i bytes ]\n", d->rxp,
+		    rx_descr[0], rx_descr[1], rx_descr[2], rx_descr[3],
 		    bufaddr, buflen);
 
 		/*  Copy data from the packet into SRAM:  */
@@ -433,19 +446,29 @@ void le_rx(struct le_data *d)
 	} while (d->rxp != start_rxp);
 
 	/*  We are here if all descriptors were taken care of.  */
-	fatal("[ le: le_rx(): all RX descriptors used up? ]\n");
+	fatal("[ le_rx(): all RX descriptors used up? ]\n");
 }
 
 
 /*
  *  le_register_fix():
  */
-void le_register_fix(struct le_data *d)
+static void le_register_fix(struct net *net, struct le_data *d)
 {
-	/*  Should the chip be initialized with a new Initialization block?  */
+	/*  Init with new Initialization block, if needed.  */
 	if (d->reg[0] & LE_INIT)
 		le_chip_init(d);
 
+#ifdef LE_DEBUG
+	{
+		static int x = 1234;
+		if (x != d->reg[0]) {
+			debug("[ le reg[0] = 0x%04x ]\n", d->reg[0]);
+			x = d->reg[0];
+		}
+	}
+#endif
+
 	/*
 	 *  If the receiver is on:
 	 *  If there is a current rx_packet, try to receive it into the
@@ -455,22 +478,23 @@ void le_register_fix(struct le_data *d)
 		do {
 			if (d->rx_packet != NULL)
 				/*  Try to receive the packet:  */
-				le_rx(d);
+				le_rx(net, d);
 
 			if (d->rx_packet != NULL)
 				/*  If the packet wasn't fully received, 
 				    then abort for now.  */
 				break;
 
-			if (d->rx_packet == NULL && net_ethernet_rx_avail(d))
-				net_ethernet_rx(d, &d->rx_packet,
-				    &d->rx_packet_len);
+			if (d->rx_packet == NULL &&
+			    net_ethernet_rx_avail(net, d))
+				net_ethernet_rx(net, d,
+				    &d->rx_packet, &d->rx_packet_len);
 		} while (d->rx_packet != NULL);
 	}
 
 	/*  If the transmitter is on, check for outgoing buffers:  */
 	if (d->reg[0] & LE_TXON)
-		le_tx(d);
+		le_tx(net, d);
 
 	/*  SERR should be the OR of BABL, CERR, MISS, and MERR:  */
 	d->reg[0] &= ~LE_SERR;
@@ -509,7 +533,7 @@ void dev_le_tick(struct cpu *cpu, void *extra)
 {
 	struct le_data *d = (struct le_data *) extra;
 
-	le_register_fix(d);
+	le_register_fix(cpu->machine->emul->net, d);
 
 	if (d->reg[0] & LE_INTR && d->reg[0] & LE_INEA)
 		cpu_interrupt(cpu, d->irq_nr);
@@ -552,7 +576,8 @@ void le_register_write(struct le_data *d, int r, uint32_t x)
 		}
 		if (x & LE_INIT) {
 			if (!(d->reg[r] & LE_STOP))
-				fatal("[ le: attempt to INIT before STOPped! ]\n");
+				fatal("[ le: attempt to INIT before"
+				    " STOPped! ]\n");
 			d->reg[r] |= LE_INIT;
 			d->reg[r] &= ~LE_STOP;
 		}
@@ -577,19 +602,20 @@ void le_register_write(struct le_data *d, int r, uint32_t x)
 /*
  *  dev_le_sram_access():
  */
-int dev_le_sram_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
-	unsigned char *data, size_t len, int writeflag, void *extra)
+int dev_le_sram_access(struct cpu *cpu, struct memory *mem,
+	uint64_t relative_addr, unsigned char *data, size_t len,
+	int writeflag, void *extra)
 {
-	uint64_t idata = 0;
 	int i, retval;
 	struct le_data *d = extra;
 
-	idata = memory_readmax64(cpu, data, len);
-
 #ifdef LE_DEBUG
-	if (writeflag == MEM_WRITE)
-		fatal("[ le_sram: write to addr 0x%06x: 0x%08x ]\n",
-		    relative_addr, idata);
+	if (writeflag == MEM_WRITE) {
+		fatal("[ le_sram: write to addr 0x%06x: ", (int)relative_addr);
+		for (i=0; i<len; i++)
+			fatal("%02x ", data[i]);
+		fatal("]\n");
+	}
 #endif
 
 	/*  Read/write of the SRAM:  */
@@ -635,17 +661,16 @@ int dev_le_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	idata = memory_readmax64(cpu, data, len);
 
 #ifdef LE_DEBUG
-	if (writeflag == MEM_WRITE)
-		fatal("[ le: write to addr 0x%06x: 0x%08x ]\n",
-		    relative_addr, idata);
+	if (writeflag == MEM_WRITE) {
+		fatal("[ le: write to addr 0x%06x: ", (int)relative_addr);
+		for (i=0; i<len; i++)
+			fatal("%02x ", data[i]);
+		fatal("]\n");
+	}
 #endif
 
-	/*  The rest of this code was written before the SRAM
-	    was separated out for bintrans optimization.  */
-	relative_addr += 0x100000;
-
 	/*  Read from station's ROM (ethernet address):  */
-	if (relative_addr >= 0x1c0000 && relative_addr <= 0x1fffff) {
+	if (relative_addr >= 0xc0000 && relative_addr <= 0xfffff) {
 		i = (relative_addr & 0xff) / 4;
 		i = d->rom[i & (ROM_SIZE-1)];
 
@@ -667,12 +692,12 @@ int dev_le_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	switch (relative_addr) {
 
 	/*  Register read/write:  */
-	case 0x100000:
+	case 0:
 		if (writeflag==MEM_READ) {
 			odata = d->reg[d->reg_select];
 			if (!quiet_mode)
-				debug("[ le: read from register 0x%02x: 0x%02x ]\n",
-				    d->reg_select, (int)odata);
+				debug("[ le: read from register 0x%02x: 0x"
+				    "%02x ]\n", d->reg_select, (int)odata);
 			/*
 			 *  A read from csr1..3 should return "undefined"
 			 *  result if the stop bit is set.  However, Ultrix
@@ -681,8 +706,8 @@ int dev_le_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 			 */
 		} else {
 			if (!quiet_mode)
-				debug("[ le: write to register 0x%02x: 0x%02x ]\n",
-				    d->reg_select, (int)idata);
+				debug("[ le: write to register 0x%02x: 0x"
+				    "%02x ]\n", d->reg_select, (int)idata);
 			/*
 			 *  A write to from csr1..3 when the stop bit is
 			 *  set should be ignored. However, Ultrix writes
@@ -694,36 +719,34 @@ int dev_le_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 		break;
 
 	/*  Register select:  */
-	case 0x100004:
+	case 4:
 		if (writeflag==MEM_READ) {
 			odata = d->reg_select;
 			if (!quiet_mode)
-				debug("[ le: read from register select: 0x%02x ]\n",
-				    (int)odata);
+				debug("[ le: read from register select: "
+				    "0x%02x ]\n", (int)odata);
 		} else {
 			if (!quiet_mode)
-				debug("[ le: write to register select: 0x%02x ]\n",
-				    (int)idata);
+				debug("[ le: write to register select: "
+				    "0x%02x ]\n", (int)idata);
 			d->reg_select = idata & (N_REGISTERS - 1);
 			if (idata >= N_REGISTERS)
-				fatal("[ le: WARNING! register select %i (max is %i) ]\n",
-				    idata, N_REGISTERS - 1);
+				fatal("[ le: WARNING! register select %i "
+				    "(max is %i) ]\n", idata, N_REGISTERS - 1);
 		}
 		break;
 
 	default:
 		if (writeflag==MEM_READ) {
 			fatal("[ le: read from UNIMPLEMENTED addr 0x%06x ]\n",
-			    relative_addr);
+			    (int)relative_addr);
 		} else {
-			fatal("[ le: write to UNIMPLEMENTED addr 0x%06x: 0x%08x ]\n",
-			    relative_addr, (int)idata);
+			fatal("[ le: write to UNIMPLEMENTED addr 0x%06x: "
+			    "0x%08x ]\n", (int)relative_addr, (int)idata);
 		}
 	}
 
-
 do_return:
-
 	if (writeflag == MEM_READ) {
 		memory_writemax64(cpu, data, len, odata);
 #ifdef LE_DEBUG
@@ -741,10 +764,12 @@ do_return:
 /*
  *  dev_le_init():
  */
-void dev_le_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
+void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
 	uint64_t buf_start, uint64_t buf_end, int irq_nr, int len)
 {
+	char *name2;
 	struct le_data *d = malloc(sizeof(struct le_data));
+
 	if (d == NULL) {
 		fprintf(stderr, "out of memory\n");
 		exit(1);
@@ -765,12 +790,7 @@ void dev_le_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	d->rx_packet = NULL;
 
 	/*  ROM (including the MAC address):  */
-	d->rom[0] = 0x10;
-	d->rom[1] = 0x20;
-	d->rom[2] = 0x30;
-	d->rom[3] = 0x40;
-	d->rom[4] = 0x50;
-	d->rom[5] = 0x60;
+	net_generate_unique_mac(&d->rom[0]);
 
 	/*
 	 *  NOTE:  According to the Lance documentation, the low order bit of
@@ -798,10 +818,19 @@ void dev_le_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	    SRAM_SIZE, dev_le_sram_access, (void *)d,
 	    MEM_BINTRANS_OK | MEM_BINTRANS_WRITE_OK, d->sram);
 
-	memory_device_register(mem, "le", baseaddr + 0x100000,
-	    len - 0x100000,
-	    dev_le_access, (void *)d, MEM_DEFAULT, NULL);
+	name2 = malloc(50);
+	if (name2 == NULL) {
+		fprintf(stderr, "out of memory in dev_le_init()\n");
+		exit(1);
+	}
+	sprintf(name2, "le [%02x:%02x:%02x:%02x:%02x:%02x]",
+	    d->rom[0], d->rom[1], d->rom[2], d->rom[3], d->rom[4], d->rom[5]);
 
-	cpu_add_tickfunction(cpu, dev_le_tick, d, LE_TICK_SHIFT);
+	memory_device_register(mem, name2, baseaddr + 0x100000,
+	    len - 0x100000, dev_le_access, (void *)d, MEM_DEFAULT, NULL);
+
+	machine_add_tickfunction(machine, dev_le_tick, d, LE_TICK_SHIFT);
+
+	net_add_nic(machine->emul->net, d, &d->rom[0]);
 }
 
