@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,19 +25,21 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_mp.c,v 1.16 2005/01/09 01:55:25 debug Exp $
- *  
+ *  $Id: dev_mp.c,v 1.23 2005/02/18 06:32:39 debug Exp $
+ *
  *  This is a fake multiprocessor (MP) device. It can be useful for
  *  theoretical experiments, but probably bares no resemblance to any
- *  actual multiprocessor controller for any MIPS-based machine.
+ *  multiprocessor controller used in any real machine.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "cpu.h"
+#include "cpu_mips.h"
 #include "devices.h"
-#include "emul.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
 #include "mp.h"
@@ -76,13 +78,18 @@ int dev_mp_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 		break;
 
 	case DEV_MP_NCPUS:
-		odata = cpu->emul->ncpus;
+		odata = cpu->machine->ncpus;
 		break;
 
 	case DEV_MP_STARTUPCPU:
+		if (cpu->machine->arch != ARCH_MIPS) {
+			fatal("dev_mp_access(): DEV_MP_STARTUPCPU:"
+			    " not for !MIPS yet\n");
+			exit(1);
+		}
 		which_cpu = idata;
-		d->cpus[which_cpu]->pc = d->startup_addr;
-		d->cpus[which_cpu]->gpr[GPR_SP] = d->stack_addr;
+		d->cpus[which_cpu]->cd.mips.pc = d->startup_addr;
+		d->cpus[which_cpu]->cd.mips.gpr[MIPS_GPR_SP] = d->stack_addr;
 		d->cpus[which_cpu]->running = 1;
 		/*  debug("[ dev_mp: starting up cpu%i at 0x%llx ]\n", 
 		    which_cpu, (long long)d->startup_addr);  */
@@ -102,7 +109,7 @@ int dev_mp_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 		/*  Pause all cpus except our selves:  */
 		which_cpu = idata;
 
-		for (i=0; i<cpu->emul->ncpus; i++)
+		for (i=0; i<cpu->machine->ncpus; i++)
 			if (i!=which_cpu)
 				d->cpus[i]->running = 0;
 		break;
@@ -110,7 +117,7 @@ int dev_mp_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	case DEV_MP_UNPAUSE_CPU:
 		/*  Unpause all cpus except our selves:  */
 		which_cpu = idata;
-		for (i=0; i<cpu->emul->ncpus; i++)
+		for (i=0; i<cpu->machine->ncpus; i++)
 			if (i!=which_cpu)
 				d->cpus[i]->running = 1;
 		break;
@@ -128,6 +135,17 @@ int dev_mp_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 		odata = (odata << 31) ^ random();
 		break;
 
+	case DEV_MP_MEMORY:
+		/*
+		 *  Return the number of bytes of memory in the system.
+		 *
+		 *  (It is assumed to be located at physical address 0.
+		 *  It is actually located at machine->memory_offset_in_mb
+		 *  but that is only used for SGI emulation so far.)
+		 */
+		odata = cpu->machine->physical_ram_in_mb * 1048576;
+		break;
+
 	default:
 		fatal("[ dev_mp: unimplemented relative addr 0x%x ]\n",
 		    relative_addr);
@@ -143,7 +161,8 @@ int dev_mp_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 /*
  *  dev_mp_init():
  */
-void dev_mp_init(struct memory *mem, struct cpu *cpus[])
+void dev_mp_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr)
 {
 	struct mp_data *d;
 	d = malloc(sizeof(struct mp_data));
@@ -152,11 +171,11 @@ void dev_mp_init(struct memory *mem, struct cpu *cpus[])
 		exit(1);
 	}
 	memset(d, 0, sizeof(struct mp_data));
-	d->cpus = cpus;
+	d->cpus = machine->cpus;
 	d->startup_addr = INITIAL_PC;
 	d->stack_addr = INITIAL_STACK_POINTER;
 
-	memory_device_register(mem, "mp", DEV_MP_ADDRESS, DEV_MP_LENGTH,
+	memory_device_register(mem, "mp", baseaddr, DEV_MP_LENGTH,
 	    dev_mp_access, d, MEM_DEFAULT, NULL);
 }
 
