@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ns16550.c,v 1.24 2005/01/09 02:54:41 debug Exp $
+ *  $Id: dev_ns16550.c,v 1.32 2005/02/19 11:51:33 debug Exp $
  *  
  *  NS16550 serial controller.
  *
@@ -36,10 +36,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "console.h"
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "console.h"
-#include "devices.h"
 
 #include "comreg.h"
 
@@ -53,8 +55,11 @@
 
 struct ns_data {
 	int		reg[8];
-	int		irq_enable;
+
 	int		irqnr;
+	int		console_handle;
+
+	int		irq_enable;
 	int		addrmult;
 	int		in_use;
 	int		dlab;		/*  Divisor Latch Access bit  */
@@ -67,7 +72,6 @@ struct ns_data {
 
 /*
  *  dev_ns16550_tick():
- *
  */
 void dev_ns16550_tick(struct cpu *cpu, void *extra)
 {
@@ -78,7 +82,7 @@ void dev_ns16550_tick(struct cpu *cpu, void *extra)
 
 	d->reg[com_iir] &= ~IIR_RXRDY;
 	if (d->in_use) {
-		if (console_charavail())
+		if (console_charavail(d->console_handle))
 			d->reg[com_iir] |= IIR_RXRDY;
 	}
 
@@ -115,7 +119,7 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 #endif
 
 	if (d->in_use) {
-		if (console_charavail()) {
+		if (console_charavail(d->console_handle)) {
 			d->reg[com_lsr] |= LSR_RXRDY;
 		}
 	}
@@ -138,19 +142,22 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 
 		/*  Read write of data:  */
 		if (writeflag == MEM_WRITE) {
-			/*  Ugly hack: don't show form feeds:  */
-			if (idata != 12)
-				console_putchar(idata);
-
-			if (d->reg[com_mcr] & MCR_LOOPBACK)
-				console_makeavail(idata);
+			if (d->reg[com_mcr] & MCR_LOOPBACK) {
+				console_makeavail(d->console_handle, idata);
+			} else {
+#if 0
+				/*  Ugly hack: don't show form feeds:  */
+				if (idata != 12)
+#endif
+				console_putchar(d->console_handle, idata);
+			}
 
 			d->reg[com_iir] |= IIR_TXRDY;
 			dev_ns16550_tick(cpu, d);
 			return 1;
 		} else {
 			if (d->in_use)
-				odata = console_readchar();
+				odata = console_readchar(d->console_handle);
 			else
 				odata = 0;
 			dev_ns16550_tick(cpu, d);
@@ -163,7 +170,8 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 				/*  Set the high byte of the divisor:  */
 				d->divisor &= ~0xff00;
 				d->divisor |= ((idata & 0xff) << 8);
-				debug("[ ns16550 speed set to %i bps ]\n", 115200 / d->divisor);
+				debug("[ ns16550 speed set to %i bps ]\n",
+				    115200 / d->divisor);
 			} else {
 				odata = (d->divisor & 0xff00) >> 8;
 			}
@@ -172,8 +180,10 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 
 		/*  IER:  */
 		if (writeflag == MEM_WRITE) {
-			if (idata != 0)		/*  <-- to supress Linux' behaviour  */
-				debug("[ ns16550 write to ier: 0x%02x ]\n", idata);
+			/*  This is to supress Linux' behaviour  */
+			if (idata != 0)
+				debug("[ ns16550 write to ier: 0x%02x ]\n",
+				    idata);
 
 			/*  Needed for NetBSD 2.0, but not 1.6.2?  */
 			if (!(d->irq_enable & IER_ETXRDY)
@@ -238,7 +248,8 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 
 			d->dlab = idata & 0x80? 1 : 0;
 
-			debug("[ ns16550 write to lctl: 0x%02x (%s%ssetting mode %i%c%s) ]\n",
+			debug("[ ns16550 write to lctl: 0x%02x (%s%s"
+			    "setting mode %i%c%s) ]\n",
 			    (int)idata,
 			    d->dlab? "Divisor Latch access, " : "",
 			    idata&0x40? "sending BREAK, " : "",
@@ -259,10 +270,12 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 		break;
 	default:
 		if (writeflag==MEM_READ) {
-			debug("[ ns16550 read from reg %i ]\n", (int)relative_addr);
+			debug("[ ns16550 read from reg %i ]\n",
+			    (int)relative_addr);
 			odata = d->reg[relative_addr];
 		} else {
-			debug("[ ns16550 write to reg %i:", (int)relative_addr);
+			debug("[ ns16550 write to reg %i:",
+			    (int)relative_addr);
 			for (i=0; i<len; i++)
 				debug(" %02x", data[i]);
 			debug(" ]\n");
@@ -280,10 +293,12 @@ int dev_ns16550_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_ns16550_init():
  */
-void dev_ns16550_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	int irq_nr, int addrmult, int in_use)
+int dev_ns16550_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, int irq_nr, int addrmult, int in_use,
+	char *name)
 {
 	struct ns_data *d;
+	char *name2;
 
 	d = malloc(sizeof(struct ns_data));
 	if (d == NULL) {
@@ -294,16 +309,29 @@ void dev_ns16550_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	d->irqnr    = irq_nr;
 	d->addrmult = addrmult;
 	d->in_use   = in_use;
-
-	d->dlab = 0;
+	d->dlab     = 0;
 	d->divisor  = 115200 / 9600;
 	d->databits = 8;
 	d->parity   = 'N';
 	d->stopbits = "1";
+	d->console_handle = console_start_slave(machine, name);
 
-	memory_device_register(mem, "ns16550", baseaddr,
+	name2 = malloc(strlen(name) + 20);
+	if (name2 == NULL) {
+		fprintf(stderr, "out of memory in dev_ns16550_init()\n");
+		exit(1);
+	}
+	if (name != NULL && name[0])
+		sprintf(name2, "ns16550 [%s]", name);
+	else
+		sprintf(name2, "ns16550");
+
+	memory_device_register(mem, name2, baseaddr,
 	    DEV_NS16550_LENGTH * addrmult, dev_ns16550_access, d,
 	    MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_ns16550_tick, d, NS16550_TICK_SHIFT);
+	machine_add_tickfunction(machine, dev_ns16550_tick,
+	    d, NS16550_TICK_SHIFT);
+
+	return d->console_handle;
 }
 
