@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pckbc.c,v 1.32 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_pckbc.c,v 1.36 2005/02/07 07:12:25 debug Exp $
  *  
  *  Standard 8042 PC keyboard controller, and a 8242WB PS2 keyboard/mouse
  *  controller.
@@ -38,10 +38,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "console.h"
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "console.h"
-#include "devices.h"
 
 #include "kbdreg.h"
 
@@ -65,6 +67,7 @@
 #define	PCKBC_TICKSHIFT		14
 
 struct pckbc_data {
+	int		console_handle;
 	int		in_use;
 	int		any_command_used;
 
@@ -271,8 +274,9 @@ void dev_pckbc_tick(struct cpu *cpu, void *extra)
 	int port_nr;
 	int ch;
 
-	if (d->in_use && d->any_command_used && console_charavail()) {
-		ch = console_readchar();
+	if (d->in_use && d->any_command_used &&
+	    console_charavail(d->console_handle)) {
+		ch = console_readchar(d->console_handle);
 		if (ch >= 0)
 			ascii_to_pc_scancodes(ch, d);
 	}
@@ -595,8 +599,9 @@ int dev_pckbc_access(struct cpu *cpu, struct memory *mem,
  *
  *  Type should be PCKBC_8042 or PCKBC_8242.
  */
-void dev_pckbc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	int type, int keyboard_irqnr, int mouse_irqnr, int in_use)
+int dev_pckbc_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, int type, int keyboard_irqnr, int mouse_irqnr,
+	int in_use)
 {
 	struct pckbc_data *d;
 	int len = DEV_PCKBC_LENGTH;
@@ -617,9 +622,12 @@ void dev_pckbc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	d->keyboard_irqnr = keyboard_irqnr;
 	d->mouse_irqnr    = mouse_irqnr;
 	d->in_use         = in_use;
+	d->console_handle = console_start_slave_inputonly(machine, "pckbc");
 
 	memory_device_register(mem, "pckbc", baseaddr,
 	    len, dev_pckbc_access, d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_pckbc_tick, d, PCKBC_TICKSHIFT);
+	machine_add_tickfunction(machine, dev_pckbc_tick, d, PCKBC_TICKSHIFT);
+
+	return d->console_handle;
 }
 
