@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ps2_stuff.c,v 1.10 2005/01/12 08:45:49 debug Exp $
+ *  $Id: dev_ps2_stuff.c,v 1.16 2005/02/11 09:29:48 debug Exp $
  *  
  *  Playstation 2 misc. stuff:
  *
@@ -38,12 +38,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "devices.h"
 
-#include "ps2_dmacreg.h"
 #include "ee_timerreg.h"
+#include "ps2_dmacreg.h"
 
 #define	TICK_STEPS_SHIFT	16
 
@@ -163,20 +165,28 @@ int dev_ps2_stuff_access(struct cpu *cpu, struct memory *mem,
 
 				copy_buf = malloc(length);
 				if (copy_buf == NULL) {
-					fprintf(stderr, "out of memory in dev_ps2_stuff_access()\n");
+					fprintf(stderr, "out of memory in "
+					    "dev_ps2_stuff_access()\n");
 					exit(1);
 				}
-				memory_rw(cpu, cpu->mem, from_addr, copy_buf, length, MEM_READ, CACHE_NONE | PHYSICAL);
-				memory_rw(cpu, cpu->mem, d->other_memory_base[DMA_CH_GIF] + to_addr, copy_buf, length, MEM_WRITE, CACHE_NONE | PHYSICAL);
+				cpu->memory_rw(cpu, cpu->mem, from_addr,
+				    copy_buf, length, MEM_READ,
+				    CACHE_NONE | PHYSICAL);
+				cpu->memory_rw(cpu, cpu->mem,
+				    d->other_memory_base[DMA_CH_GIF] + to_addr,
+				    copy_buf, length, MEM_WRITE,
+				    CACHE_NONE | PHYSICAL);
 				free(copy_buf);
 
 				/*  Done with the transfer:  */
 				d->dmac_reg[D2_QWC_REG/0x10] = 0;
 				idata &= ~D_CHCR_STR;
 
-				cpu_interrupt(cpu, (1 << 18) +8);		/*  1<<18 is dma2  */
+				cpu_interrupt(cpu, (1 << 18) +8);
+				    /*  1<<18 is dma2  */
 			} else
-				debug("[ ps2_stuff: dmac [ch2] stopping transfer ]\n");
+				debug("[ ps2_stuff: dmac [ch2] stopping "
+				    "transfer ]\n");
 			d->dmac_reg[regnr] = idata;
 			return 1;
 		}
@@ -250,8 +260,8 @@ int dev_ps2_stuff_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_ps2_stuff_init():
  */
-struct ps2_data *dev_ps2_stuff_init(struct cpu *cpu, struct memory *mem,
-	uint64_t baseaddr)
+struct ps2_data *dev_ps2_stuff_init(struct machine *machine,
+	struct memory *mem, uint64_t baseaddr)
 {
 	struct ps2_data *d;
 
@@ -266,7 +276,8 @@ struct ps2_data *dev_ps2_stuff_init(struct cpu *cpu, struct memory *mem,
 
 	memory_device_register(mem, "ps2_stuff", baseaddr,
 	    DEV_PS2_STUFF_LENGTH, dev_ps2_stuff_access, d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_ps2_stuff_tick, d, TICK_STEPS_SHIFT);
+	machine_add_tickfunction(machine,
+	    dev_ps2_stuff_tick, d, TICK_STEPS_SHIFT);
 
 	return d;
 }
