@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_px.c,v 1.20 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_px.c,v 1.26 2005/02/18 07:29:56 debug Exp $
  *  
  *  TURBOchannel Pixelstamp graphics device.
  *
@@ -40,26 +40,34 @@
  *  boards are recognizes under different names depending on operating system:
  *
  *	NetBSD/pmax:  (works fine both with and without console on framebuffer)
- *		PMAG-CA:	px0 at tc0 slot 0 offset 0x0: 2D, 4x1 stamp, 8 plane
- *		PMAG-DA:	px0 at tc0 slot 0 offset 0x0: 3D, 4x1 stamp, 8 plane, 128KB SRAM
+ *		PMAG-CA:	px0 at tc0 slot 0 offset 0x0: 2D, 4x1 stamp,
+ *				  8 plane
+ *		PMAG-DA:	px0 at tc0 slot 0 offset 0x0: 3D, 4x1 stamp,
+ *				  8 plane, 128KB SRAM
  *		PMAG-EA:	(not supported)
- *		PMAG-FA:	px0 at tc0 slot 0 offset 0x0: 3D, 5x2 stamp, 24 plane, 128KB SRAM
+ *		PMAG-FA:	px0 at tc0 slot 0 offset 0x0: 3D, 5x2 stamp,
+ *				  24 plane, 128KB SRAM
  *
- *	Ultrix 4.2A rev 47:  (usually crashes if the device is installed, but serial console is used)
+ *	Ultrix 4.2A rev 47:  (usually crashes if the device is installed, but
+ *						serial console is used)
  *		PMAG-CA:	px0 at ibus0, pa0 (5x1 8+8+0+0)
- *		PMAG-DA:	px0 at ibus0, pq0 (5x1 16+16+16+0 128KB)    or (5x1 0+0+16+0 128KB)
+ *		PMAG-DA:	px0 at ibus0, pq0 (5x1 16+16+16+0 128KB)
+ *				    or (5x1 0+0+16+0 128KB)
  *		PMAG-EA:	(not supported)
  *		PMAG-FA:	px0 at ibus0, pq0 (5x2 24+24+16+16 128KB)
  *
- *	Ultrix 4.2 rev 85:  (usually crashes if the device is installed, but serial console is used)
+ *	Ultrix 4.2 rev 85:  (usually crashes if the device is installed,
+ *					 but serial console is used)
  *		PMAG-CA:	ga0 at ibus0, ga0 ( 8 planes 4x1 stamp )
  *		PMAG-DA:	gq0 at ibus0, gq0 ( 8+8+16Z+0X plane 4x1 stamp )
  *		PMAG-EA:	(not supported)
- *		PMAG-FA:	gq0 at ibus0, gq0 ( 24+24+24Z+24X plane 5x2 stamp )  (crashes in serial console mode)
+ *		PMAG-FA:	gq0 at ibus0, gq0 ( 24+24+24Z+24X plane
+ *				 5x2 stamp )  (crashes in serial console mode)
  *
  *  TODO:  A lot of stuff:
  *
- *	Read http://www.mit.edu/afs/athena/system/pmax_ul3/srvd.73/sys/io/tc/gq.h
+ *	Read http://www.mit.edu/afs/athena/system/pmax_ul3/srvd.73/sys/
+ *		io/tc/gq.h
  *	and try to figure out the interrupt and memory management stuff.
  *
  *	Color support: foreground, background, 8-bit palette?
@@ -76,9 +84,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "devices.h"
 
 #include "pxreg.h"
 
@@ -113,9 +123,11 @@ void dev_px_tick(struct cpu *cpu, void *extra)
 uint32_t px_readword(struct cpu *cpu, unsigned char *dma_buf, int ofs)
 {
 	if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-		return dma_buf[ofs+0] + (dma_buf[ofs+1] << 8) + (dma_buf[ofs+2] << 16) + (dma_buf[ofs+3] << 24);
+		return dma_buf[ofs+0] + (dma_buf[ofs+1] << 8) +
+		    (dma_buf[ofs+2] << 16) + (dma_buf[ofs+3] << 24);
 	else
-		return dma_buf[ofs+3] + (dma_buf[ofs+2] << 8) + (dma_buf[ofs+1] << 16) + (dma_buf[ofs+0] << 24);
+		return dma_buf[ofs+3] + (dma_buf[ofs+2] << 8) +
+		    (dma_buf[ofs+1] << 16) + (dma_buf[ofs+0] << 24);
 }
 
 
@@ -137,17 +149,23 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 
 	bytesperpixel = d->bitdepth >> 3;
 
-	dma_len = 56 * 4;	/*  TODO: this is just enough for NetBSD's putchar  */
+	dma_len = 56 * 4;	/*  TODO: this is just enough for NetBSD's
+					 putchar  */
 
-	if (d->type == DEV_PX_TYPE_PX)
-		memory_rw(cpu, cpu->mem, sys_addr, dma_buf, dma_len, MEM_READ, NO_EXCEPTIONS | PHYSICAL);
-	else
-		memmove(dma_buf, &d->sram[sys_addr & 0x1ffff], dma_len);	/*  TODO:  past end of sram?  */
+	if (d->type == DEV_PX_TYPE_PX) {
+		cpu->memory_rw(cpu, cpu->mem, sys_addr, dma_buf,
+		    dma_len, MEM_READ, NO_EXCEPTIONS | PHYSICAL);
+	} else {
+		/*  TODO:  past end of sram?  */
+		memmove(dma_buf, &d->sram[sys_addr & 0x1ffff], dma_len);
+	}
 
 	if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-		cmdword = dma_buf[0] + (dma_buf[1] << 8) + (dma_buf[2] << 16) + (dma_buf[3] << 24);
+		cmdword = dma_buf[0] + (dma_buf[1] << 8) +
+		    (dma_buf[2] << 16) + (dma_buf[3] << 24);
 	else
-		cmdword = dma_buf[3] + (dma_buf[2] << 8) + (dma_buf[1] << 16) + (dma_buf[0] << 24);
+		cmdword = dma_buf[3] + (dma_buf[2] << 8) +
+		    (dma_buf[1] << 16) + (dma_buf[0] << 24);
 
 #ifdef PX_DEBUG
 	debug("[ px: dma from 0x%08x: ", (int)sys_addr);
@@ -229,14 +247,18 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 		/*  unsigned char pixels[PX_XSIZE * 3];  */
 
 		if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-			nspans = dma_buf[4] + (dma_buf[5] << 8) + (dma_buf[6] << 16) + (dma_buf[7] << 24);
+			nspans = dma_buf[4] + (dma_buf[5] << 8) +
+			    (dma_buf[6] << 16) + (dma_buf[7] << 24);
 		else
-			nspans = dma_buf[7] + (dma_buf[6] << 8) + (dma_buf[5] << 16) + (dma_buf[4] << 24);
+			nspans = dma_buf[7] + (dma_buf[6] << 8) +
+			    (dma_buf[5] << 16) + (dma_buf[4] << 24);
 
 		if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-			lw = dma_buf[16] + (dma_buf[17] << 8) + (dma_buf[18] << 16) + (dma_buf[19] << 24);
+			lw = dma_buf[16] + (dma_buf[17] << 8) +
+			    (dma_buf[18] << 16) + (dma_buf[19] << 24);
 		else
-			lw = dma_buf[19] + (dma_buf[18] << 8) + (dma_buf[17] << 16) + (dma_buf[16] << 24);
+			lw = dma_buf[19] + (dma_buf[18] << 8) +
+			    (dma_buf[17] << 16) + (dma_buf[16] << 24);
 
 		nspans >>= 24;
 		/*  Why not this?  lw = (lw + 1) >> 2;  */
@@ -249,29 +271,44 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 		if (dma_len < 4*(5 + nspans*3)) {
 			dma_len = 4 * (5+nspans*3);
 			if (d->type == DEV_PX_TYPE_PX)
-				memory_rw(cpu, cpu->mem, sys_addr, dma_buf, dma_len, MEM_READ, NO_EXCEPTIONS | PHYSICAL);
+				cpu->memory_rw(cpu, cpu->mem, sys_addr,
+				    dma_buf, dma_len, MEM_READ,
+				    NO_EXCEPTIONS | PHYSICAL);
 			else
-				memmove(dma_buf, &d->sram[sys_addr & 0x1ffff], dma_len);	/*  TODO:  past end of sram?  */
+				memmove(dma_buf, &d->sram[sys_addr & 0x1ffff],
+				    dma_len);	/*  TODO:  past end of sram?  */
 		}
 
 		ofs = 4*5;
 		for (spannr=0; spannr<nspans; spannr++) {
 			if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-				span_len = dma_buf[ofs+0] + (dma_buf[ofs+1] << 8) + (dma_buf[ofs+2] << 16) + (dma_buf[ofs+3] << 24);
+				span_len = dma_buf[ofs+0] + (dma_buf[ofs+1] <<
+				    8) + (dma_buf[ofs+2] << 16) +
+				    (dma_buf[ofs+3] << 24);
 			else
-				span_len = dma_buf[ofs+3] + (dma_buf[ofs+2] << 8) + (dma_buf[ofs+1] << 16) + (dma_buf[ofs+0] << 24);
+				span_len = dma_buf[ofs+3] + (dma_buf[ofs+2] <<
+				    8) + (dma_buf[ofs+1] << 16) +
+				    (dma_buf[ofs+0] << 24);
 			ofs += 4;
 
 			if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-				span_src = dma_buf[ofs+0] + (dma_buf[ofs+1] << 8) + (dma_buf[ofs+2] << 16) + (dma_buf[ofs+3] << 24);
+				span_src = dma_buf[ofs+0] + (dma_buf[ofs+1] <<
+				    8) + (dma_buf[ofs+2] << 16) +
+				    (dma_buf[ofs+3] << 24);
 			else
-				span_src = dma_buf[ofs+3] + (dma_buf[ofs+2] << 8) + (dma_buf[ofs+1] << 16) + (dma_buf[ofs+0] << 24);
+				span_src = dma_buf[ofs+3] + (dma_buf[ofs+2] <<
+				    8) + (dma_buf[ofs+1] << 16) +
+				    (dma_buf[ofs+0] << 24);
 			ofs += 4;
 
 			if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-				span_dst = dma_buf[ofs+0] + (dma_buf[ofs+1] << 8) + (dma_buf[ofs+2] << 16) + (dma_buf[ofs+3] << 24);
+				span_dst = dma_buf[ofs+0] + (dma_buf[ofs+1] <<
+				    8) + (dma_buf[ofs+2] << 16) +
+				    (dma_buf[ofs+3] << 24);
 			else
-				span_dst = dma_buf[ofs+3] + (dma_buf[ofs+2] << 8) + (dma_buf[ofs+1] << 16) + (dma_buf[ofs+0] << 24);
+				span_dst = dma_buf[ofs+3] + (dma_buf[ofs+2] <<
+				    8) + (dma_buf[ofs+1] << 16) +
+				    (dma_buf[ofs+0] << 24);
 			ofs += 4;
 
 			span_len >>= 3;
@@ -281,13 +318,16 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 			if (span_len > PX_XSIZE)
 				span_len = PX_XSIZE;
 
-			/*  debug("  span %i: len=%i src=%i dst=%i\n", spannr, span_len, span_src, span_dst);  */
+			/*  debug("  span %i: len=%i src=%i dst=%i\n",
+			    spannr, span_len, span_src, span_dst);  */
 
-			memmove(d->vfb_data->framebuffer + span_dst * PX_XSIZE * bytesperpixel,
-			        d->vfb_data->framebuffer + span_src * PX_XSIZE * bytesperpixel,
-			        span_len * bytesperpixel);
+			memmove(d->vfb_data->framebuffer + span_dst *
+			    PX_XSIZE * bytesperpixel, d->vfb_data->framebuffer
+			    + span_src * PX_XSIZE * bytesperpixel, span_len *
+			    bytesperpixel);
 
-			d->vfb_data->update_x1 = 0; d->vfb_data->update_x2 = PX_XSIZE-1;
+			d->vfb_data->update_x1 = 0; d->vfb_data->update_x2 =
+			    PX_XSIZE-1;
 			if (span_dst < d->vfb_data->update_y1)
 				d->vfb_data->update_y1 = span_dst;
 			if (span_dst > d->vfb_data->update_y2)
@@ -313,19 +353,25 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 		v2 = px_readword(cpu, dma_buf, 28);
 #if 0
 		if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-			lw = dma_buf[16] + (dma_buf[17] << 8) + (dma_buf[18] << 16) + (dma_buf[19] << 24);
+			lw = dma_buf[16] + (dma_buf[17] << 8) +
+			    (dma_buf[18] << 16) + (dma_buf[19] << 24);
 		else
-			lw = dma_buf[19] + (dma_buf[18] << 8) + (dma_buf[17] << 16) + (dma_buf[16] << 24);
+			lw = dma_buf[19] + (dma_buf[18] << 8) +
+			    (dma_buf[17] << 16) + (dma_buf[16] << 24);
 
 		if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-			v1 = dma_buf[24] + (dma_buf[25] << 8) + (dma_buf[26] << 16) + (dma_buf[27] << 24);
+			v1 = dma_buf[24] + (dma_buf[25] << 8) +
+			    (dma_buf[26] << 16) + (dma_buf[27] << 24);
 		else
-			v1 = dma_buf[27] + (dma_buf[26] << 8) + (dma_buf[25] << 16) + (dma_buf[24] << 24);
+			v1 = dma_buf[27] + (dma_buf[26] << 8) +
+			    (dma_buf[25] << 16) + (dma_buf[24] << 24);
 
 		if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-			v2 = dma_buf[28] + (dma_buf[29] << 8) + (dma_buf[30] << 16) + (dma_buf[31] << 24);
+			v2 = dma_buf[28] + (dma_buf[29] << 8) +
+			    (dma_buf[30] << 16) + (dma_buf[31] << 24);
 		else
-			v2 = dma_buf[31] + (dma_buf[30] << 8) + (dma_buf[29] << 16) + (dma_buf[28] << 24);
+			v2 = dma_buf[31] + (dma_buf[30] << 8) +
+			    (dma_buf[29] << 16) + (dma_buf[28] << 24);
 #endif
 		bg_r = (attr >> 16) & 255;
 		bg_g = (attr >> 8) & 255;
@@ -352,7 +398,9 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 			x2 = PX_XSIZE;
 
 #ifdef PX_DEBUG
-		debug("[ px: clear/fill: v1 = 0x%08x  v2 = 0x%08x lw=%i x=%i y=%i x2=%i y2=%i ]\n", (int)v1, (int)v2, lw, x,y, x2,y2);
+		debug("[ px: clear/fill: v1 = 0x%08x  v2 = 0x%08x "
+		    "lw=%i x=%i y=%i x2=%i y2=%i ]\n", (int)v1, (int)v2,
+		    lw, x,y, x2,y2);
 #endif
 		if (bytesperpixel == 3) {
 			int xi;
@@ -371,7 +419,8 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 			d->vfb_data->update_x2 = x2;
 
 		for (fb_y=y; fb_y < y2 + lw; fb_y ++) {
-			memcpy(d->vfb_data->framebuffer + (fb_y * PX_XSIZE + x) * bytesperpixel, pixels, (x2-x)*bytesperpixel);
+			memcpy(d->vfb_data->framebuffer + (fb_y * PX_XSIZE + x)
+			    * bytesperpixel, pixels, (x2-x)*bytesperpixel);
 
 			if (fb_y < d->vfb_data->update_y1)
 				d->vfb_data->update_y1 = fb_y;
@@ -397,8 +446,11 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 		fgcolor = px_readword(cpu, dma_buf, 16 * 4);
 		bgcolor = px_readword(cpu, dma_buf, 29 * 4);
 
-		/*  TODO:  Which one is r, which one is g, and which one is b?  */
-		/*  TODO 2:  Use the BT459 palette, these values are hardcoded for NetBSD and Ultrix grayscale only  */
+		/*
+		 *  TODO:  Which one is r, which one is g, and which one is b?
+		 *  TODO 2:  Use the BT459 palette, these values are hardcoded
+		 *  for NetBSD and Ultrix grayscale only.
+		 */
 		fg_r = (fgcolor >> 16) & 255;
 		fg_g = (fgcolor >> 8) & 255;
 		fg_b = fgcolor & 255;
@@ -427,7 +479,8 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 		y2 = ((v2 - 63) >> 3) & 1023;
 
 #ifdef PX_DEBUG
-		debug("[ px putchar: v1 = 0x%08x  v2 = 0x%08x x=%i y=%i ]\n", (int)v1, (int)v2, x,y, x2,y2);
+		debug("[ px putchar: v1 = 0x%08x  v2 = 0x%08x x=%i y=%i ]\n",
+		    (int)v1, (int)v2, x,y, x2,y2);
 #endif
 		x %= PX_XSIZE;
 		y %= PX_YSIZE;
@@ -450,21 +503,41 @@ void dev_px_dma(struct cpu *cpu, uint32_t sys_addr, struct px_data *d)
 				for (xbit = 0; xbit < 8; xbit ++) {
 					if (bytesperpixel == 3) {
 						/*  24-bit:  */
-						/*  TODO:  Which one is r, which one is g, which one is b?  */
-						pixels[xbit * 3 + 0]       = (dma_buf[i*4 + j*2 + 0] & (1 << xbit))? fg_r : bg_r;
-						pixels[xbit * 3 + 1]       = (dma_buf[i*4 + j*2 + 0] & (1 << xbit))? fg_g : bg_g;
-						pixels[xbit * 3 + 2]       = (dma_buf[i*4 + j*2 + 0] & (1 << xbit))? fg_b : bg_b;
-						pixels[(xbit + 8) * 3 + 0] = (dma_buf[i*4 + j*2 + 1] & (1 << xbit))? fg_r : bg_r;
-						pixels[(xbit + 8) * 3 + 1] = (dma_buf[i*4 + j*2 + 1] & (1 << xbit))? fg_g : bg_g;
-						pixels[(xbit + 8) * 3 + 2] = (dma_buf[i*4 + j*2 + 1] & (1 << xbit))? fg_b : bg_b;
+						/*  TODO:  Which one is r,
+						    which one is g, and b?  */
+						pixels[xbit * 3 + 0] =
+						    (dma_buf[i*4 + j*2 + 0] &
+						    (1 << xbit))? fg_r : bg_r;
+						pixels[xbit * 3 + 1] =
+						    (dma_buf[i*4 + j*2 + 0] &
+						    (1 << xbit))? fg_g : bg_g;
+						pixels[xbit * 3 + 2] =
+						    (dma_buf[i*4 + j*2 + 0] &
+						    (1 << xbit))? fg_b : bg_b;
+						pixels[(xbit + 8) * 3 + 0] =
+						    (dma_buf[i*4 + j*2 + 1] &
+						    (1 << xbit))? fg_r : bg_r;
+						pixels[(xbit + 8) * 3 + 1] =
+						    (dma_buf[i*4 + j*2 + 1] &
+						    (1 << xbit))? fg_g : bg_g;
+						pixels[(xbit + 8) * 3 + 2] =
+						    (dma_buf[i*4 + j*2 + 1] &
+						    (1 << xbit))? fg_b : bg_b;
 					} else {
 						/*  8-bit:  */
-						pixels[xbit]     = (dma_buf[i*4 + j*2 + 0] & (1 << xbit))? (fgcolor & 255) : (bgcolor & 255);
-						pixels[xbit + 8] = (dma_buf[i*4 + j*2 + 1] & (1 << xbit))? (fgcolor & 255) : (bgcolor & 255);
+						pixels[xbit] = (dma_buf[i*4 +
+						    j*2 + 0] & (1 << xbit))?
+						    (fgcolor & 255) :
+						    (bgcolor & 255);
+						pixels[xbit + 8] = (dma_buf[i*4
+						    + j*2 + 1] & (1 << xbit))?
+						    (fgcolor & 255) :
+						    (bgcolor & 255);
 					}
 				}
 
-				memcpy(d->vfb_data->framebuffer + ((y+suby) * PX_XSIZE + x) * bytesperpixel,
+				memcpy(d->vfb_data->framebuffer + ((y+suby)
+				    * PX_XSIZE + x) * bytesperpixel,
 				    pixels, pixels_len * bytesperpixel);
 
 				if (y+suby < d->vfb_data->update_y1)
@@ -499,29 +572,34 @@ int dev_px_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	if (relative_addr < 0x0c0000) {
 		/*
 		 *  DMA poll:  a read from this address should start a DMA
-		 *  transfer, and return 1 in odata while the DMA is in progress (STAMP_BUSY),
-		 *  and then 0 (STAMP_OK) once we're done.
+		 *  transfer, and return 1 in odata while the DMA is in
+		 *  progress (STAMP_BUSY), and then 0 (STAMP_OK) once we're
+		 *  done.
 		 *
-		 *  According to NetBSD's pxreg.h, the following formula gets us from
-		 *  system address to DMA address:  (v is the system address)
+		 *  According to NetBSD's pxreg.h, the following formula gets
+		 *  us from system address to DMA address:  (v is the system
+		 *  address)
 		 *
-		 *	dma_addr = ( ( ((v & ~0x7fff) << 3) | (v & 0x7fff) ) & 0x1ffff800) >> 9;
+		 *	dma_addr = ( ( ((v & ~0x7fff) << 3) |
+		 *		(v & 0x7fff) ) & 0x1ffff800) >> 9;
 		 *
 		 *  Hopefully, this is a good enough reversal of that formula:
 		 *
 		 *	sys_addr = ((dma_addr << 9) & 0x7800) +
 		 *		   ((dma_addr << 6) & 0xffff8000);
 		 *
-		 *  If the board type is "PX" then the system address is an address in host
-		 *  memory.  Otherwise, it is relative to 0x200000 (the i860's memory space
-		 *  on the board).
+		 *  If the board type is "PX" then the system address is an
+		 *  address in host memory.  Otherwise, it is relative to
+		 *  0x200000 (the i860's memory space on the board).
 		 */
 		uint32_t sys_addr;	/*  system address for DMA transfers  */
-		sys_addr = ((relative_addr << 9) & 0x7800) + ((relative_addr << 6) & 0xffff8000);
+		sys_addr = ((relative_addr << 9) & 0x7800) +
+		    ((relative_addr << 6) & 0xffff8000);
 
 		/*
-		 *  If the system address is sane enough, then start a DMA transfer:
-		 *  (for the "PX" board type, don't allow obviously too-low physical addresses)
+		 *  If the system address is sane enough, then start a DMA
+		 *  transfer:  (for the "PX" board type, don't allow obviously
+		 *  too-low physical addresses)
 		 */
 		if (sys_addr >= 0x4000 || d->type != DEV_PX_TYPE_PX)
 			dev_px_dma(cpu, sys_addr, d);
@@ -533,20 +611,22 @@ int dev_px_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	/*  N10 sram:  */
 	if (relative_addr >= 0x200000 && relative_addr < 0x280000) {
 		if (d->type == DEV_PX_TYPE_PX)
-			fatal("WARNING: the vdac should be at this address. overlap problems?\n");
+			fatal("WARNING: the vdac should be at this "
+			    "address. overlap problems?\n");
 
 		if (writeflag == MEM_WRITE) {
 			for (i=0; i<len; i++)
 				d->sram[relative_addr - 0x200000 + i] = data[i];
-			/*  NOTE:  this return here supresses debug output (which
-				would be printed if we continue)  */
+			/*  NOTE:  this return here supresses debug output
+			    (which would be printed if we continue)  */
 			return 1;
 		} else {
 			/*
-			 *
+			 *  Huh? Why have I commented out this? TODO
 			 */
 			/*  for (i=0; i<len; i++)
-				data[i] = d->sram[relative_addr - 0x200000 + i];  */
+				data[i] = d->sram[relative_addr - 0x200000
+			    + i];  */
 			odata = 1;
 		}
 	}
@@ -556,37 +636,47 @@ int dev_px_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 	switch (relative_addr) {
 	case 0x180008:		/*  hsync  */
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from hsync: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from hsync: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to hsync: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to hsync: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x18000c:		/*  hsync2  */
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from hsync2: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from hsync2: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to hsync2: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to hsync2: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x180010:		/*  hblank  */
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from hblank: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from hblank: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to hblank: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to hblank: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x180014:		/*  vsync  */
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from vsync: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from vsync: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to vsync: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to vsync: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x180018:		/*  vblank  */
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from vblank: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from vblank: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to vblank: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to vblank: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x180020:		/*  ipdvint  */
@@ -596,7 +686,8 @@ int dev_px_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr,
 /*  TODO:  how do interrupts work on the pixelstamp boards?  */
 odata = random();
 
-			debug("[ px: read from ipdvint: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from ipdvint: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
 			d->intr = idata;
 			if (idata & STIC_INT_E_WE)
@@ -605,36 +696,46 @@ odata = random();
 				d->intr &= ~STIC_INT_V;
 			if (idata & STIC_INT_P_WE)
 				d->intr &= ~STIC_INT_P;
-			debug("[ px: write to ipdvint: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to ipdvint: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x180028:		/*  sticsr  */
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from sticsr: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from sticsr: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to sticsr: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to sticsr: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x180038:		/*  buscsr  */
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from buscsr: 0x%08llx ]\n", (long long)odata);
+			debug("[ px: read from buscsr: 0x%08llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to buscsr: 0x%08llx ]\n", (long long)idata);
+			debug("[ px: write to buscsr: 0x%08llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	case 0x18003c:		/*  modcl  */
 		if (writeflag==MEM_READ) {
-			odata = (d->type << 12) + (d->xconfig << 11) + (d->yconfig << 9);
-			debug("[ px: read from modcl: 0x%llx ]\n", (long long)odata);
+			odata = (d->type << 12) + (d->xconfig << 11) +
+			    (d->yconfig << 9);
+			debug("[ px: read from modcl: 0x%llx ]\n",
+			    (long long)odata);
 		} else {
-			debug("[ px: write to modcl: 0x%llx ]\n", (long long)idata);
+			debug("[ px: write to modcl: 0x%llx ]\n",
+			    (long long)idata);
 		}
 		break;
 	default:
 		if (writeflag==MEM_READ) {
-			debug("[ px: read from addr 0x%x: 0x%llx ]\n", (int)relative_addr, (long long)odata);
+			debug("[ px: read from addr 0x%x: 0x%llx ]\n",
+			    (int)relative_addr, (long long)odata);
 		} else {
-			debug("[ px: write to addr 0x%x: 0x%llx ]\n", (int)relative_addr, (long long)idata);
+			debug("[ px: write to addr 0x%x: 0x%llx ]\n",
+			    (int)relative_addr, (long long)idata);
 		}
 	}
 
@@ -648,8 +749,8 @@ odata = random();
 /*
  *  dev_px_init():
  */
-void dev_px_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	int px_type, int irq_nr)
+void dev_px_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, int px_type, int irq_nr)
 {
 	struct px_data *d;
 
@@ -694,7 +795,7 @@ void dev_px_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 		exit(1);
 	}
 
-	d->vfb_data = dev_fb_init(cpu, d->fb_mem, 0, VFB_GENERIC,
+	d->vfb_data = dev_fb_init(machine, d->fb_mem, 0, VFB_GENERIC,
 	    PX_XSIZE, PX_YSIZE, PX_XSIZE, PX_YSIZE, d->bitdepth, d->px_name, 1);
 	if (d->vfb_data == NULL) {
 		fprintf(stderr, "dev_px_init(): out of memory (2)\n");
@@ -703,13 +804,13 @@ void dev_px_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 
 	switch (d->type) {
 	case DEV_PX_TYPE_PX:
-		dev_bt459_init(cpu, mem, baseaddr + 0x200000, 0,
+		dev_bt459_init(machine, mem, baseaddr + 0x200000, 0,
 		    d->vfb_data, 8, irq_nr, BT459_PX);
 		break;
 	case DEV_PX_TYPE_PXG:
 	case DEV_PX_TYPE_PXGPLUS:
 	case DEV_PX_TYPE_PXGPLUSTURBO:
-		dev_bt459_init(cpu, mem, baseaddr + 0x300000, 0,
+		dev_bt459_init(machine, mem, baseaddr + 0x300000, 0,
 		    d->vfb_data, d->bitdepth, irq_nr, BT459_PX);
 		break;
 	default:
@@ -718,6 +819,6 @@ void dev_px_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 
 	memory_device_register(mem, "px", baseaddr, DEV_PX_LENGTH,
 	    dev_px_access, d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_px_tick, d, 14);
+	machine_add_tickfunction(machine, dev_px_tick, d, 14);
 }
 
