@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_rd94.c,v 1.16 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_rd94.c,v 1.22 2005/02/02 09:54:29 debug Exp $
  *  
  *  Used by NEC-RD94, -R94, and -R96.
  */
@@ -36,9 +36,13 @@
 
 #include "bus_pci.h"
 #include "cop0.h"
+#include "cpu.h"
+#include "cpu_mips.h"
 #include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
+
 #include "rd94.h"
 
 
@@ -87,6 +91,16 @@ int dev_rd94_access(struct cpu *cpu, struct memory *mem,
 	regnr = relative_addr / sizeof(uint32_t);
 
 	switch (relative_addr) {
+	case RD94_SYS_CONFIG:
+		if (writeflag == MEM_WRITE) {
+			fatal("[ rd94: write to CONFIG: 0x%llx ]\n",
+			    (long long)idata);
+		} else {
+			odata = 0;
+			fatal("[ rd94: read from CONFIG: 0x%llx ]\n",
+			    (long long)odata);
+		}
+		break;
 	case RD94_SYS_INTSTAT1:		/*  LB (Local Bus ???)  */
 		if (writeflag == MEM_WRITE) {
 		} else {
@@ -94,7 +108,7 @@ int dev_rd94_access(struct cpu *cpu, struct memory *mem,
 			odata = (8+1) << 2;
 
 			/*  Ugly hack:  */
-			if ((cpu->coproc[0]->reg[COP0_CAUSE] & 0x800) == 0)
+			if ((cpu->cd.mips.coproc[0]->reg[COP0_CAUSE] & 0x800) == 0)
 				odata = 0;
 		}
 		debug("[ rd94: intstat1 ]\n");
@@ -177,7 +191,7 @@ int dev_rd94_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_rd94_init():
  */
-struct pci_data *dev_rd94_init(struct cpu *cpu, struct memory *mem,
+struct pci_data *dev_rd94_init(struct machine *machine, struct memory *mem,
 	uint64_t baseaddr, int pciirq)
 {
 	struct rd94_data *d = malloc(sizeof(struct rd94_data));
@@ -191,7 +205,7 @@ struct pci_data *dev_rd94_init(struct cpu *cpu, struct memory *mem,
 
 	memory_device_register(mem, "rd94", baseaddr, DEV_RD94_LENGTH,
 	    dev_rd94_access, (void *)d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_rd94_tick, d, RD94_TICK_SHIFT);
+	machine_add_tickfunction(machine, dev_rd94_tick, d, RD94_TICK_SHIFT);
 
 	return d->pci_data;
 }
