@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_scc.c,v 1.17 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_scc.c,v 1.25 2005/02/18 07:29:56 debug Exp $
  *  
  *  Serial controller on some DECsystems and SGI machines. (Z8530 ?)
  *  Most of the code in here is written for DECsystem emulation, though.
@@ -46,10 +46,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "console.h"
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "console.h"
-#include "devices.h"
 
 #include "sccreg.h"
 
@@ -63,6 +65,8 @@
 struct scc_data {
 	int		irq_nr;
 	int		use_fb;
+	int		console_handle;
+
 	int		scc_nr;
 	int		addrmul;
 
@@ -100,26 +104,30 @@ void dev_scc_add_to_rx_queue(void *e, int ch, int portnr)
 
 	portnr &= (N_SCC_PORTS - 1);
 
-        d->rx_queue_char[portnr * MAX_QUEUE_LEN + d->cur_rx_queue_pos_write[portnr]] = ch; 
+        d->rx_queue_char[portnr * MAX_QUEUE_LEN +
+	    d->cur_rx_queue_pos_write[portnr]] = ch; 
         d->cur_rx_queue_pos_write[portnr] ++;
         if (d->cur_rx_queue_pos_write[portnr] == MAX_QUEUE_LEN)
                 d->cur_rx_queue_pos_write[portnr] = 0;
 
-        if (d->cur_rx_queue_pos_write[portnr] == d->cur_rx_queue_pos_read[portnr])
+        if (d->cur_rx_queue_pos_write[portnr] ==
+	    d->cur_rx_queue_pos_read[portnr])
                 fatal("warning: add_to_rx_queue(): rx_queue overrun!\n");
 }
 
 
-int rx_avail(struct scc_data *d, int portnr)
+static int rx_avail(struct scc_data *d, int portnr)
 {
-	return d->cur_rx_queue_pos_write[portnr] != d->cur_rx_queue_pos_read[portnr];
+	return d->cur_rx_queue_pos_write[portnr] !=
+	    d->cur_rx_queue_pos_read[portnr];
 }
 
 
-unsigned char rx_nextchar(struct scc_data *d, int portnr)
+static unsigned char rx_nextchar(struct scc_data *d, int portnr)
 {
 	unsigned char ch;
-	ch = d->rx_queue_char[portnr * MAX_QUEUE_LEN + d->cur_rx_queue_pos_read[portnr]];
+	ch = d->rx_queue_char[portnr * MAX_QUEUE_LEN +
+	    d->cur_rx_queue_pos_read[portnr]];
 	d->cur_rx_queue_pos_read[portnr]++;
 	if (d->cur_rx_queue_pos_read[portnr] == MAX_QUEUE_LEN)
 		d->cur_rx_queue_pos_read[portnr] = 0;
@@ -137,25 +145,30 @@ void dev_scc_tick(struct cpu *cpu, void *extra)
 
 	/*  Add keystrokes to the rx queue:  */
 	if (d->use_fb == 0 && d->scc_nr == 1) {
-		if (console_charavail())
-			dev_scc_add_to_rx_queue(extra, console_readchar(), 2);
+		if (console_charavail(d->console_handle))
+			dev_scc_add_to_rx_queue(extra, console_readchar(
+			    d->console_handle), 2);
 	}
 	if (d->use_fb == 1 && d->scc_nr == 1)
 		lk201_tick(&d->lk201);
 
 	for (i=0; i<N_SCC_PORTS; i++) {
 		d->scc_register_r[i * N_SCC_REGS + SCC_RR0] |= SCC_RR0_TX_EMPTY;
-		d->scc_register_r[i * N_SCC_REGS + SCC_RR1] = 0;		/*  No receive errors  */
+		d->scc_register_r[i * N_SCC_REGS + SCC_RR1] = 0;
+		    /*  No receive errors  */
 
-		d->scc_register_r[i * N_SCC_REGS + SCC_RR0] &= ~SCC_RR0_RX_AVAIL;
+		d->scc_register_r[i * N_SCC_REGS + SCC_RR0] &=
+		    ~SCC_RR0_RX_AVAIL;
 		if (rx_avail(d, i))
-			d->scc_register_r[i * N_SCC_REGS + SCC_RR0] |= SCC_RR0_RX_AVAIL;
+			d->scc_register_r[i * N_SCC_REGS + SCC_RR0] |=
+			    SCC_RR0_RX_AVAIL;
 
 		/*
 		 *  Interrupts:
 		 *  (NOTE: Interrupt enables are always at channel A)
 		 */
-		if (d->scc_register_w[N_SCC_REGS + SCC_WR9] & SCC_WR9_MASTER_IE) {
+		if (d->scc_register_w[N_SCC_REGS + SCC_WR9] &
+		    SCC_WR9_MASTER_IE) {
 			/*  TX interrupts?  */
 			if (d->scc_register_w[i * N_SCC_REGS + SCC_WR1] & SCC_WR1_TX_IE) {
 				if (d->scc_register_r[i * N_SCC_REGS + SCC_RR3] & SCC_RR3_TX_IP_A ||
@@ -201,9 +214,11 @@ cpu_interrupt(cpu, 8 + 0x02000000);
 /*
  *  dev_scc_dma_func():
  */
-int dev_scc_dma_func(struct cpu *cpu, void *extra, uint64_t addr, size_t dma_len, int tx)
+int dev_scc_dma_func(struct cpu *cpu, void *extra, uint64_t addr,
+	size_t dma_len, int tx)
 {
-	/*  printf("dev_scc_dma_func(): addr = %08x, len = %i\n", (int)addr, (int)dma_len);  */
+	/*  printf("dev_scc_dma_func(): addr = %08x, len = %i\n",
+	    (int)addr, (int)dma_len);  */
 	unsigned char word[4];
 	struct scc_data *d = (struct scc_data *) extra;
 	int n;
@@ -212,12 +227,15 @@ int dev_scc_dma_func(struct cpu *cpu, void *extra, uint64_t addr, size_t dma_len
 
 	if (tx) {
 		do {
-			memory_rw(cpu, cpu->mem, addr, &word[0], sizeof(word), MEM_READ, NO_EXCEPTIONS | PHYSICAL);
+			cpu->memory_rw(cpu, cpu->mem, addr, &word[0],
+			    sizeof(word), MEM_READ, NO_EXCEPTIONS | PHYSICAL);
 
 			lk201_tx_data(&d->lk201, d->scc_nr * 2 + port, word[1]);
 			/*  Loopback:  */
-			if (d->scc_register_w[port * N_SCC_REGS + SCC_WR14] & SCC_WR14_LOCAL_LOOPB)
-				dev_scc_add_to_rx_queue(d, word[1], d->scc_nr * 2 + port);
+			if (d->scc_register_w[port * N_SCC_REGS + SCC_WR14]
+			    & SCC_WR14_LOCAL_LOOPB)
+				dev_scc_add_to_rx_queue(d, word[1],
+				    d->scc_nr * 2 + port);
 
 			addr += sizeof(word);
 		} while ((addr & 0xffc) != 0);
@@ -225,7 +243,8 @@ int dev_scc_dma_func(struct cpu *cpu, void *extra, uint64_t addr, size_t dma_len
 		dev_scc_tick(cpu, extra);
 		return 1;
 	} else {
-		printf("dev_scc_dma_func(): addr = %08x, len = %i\n", (int)addr, (int)dma_len);
+		printf("dev_scc_dma_func(): addr = %08x, len = %i\n",
+		    (int)addr, (int)dma_len);
 
 
 /*  TODO: all this is just nonsense  */
@@ -233,9 +252,11 @@ int dev_scc_dma_func(struct cpu *cpu, void *extra, uint64_t addr, size_t dma_len
 		n = 0;
 		while (rx_avail(d, port)) {
 			word[0] = word[1] = word[2] = word[3] = 0;
-			word[0] = word[1] = word[2] = word[3] = rx_nextchar(d, port);
+			word[0] = word[1] = word[2] = word[3] =
+			    rx_nextchar(d, port);
 			n++;
-			memory_rw(cpu, cpu->mem, addr, &word[0], sizeof(word), MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
+			cpu->memory_rw(cpu, cpu->mem, addr, &word[0],
+			    sizeof(word), MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
 
 			addr += sizeof(word);
 			/*  Half-page?  */
@@ -262,7 +283,8 @@ int dev_scc_access(struct cpu *cpu, struct memory *mem,
 
 	idata = memory_readmax64(cpu, data, len);
 
-	/*  relative_addr /= d->addrmul;  */  /*  See SGI comment below instead.  */
+	/*  relative_addr /= d->addrmul;  */
+		/*  See SGI comment below instead.  */
 	/*
 	 *  SGI writes command to 0x0f, and data to 0x1f.
 	 *  (TODO: This works for port nr 0, how about port nr 1?)
@@ -386,8 +408,8 @@ int dev_scc_access(struct cpu *cpu, struct memory *mem,
  *	scc_nr = 0 or 1
  *	addmul = 1 in most cases, 8 on SGI?
  */
-void *dev_scc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	int irq_nr, int use_fb, int scc_nr, int addrmul)
+void *dev_scc_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, int irq_nr, int use_fb, int scc_nr, int addrmul)
 {
 	struct scc_data *d;
 
@@ -401,12 +423,14 @@ void *dev_scc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	d->scc_nr  = scc_nr;
 	d->use_fb  = use_fb;
 	d->addrmul = addrmul;
+	d->console_handle = console_start_slave(machine, "SCC");
 
-	lk201_init(&d->lk201, use_fb, dev_scc_add_to_rx_queue, d);
+	lk201_init(&d->lk201, use_fb, dev_scc_add_to_rx_queue,
+	    d->console_handle, d);
 
 	memory_device_register(mem, "scc", baseaddr, DEV_SCC_LENGTH,
 	    dev_scc_access, d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_scc_tick, d, 10);
+	machine_add_tickfunction(machine, dev_scc_tick, d, 10);
 
 	return (void *) d;
 }
