@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_gbe.c,v 1.16 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_sgi_gbe.c,v 1.21 2005/02/11 09:29:48 debug Exp $
  *
  *  SGI "gbe", graphics controller. Framebuffer.
  *  Loosely inspired by Linux code.
@@ -35,10 +35,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "console.h"
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "console.h"
-#include "devices.h"
 
 
 /*  Let's hope nothing is there already...  */
@@ -98,14 +100,17 @@ void dev_sgi_gbe_tick(struct cpu *cpu, void *extra)
 
 	while (on_screen) {
 		/*  Get pointer to a tile:  */
-		memory_rw(cpu, cpu->mem, tiletable + sizeof(tileptr_buf) * tile_nr,
-		    tileptr_buf, sizeof(tileptr_buf), MEM_READ, NO_EXCEPTIONS | PHYSICAL);
-		tileptr = 256 * tileptr_buf[0] + tileptr_buf[1];	/*  TODO: endianness  */
+		cpu->memory_rw(cpu, cpu->mem, tiletable +
+		    sizeof(tileptr_buf) * tile_nr,
+		    tileptr_buf, sizeof(tileptr_buf), MEM_READ,
+		    NO_EXCEPTIONS | PHYSICAL);
+		tileptr = 256 * tileptr_buf[0] + tileptr_buf[1];
+		/*  TODO: endianness  */
 		tileptr <<= 16;
 
 		/*  tileptr is now a physical address of a tile.  */
-		debug("[ sgi_gbe:   tile_nr = %2i, tileptr = 0x%08lx, xbase = %4i, ybase = %4i ]\n",
-		    tile_nr, tileptr, xbase, ybase);
+		debug("[ sgi_gbe:   tile_nr = %2i, tileptr = 0x%08lx, xbase"
+		    " = %4i, ybase = %4i ]\n", tile_nr, tileptr, xbase, ybase);
 
 		if (tweaked) {
 			/*  Tweaked (linear) mode:  */
@@ -123,10 +128,14 @@ void dev_sgi_gbe_tick(struct cpu *cpu, void *extra)
 					on_screen = 0;
 				}
 
-				/*  debug("old_fb_offset = %08x copylen=%i\n", old_fb_offset, copy_len);  */
+				/*  debug("old_fb_offset = %08x copylen"
+				    "=%i\n", old_fb_offset, copy_len);  */
 
-				memory_rw(cpu, cpu->mem, tileptr + copy_offset, buf, copy_len, MEM_READ, NO_EXCEPTIONS | PHYSICAL);
-				dev_fb_access(cpu, cpu->mem, old_fb_offset, buf, copy_len, MEM_WRITE, d->fb_data);
+				cpu->memory_rw(cpu, cpu->mem, tileptr +
+				    copy_offset, buf, copy_len, MEM_READ,
+				    NO_EXCEPTIONS | PHYSICAL);
+				dev_fb_access(cpu, cpu->mem, old_fb_offset,
+				    buf, copy_len, MEM_WRITE, d->fb_data);
 				copy_offset += sizeof(buf);
 				old_fb_offset += sizeof(buf);
 			}
@@ -143,11 +152,14 @@ void dev_sgi_gbe_tick(struct cpu *cpu, void *extra)
 				pixels_per_line = d->xres - xbase;
 
 			for (y=0; y<lines_to_copy; y++) {
-				memory_rw(cpu, cpu->mem, tileptr + 512 * y,
-				    buf, pixels_per_line * d->bitdepth / 8, MEM_READ, NO_EXCEPTIONS | PHYSICAL);
+				cpu->memory_rw(cpu, cpu->mem, tileptr + 512 * y,
+				    buf, pixels_per_line * d->bitdepth / 8,
+				    MEM_READ, NO_EXCEPTIONS | PHYSICAL);
 
-				dev_fb_access(cpu, cpu->mem, ((ybase + y) * d->xres + xbase) * d->bitdepth / 8,
-				    buf, pixels_per_line * d->bitdepth / 8, MEM_WRITE, d->fb_data);
+				dev_fb_access(cpu, cpu->mem, ((ybase + y) *
+				    d->xres + xbase) * d->bitdepth / 8,
+				    buf, pixels_per_line * d->bitdepth / 8,
+				    MEM_WRITE, d->fb_data);
 			}
 
 			/*  Go to next tile:  */
@@ -264,7 +276,7 @@ odata = random();	/*  testhack for the ip32 prom  */
 			d->bitdepth = 8 << ((d->plane0ctrl >> 13) & 3);
 			debug("[ sgi_gbe: setting color depth to %i bits ]\n", d->bitdepth);
 			if (d->bitdepth != 8)
-				fatal("sgi_gbe: warning: bitdepth %i not really implemented yet\n");
+				fatal("sgi_gbe: warning: bitdepth %i not really implemented yet\n", d->bitdepth);
 		} else
 			odata = d->plane0ctrl;
 		break;
@@ -361,7 +373,8 @@ odata = random();	/*  testhack for the ip32 prom  */
 /*
  *  dev_sgi_gbe_init():
  */
-void dev_sgi_gbe_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr)
+void dev_sgi_gbe_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr)
 {
 	struct sgi_gbe_data *d;
 
@@ -375,12 +388,12 @@ void dev_sgi_gbe_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr)
 	d->yres = GBE_DEFAULT_YRES;
 	d->bitdepth = 8;
 	d->control = 0x20aa000;		/*  or 0x00000001?  */
-	d->fb_data = dev_fb_init(cpu, mem, FAKE_GBE_FB_ADDRESS,
+	d->fb_data = dev_fb_init(machine, mem, FAKE_GBE_FB_ADDRESS,
 	    VFB_GENERIC, d->xres, d->yres, d->xres, d->yres, 8, "SGI GBE", 0);
 	set_grayscale_palette(d->fb_data, 256);
 
 	memory_device_register(mem, "sgi_gbe", baseaddr, DEV_SGI_GBE_LENGTH,
 	    dev_sgi_gbe_access, d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_sgi_gbe_tick, d, 18);
+	machine_add_tickfunction(machine, dev_sgi_gbe_tick, d, 18);
 }
 
