@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_ip32.c,v 1.4 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_sgi_ip32.c,v 1.23 2005/02/18 06:51:55 debug Exp $
  *  
  *  SGI IP32 devices.
  *
@@ -43,7 +43,10 @@
 
 #include "bus_pci.h"
 #include "console.h"
+#include "cpu.h"
 #include "devices.h"
+#include "emul.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
 #include "net.h"
@@ -78,7 +81,7 @@ struct macepci_data {
  */
 void dev_crime_tick(struct cpu *cpu, void *extra)
 {
-	int j, carry, old, new;
+	int j, carry, old, new, add_byte;
 	uint64_t what_to_add = (1<<CRIME_TICKSHIFT)
 	    * CRIME_SPEED_DIV_FACTOR / CRIME_SPEED_MUL_FACTOR;
 	struct crime_data *d = extra;
@@ -87,7 +90,9 @@ void dev_crime_tick(struct cpu *cpu, void *extra)
 	carry = 0;
 	while (j < 8) {
 		old = d->reg[CRIME_TIME + 7 - j];
-		new = old + ((what_to_add >> (j * 8)) & 255) + carry;
+		add_byte = what_to_add >> ((int64_t)j * 8);
+		add_byte &= 255;
+		new = old + add_byte + carry;
 		d->reg[CRIME_TIME + 7 - j] = new & 255;
 		if (new >= 256)
 			carry = 1;
@@ -107,6 +112,9 @@ int dev_crime_access(struct cpu *cpu, struct memory *mem,
 {
 	int i;
 	struct crime_data *d = extra;
+	uint64_t idata;
+
+	idata = memory_readmax64(cpu, data, len);
 
 	/*
 	 *  Set crime version/revision:
@@ -120,15 +128,26 @@ int dev_crime_access(struct cpu *cpu, struct memory *mem,
 	 *
 	 *  (TODO?)
 	 */
-	d->reg[4] = 0x00; d->reg[5] = 0x00; d->reg[6] = 0x00; d->reg[7] = d->use_fb? 0xa1 : 0x11;
+	d->reg[4] = 0x00; d->reg[5] = 0x00; d->reg[6] = 0x00;
+	d->reg[7] = d->use_fb? 0xa1 : 0x11;
 
-	/*  Amount of memory.  Bit 8 of bank control set ==> 128MB instead of 32MB per bank (?)  */
-	/*  When the bank control registers contain the same value as the previous one, that
-		bank is not valid. (?)  */
-	d->reg[CRM_MEM_BANK_CTRL0 + 6] = 0x0;	/*  lowest bit set = 128MB, clear = 32MB  */
-	d->reg[CRM_MEM_BANK_CTRL0 + 7] = 0x0;	/*  address * 32MB  */
-	d->reg[CRM_MEM_BANK_CTRL1 + 6] = 0x0;	/*  lowest bit set = 128MB, clear = 32MB  */
-	d->reg[CRM_MEM_BANK_CTRL1 + 7] = 0x1;	/*  address * 32MB  */
+	/*
+	 *  Amount of memory.  Bit 8 of bank control set ==> 128MB instead
+	 *  of 32MB per bank (?)
+	 *
+	 *  When the bank control registers contain the same value as the
+	 *  previous one, that bank is not valid. (?)
+	 */
+	d->reg[CRM_MEM_BANK_CTRL0 + 6] = 0;  /* lowbit set=128MB, clear=32MB */
+	d->reg[CRM_MEM_BANK_CTRL0 + 7] = 0;  /* address * 32MB  */
+	d->reg[CRM_MEM_BANK_CTRL1 + 6] = 0;  /* lowbit set=128MB, clear=32MB */
+	d->reg[CRM_MEM_BANK_CTRL1 + 7] = 1;  /* address * 32MB  */
+
+	if (relative_addr >= CRIME_TIME && relative_addr < CRIME_TIME+8) {
+		if (writeflag == MEM_READ)
+			memcpy(data, &d->reg[relative_addr], len);
+		return 1;
+	}
 
 	if (writeflag == MEM_WRITE)
 		memcpy(&d->reg[relative_addr], data, len);
@@ -137,39 +156,61 @@ int dev_crime_access(struct cpu *cpu, struct memory *mem,
 
 	if (relative_addr == 0x18 || relative_addr == 0x1c) {
 		/*
-		 *  NOTE:  This is super-ugly. I want to force the MIPS interrupt
-		 *  assertion bits to be updated, taking the interrupt mask
-		 *  into account, so I call cpu_interrupt_ack() with something
-		 *  which "hopefully" doesn't do any damage. cpu_interrupt_ack()
-		 *  will in turn call the SGI crime interrupt stuff in src/machine.c,
-		 *  which takes the interrupt mask into account when asserting or
-		 *  deasserting the crime interrupt pin.
+		 *  Force interrupt re-assertion:
 		 *
-		 *  TODO: How to solve this nicer?
+		 *  NOTE: Ugly hack. Hopefully CRMERR is never used.
 		 */
-		cpu_interrupt_ack(cpu, 8);
+#if 0
+
+No. If this is enabled, the mec bugs out on either NetBSD or OpenBSD.
+TODO.
+
+		cpu_interrupt_ack(cpu, 8); /* CRM_INT_CRMERR); */
+#endif
 	}
 
 	switch (relative_addr) {
+	case CRIME_CONTROL:	/*  0x008  */
+		/*  TODO: 64-bit write to CRIME_CONTROL, but some things
+		    (such as NetBSD 1.6.2) write to 0x00c!  */
+		if (writeflag == MEM_WRITE) {
+			/*
+			 *  0x200 = watchdog timer (according to NetBSD)
+			 *  0x800 = "reboot" used by the IP32 PROM
+			 */
+			if (idata & 0x200) {
+				idata &= ~0x200;
+			}
+			if (idata & 0x800) {
+				/*  This is used by the IP32 PROM's
+				    "reboot" command:  */
+				for (i=0; i<cpu->machine->ncpus; i++)
+					cpu->machine->cpus[i]->running = 0;
+				cpu->machine->
+				    exit_without_entering_debugger = 1;
+				idata &= ~0x800;
+			}
+			if (idata != 0)
+				fatal("[ CRIME_CONTROL: unimplemented "
+				    "control 0x%016llx ]\n", (long long)idata);
+		}
+		break;
 #if 1
-	case 0x10:	/*  Current interrupt status  */
+	case CRIME_INTSTAT:	/*  0x010, Current interrupt status  */
 	case 0x14:
-	case 0x18:	/*  Current interrupt mask  */
+	case CRIME_INTMASK:	/*  0x018,  Current interrupt mask  */
 	case 0x1c:
 	case 0x34:
-#endif
-#if 1
-	case CRIME_TIME:
-	case CRIME_TIME+4:
 #endif
 		/*  don't dump debug info for these  */
 		break;
 	default:
 		if (writeflag==MEM_READ) {
-			debug("[ crime: read from 0x%x, len=%i:", (int)relative_addr, len);
+			debug("[ crime: read from 0x%x, len=%i:",
+			    (int)relative_addr, len);
 			for (i=0; i<len; i++)
 				debug(" %02x", data[i]);
-			debug(" ]\n", len);
+			debug(" ]\n");
 		} else {
 			debug("[ crime: write to 0x%x:", (int)relative_addr);
 			for (i=0; i<len; i++)
@@ -185,7 +226,7 @@ int dev_crime_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_crime_init():
  */
-struct crime_data *dev_crime_init(struct cpu *cpu, struct memory *mem,
+struct crime_data *dev_crime_init(struct machine *machine, struct memory *mem,
 	uint64_t baseaddr, int irq_nr, int use_fb)
 {
 	struct crime_data *d;
@@ -201,7 +242,7 @@ struct crime_data *dev_crime_init(struct cpu *cpu, struct memory *mem,
 
 	memory_device_register(mem, "crime", baseaddr, DEV_CRIME_LENGTH,
 	    dev_crime_access, d, MEM_DEFAULT, NULL);
-	cpu_add_tickfunction(cpu, dev_crime_tick, d, CRIME_TICKSHIFT);
+	machine_add_tickfunction(machine, dev_crime_tick, d, CRIME_TICKSHIFT);
 
 	return d;
 }
@@ -235,7 +276,8 @@ int dev_mace_access(struct cpu *cpu, struct memory *mem,
 #endif
 	default:
 		if (writeflag==MEM_READ) {
-			debug("[ mace: read from 0x%x, len=%i ]\n", (int)relative_addr, len);
+			debug("[ mace: read from 0x%x, len=%i ]\n",
+			    (int)relative_addr, len);
 		} else {
 			debug("[ mace: write to 0x%x:", (int)relative_addr);
 			for (i=0; i<len; i++)
@@ -311,17 +353,22 @@ int dev_macepci_access(struct cpu *cpu, struct memory *mem,
 	case 0xcf8:	/*  PCI ADDR  */
 	case 0xcfc:	/*  PCI DATA  */
 		if (writeflag == MEM_WRITE) {
-			res = bus_pci_access(cpu, mem, relative_addr, &idata, writeflag, d->pci_data);
+			res = bus_pci_access(cpu, mem, relative_addr,
+			    &idata, writeflag, d->pci_data);
 		} else {
-			res = bus_pci_access(cpu, mem, relative_addr, &odata, writeflag, d->pci_data);
+			res = bus_pci_access(cpu, mem, relative_addr,
+			    &odata, writeflag, d->pci_data);
 			/*  odata = 0;  */
 		}
 		break;
 	default:
 		if (writeflag == MEM_WRITE) {
-			debug("[ macepci: unimplemented write to address 0x%x, data=0x%02x ]\n", relative_addr, idata);
+			debug("[ macepci: unimplemented write to address "
+			    "0x%x, data=0x%02x ]\n",
+			    (int)relative_addr, (int)idata);
 		} else {
-			debug("[ macepci: unimplemented read from address 0x%x ]\n", relative_addr);
+			debug("[ macepci: unimplemented read from address "
+			    "0x%x ]\n", (int)relative_addr);
 		}
 	}
 
@@ -366,17 +413,16 @@ struct pci_data *dev_macepci_init(struct memory *mem, uint64_t baseaddr,
  *	x)  tx and rx interrupts/ring/slot stuff
  */
 
-/*  #define debug fatal  */
-
 #define	MEC_TICK_SHIFT		14
 
 #define	MAX_TX_PACKET_LEN	1700
-#define	N_RX_ADDRESSES		32
+#define	N_RX_ADDRESSES		16
 
 struct sgi_mec_data {
 	uint64_t	reg[DEV_SGI_MEC_LENGTH / sizeof(uint64_t)];
 
 	int		irq_nr;
+	unsigned char	macaddr[6];
 
 	unsigned char	cur_tx_packet[MAX_TX_PACKET_LEN];
 	int		cur_tx_packet_len;
@@ -395,6 +441,9 @@ struct sgi_mec_data {
  */
 static void mec_reset(struct sgi_mec_data *d)
 {
+	if (d->cur_rx_packet != NULL)
+		free(d->cur_rx_packet);
+
 	memset(d->reg, 0, sizeof(d->reg));
 }
 
@@ -415,24 +464,27 @@ static void mec_control_write(struct cpu *cpu, struct sgi_mec_data *d,
 /*
  *  mec_try_rx():
  */
-static void mec_try_rx(struct cpu *cpu, struct sgi_mec_data *d)
+static int mec_try_rx(struct cpu *cpu, struct sgi_mec_data *d)
 {
 	uint64_t base;
 	unsigned char data[8];
-	int i, res;
+	int i, res, retval = 0;
 
 	base = d->rx_addr[d->cur_rx_addr_index];
+	if (base & 0xfff)
+		fatal("[ mec_try_rx(): WARNING! lowest bits of base are "
+		    "non-zero (0x%3x). TODO ]\n", (int)(base & 0xfff));
 	base &= 0xfffff000ULL;
 	if (base == 0)
-		goto skip_but_goto_next;
+		goto skip;
 
 	/*  printf("rx base = 0x%016llx\n", (long long)base);  */
 
 	/*  Read an rx descriptor from memory:  */
-	res = memory_rw(cpu, cpu->mem, base,
+	res = cpu->memory_rw(cpu, cpu->mem, base,
 	    &data[0], sizeof(data), MEM_READ, PHYSICAL);
 	if (!res)
-		return;
+		return 0;
 
 #if 0
 	printf("{ mec: rxdesc %i: ", d->cur_rx_addr_index);
@@ -446,12 +498,14 @@ static void mec_try_rx(struct cpu *cpu, struct sgi_mec_data *d)
 
 	/*  Is this descriptor already in use?  */
 	if (data[0] & 0x80) {
-printf("INTERRUPT for base = 0x%x\n", (int)base);
-		goto skip_but_interrupt;
+		/*  printf("INTERRUPT for base = 0x%x\n", (int)base);  */
+		goto skip_and_advance;
 	}
 
-	if (d->cur_rx_packet == NULL && net_ethernet_rx_avail(d))
-		net_ethernet_rx(d, &d->cur_rx_packet, &d->cur_rx_packet_len);
+	if (d->cur_rx_packet == NULL &&
+	    net_ethernet_rx_avail(cpu->machine->emul->net, d))
+		net_ethernet_rx(cpu->machine->emul->net, d,
+		    &d->cur_rx_packet, &d->cur_rx_packet_len);
 
 	if (d->cur_rx_packet == NULL)
 		goto skip;
@@ -459,13 +513,16 @@ printf("INTERRUPT for base = 0x%x\n", (int)base);
 	/*  Copy the packet data:  */
 	/*  printf("RX: ");  */
 	for (i=0; i<d->cur_rx_packet_len; i++) {
-		res = memory_rw(cpu, cpu->mem, base + 32 + i + 2,
+		res = cpu->memory_rw(cpu, cpu->mem, base + 32 + i + 2,
 		    d->cur_rx_packet + i, 1, MEM_WRITE, PHYSICAL);
 		/*  printf(" %02x", d->cur_rx_packet[i]);  */
 	}
 	/*  printf("\n");  */
 
-	printf("RX: %i bytes, base = 0x%x\n", d->cur_rx_packet_len, (int)base);
+#if 1
+	printf("RX: %i bytes, index %i, base = 0x%x\n",
+	    d->cur_rx_packet_len, d->cur_rx_addr_index, (int)base);
+#endif
 
 	/*  4 bytes of CRC at the end. Hm. TODO  */
 	d->cur_rx_packet_len += 4;
@@ -476,32 +533,30 @@ printf("INTERRUPT for base = 0x%x\n", (int)base);
 	/*  TODO: lots of bits :-)  */
 	data[4] = 0x04;		/*  match MAC  */
 	data[0] = 0x80;		/*  0x80 = received.  */
-	res = memory_rw(cpu, cpu->mem, base,
+	res = cpu->memory_rw(cpu, cpu->mem, base,
 	    &data[0], sizeof(data), MEM_WRITE, PHYSICAL);
 
 	/*  Free the packet from memory:  */
 	free(d->cur_rx_packet);
 	d->cur_rx_packet = NULL;
 
-skip_but_interrupt:
 	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] |= MEC_INT_RX_THRESHOLD;
-	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] &= ~MEC_INT_RX_MCL_FIFO_ALIAS;
-	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] |= ((d->cur_rx_addr_index + 1) & 0x1f) << 8;
-	cpu_interrupt(cpu, d->irq_nr);
-
-skip_but_goto_next:
+skip_and_advance:
 	d->cur_rx_addr_index ++;
 	d->cur_rx_addr_index %= N_RX_ADDRESSES;
+	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] &= ~MEC_INT_RX_MCL_FIFO_ALIAS;
+	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] |= (d->cur_rx_addr_index & 0x1f) << 8;
+	retval = 1;
 
 skip:
-	;
+	return retval;
 }
 
 
 /*
  *  mec_try_tx():
  */
-static void mec_try_tx(struct cpu *cpu, struct sgi_mec_data *d)
+static int mec_try_tx(struct cpu *cpu, struct sgi_mec_data *d)
 {
 	uint64_t base, addr, dma_base;
 	int tx_ring_ptr, ringread, ringwrite, res, i, j;
@@ -512,34 +567,45 @@ static void mec_try_tx(struct cpu *cpu, struct sgi_mec_data *d)
 	tx_ring_ptr = d->reg[MEC_TX_RING_PTR / sizeof(uint64_t)];
 
 	if (base == 0)
-		return;
+		return 0;
 
 	/*  printf("base = 0x%016llx\n", base);  */
 
+	ringread = tx_ring_ptr & MEC_TX_RING_READ_PTR;
+	ringwrite = tx_ring_ptr & MEC_TX_RING_WRITE_PTR;
+	ringread >>= 16;
+	/*  All done? Then abort.  */
+	if (ringread == ringwrite)
+		return 0;
+
 	tx_ring_ptr &= MEC_TX_RING_READ_PTR;
 	tx_ring_ptr >>= 16;
 
 	/*  Each tx descriptor (+ buffer) is 128 bytes:  */
 	addr = base + tx_ring_ptr*128;
-	res = memory_rw(cpu, cpu->mem, addr,
+	res = cpu->memory_rw(cpu, cpu->mem, addr,
 	    &data[0], sizeof(data), MEM_READ, PHYSICAL);
 	if (!res)
-		return;
+		return 0;
 
 	/*  Is this packet transmitted already?  */
-	if (data[0] & 0x80)
-		goto skip;
+	if (data[0] & 0x80) {
+		fatal("[ mec_try_tx: tx_ring_ptr = %i, already"
+		    " transmitted? ]\n", tx_ring_ptr);
+		goto advance_tx;
+	}
 
 	len = data[6] * 256 + data[7];
 	start_offset = data[5] & 0x7f;
 
-	/*  Is this packet empty? Then don't transmit it.  */
+	/*  Is this packet empty? Then don't transmit.  */
 	if (len == 0)
-		return;
+		return 0;
 
 	/*  Hm. Is len one too little?  TODO  */
 	len ++;
 
+#if 0
 	printf("{ mec: txdesc %i: ", tx_ring_ptr);
 	for (i=0; i<sizeof(data); i++) {
 		if ((i & 3) == 0)
@@ -547,7 +613,7 @@ static void mec_try_tx(struct cpu *cpu, struct sgi_mec_data *d)
 		printf("%02x", data[i]);
 	}
 	printf(" }\n");
-
+#endif
 	dma_ptr_nr = 0;
 
 	j = 0;
@@ -559,7 +625,7 @@ static void mec_try_tx(struct cpu *cpu, struct sgi_mec_data *d)
 		if ((i & 0x7f) == 0x00)
 			break;
 
-		res = memory_rw(cpu, cpu->mem, addr + i,
+		res = cpu->memory_rw(cpu, cpu->mem, addr + i,
 		    &ch, sizeof(ch), MEM_READ, PHYSICAL);
 		/*  printf(" %02x", ch);  */
 
@@ -587,17 +653,19 @@ static void mec_try_tx(struct cpu *cpu, struct sgi_mec_data *d)
 			dma_len = (data[dma_ptr_nr * 8 + 2] << 8)
 			        + (data[dma_ptr_nr * 8 + 3]) + 1;
 
-			/*  printf("dma_base = %08x, dma_len = %i\n", (int)dma_base, dma_len);  */
+			/*  printf("dma_base = %08x, dma_len = %i\n",
+			    (int)dma_base, dma_len);  */
 
 			while (dma_len > 0) {
 				unsigned char ch;
-				res = memory_rw(cpu, cpu->mem, dma_base,
+				res = cpu->memory_rw(cpu, cpu->mem, dma_base,
 				    &ch, sizeof(ch), MEM_READ, PHYSICAL);
 				/*  printf(" %02x", ch);  */
 
 				d->cur_tx_packet[j++] = ch;
 				if (j >= MAX_TX_PACKET_LEN) {
-					fatal("[ mec_try_tx: packet too large? ]\n");
+					fatal("[ mec_try_tx: packet too large?"
+					    " ]\n");
 					break;
 				}
 				dma_base ++;
@@ -609,32 +677,45 @@ static void mec_try_tx(struct cpu *cpu, struct sgi_mec_data *d)
 	if (j < len)
 		fatal("[ mec_try_tx: not enough data? ]\n");
 
-	net_ethernet_tx(d, d->cur_tx_packet, d->cur_tx_packet_len);
+	net_ethernet_tx(cpu->machine->emul->net, d,
+	    d->cur_tx_packet, d->cur_tx_packet_len);
 
 	/*  see openbsd's if_mec.c for details  */
+	if (data[4] & 0x01) {
+		d->reg[MEC_INT_STATUS / sizeof(uint64_t)] |=
+		    MEC_INT_TX_PACKET_SENT;
+	}
+	memset(data, 0, 6);	/*  last 2 bytes are len  */
 	data[0] = 0x80;
 	data[5] = 0x80;
-	data[4] = 0x00;
 
-	res = memory_rw(cpu, cpu->mem, addr,
+	res = cpu->memory_rw(cpu, cpu->mem, addr,
 	    &data[0], sizeof(data), MEM_WRITE, PHYSICAL);
-	if (!res)
-		return;
 
+	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] |= MEC_INT_TX_EMPTY;
 	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] |= MEC_INT_TX_PACKET_SENT;
-	cpu_interrupt(cpu, d->irq_nr);
 
-skip:
+advance_tx:
 	/*  Advance the ring Read ptr.  */
 	tx_ring_ptr = d->reg[MEC_TX_RING_PTR / sizeof(uint64_t)];
 	ringread = tx_ring_ptr & MEC_TX_RING_READ_PTR;
 	ringwrite = tx_ring_ptr & MEC_TX_RING_WRITE_PTR;
 
-	ringread = ((ringread >> 16) + 1) << 16;
+	ringread = (ringread >> 16) + 1;
+	ringread &= 63;
+	ringread <<= 16;
 
 	d->reg[MEC_TX_RING_PTR / sizeof(uint64_t)] =
 	    (ringwrite & MEC_TX_RING_WRITE_PTR) |
 	    (ringread & MEC_TX_RING_READ_PTR);
+
+	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] &=
+	    ~MEC_INT_TX_RING_BUFFER_ALIAS;
+	d->reg[MEC_INT_STATUS / sizeof(uint64_t)] |=
+	    (d->reg[MEC_TX_RING_PTR / sizeof(uint64_t)] &
+	    MEC_INT_TX_RING_BUFFER_ALIAS);
+
+	return 1;
 }
 
 
@@ -644,17 +725,23 @@ skip:
 void dev_sgi_mec_tick(struct cpu *cpu, void *extra)
 {
 	struct sgi_mec_data *d = (struct sgi_mec_data *) extra;
+	int n = 0;
 
-	/*  RX:  */
-	mec_try_rx(cpu, d);
+	while (mec_try_tx(cpu, d))
+		;
 
-	/*  TX:  */
-	mec_try_tx(cpu, d);
+	while (mec_try_rx(cpu, d) && n < 16)
+		n++;
 
-	/*  Interrupts:  */
-	if (d->reg[MEC_INT_STATUS / sizeof(uint64_t)] & MEC_INT_STATUS_MASK)
+	/*  Interrupts:  (TODO: only when enabled)  */
+	if (d->reg[MEC_INT_STATUS / sizeof(uint64_t)] & MEC_INT_STATUS_MASK) {
+#if 0
+		printf("[%02x]", (int)(d->reg[MEC_INT_STATUS /
+		    sizeof(uint64_t)] & MEC_INT_STATUS_MASK));
+		fflush(stdout);
+#endif
 		cpu_interrupt(cpu, d->irq_nr);
-	else
+	} else
 		cpu_interrupt_ack(cpu, d->irq_nr);
 }
 
@@ -677,12 +764,15 @@ int dev_sgi_mec_access(struct cpu *cpu, struct memory *mem,
 	if (writeflag == MEM_WRITE) {
 		switch (relative_addr) {
 		case MEC_INT_STATUS:	/*  0x08  */
-			/*  Clear lowest bits on write:  (This is just a guess)  */
-			d->reg[regnr] = (idata & ~0xff) | ((d->reg[regnr] & ~idata) & 0xff);
+			/*  Clear bits on write:  (This is just a guess)  */
+			d->reg[regnr] = (d->reg[regnr] & ~0xff)
+			    | ((d->reg[regnr] & ~idata) & 0xff);
 			break;
 		case MEC_TX_RING_PTR:	/*  0x30  */
-			d->reg[regnr] = (idata & MEC_TX_RING_WRITE_PTR)
-			    | (d->reg[regnr] & ~MEC_TX_RING_WRITE_PTR);
+			idata &= MEC_TX_RING_WRITE_PTR;
+			d->reg[regnr] = (d->reg[regnr] &
+			    ~MEC_TX_RING_WRITE_PTR) | idata;
+			/*  TODO  */
 			break;
 		default:
 			d->reg[regnr] = idata;
@@ -702,38 +792,64 @@ int dev_sgi_mec_access(struct cpu *cpu, struct memory *mem,
 		break;
 	case MEC_INT_STATUS:	/*  0x08  */
 		if (writeflag)
-			debug("[ sgi_mec: write to MEC_INT_STATUS: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_INT_STATUS: "
+			    "0x%016llx ]\n", (long long)idata);
 		break;
 	case MEC_DMA_CONTROL:	/*  0x10  */
-		if (writeflag)
-			debug("[ sgi_mec: write to MEC_DMA_CONTROL: 0x%016llx ]\n", (long long)idata);
+		if (writeflag) {
+			debug("[ sgi_mec: write to MEC_DMA_CONTROL: "
+			    "0x%016llx ]\n", (long long)idata);
+			if (!(idata & MEC_DMA_TX_INT_ENABLE)) {
+				/*  This should apparently stop the
+				    TX Empty interrupt.  */
+				d->reg[MEC_INT_STATUS / sizeof(uint64_t)] &=
+				    ~MEC_INT_TX_EMPTY;
+			}
+		}
 		break;
 	case MEC_TX_ALIAS:	/*  0x20  */
-		/*  TODO?  */
-		if (writeflag)
-			debug("[ sgi_mec: write to MEC_TX_ALIAS: 0x%016llx ]\n", (long long)idata);
+		if (writeflag) {
+			debug("[ sgi_mec: write to MEC_TX_ALIAS: "
+			    "0x%016llx ]\n", (long long)idata);
+		} else {
+			debug("[ sgi_mec: read from MEC_TX_ALIAS: "
+			    "0x%016llx ]\n", (long long)idata);
+			odata = d->reg[MEC_TX_RING_PTR / sizeof(uint64_t)];
+		}
 		break;
 	case MEC_RX_ALIAS:	/*  0x28  */
 		if (writeflag)
-			debug("[ sgi_mec: write to MEC_RX_ALIAS: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_RX_ALIAS: "
+			    "0x%016llx ]\n", (long long)idata);
 		break;
 	case MEC_TX_RING_PTR:	/*  0x30  */
 		if (writeflag)
-			debug("[ sgi_mec: write to MEC_TX_RING_PTR: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_TX_RING_PTR: "
+			    "0x%016llx ]\n", (long long)idata);
 		break;
 	case MEC_PHY_DATA:	/*  0x64  */
 		if (writeflag)
-			fatal("[ sgi_mec: write to MEC_PHY_DATA: 0x%016llx ]\n", (long long)idata);
+			fatal("[ sgi_mec: write to MEC_PHY_DATA: "
+			    "0x%016llx ]\n", (long long)idata);
 		else
 			odata = 0;	/*  ?  */
 		break;
 	case MEC_PHY_ADDRESS:	/*  0x6c  */
 		if (writeflag)
-			debug("[ sgi_mec: write to MEC_PHY_ADDRESS: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_PHY_ADDRESS: "
+			    "0x%016llx ]\n", (long long)idata);
 		break;
 	case MEC_PHY_READ_INITIATE:	/*  0x70  */
 		if (writeflag)
-			debug("[ sgi_mec: write to MEC_PHY_READ_INITIATE: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_PHY_READ_INITIATE: "
+			    "0x%016llx ]\n", (long long)idata);
+		break;
+	case 0x74:
+		if (writeflag)
+			debug("[ sgi_mec: write to 0x74: 0x%016llx ]\n",
+			    (long long)idata);
+		else
+			debug("[ sgi_mec: read from 0x74 ]\n");
 		break;
 	case MEC_STATION:	/*  0xa0  */
 		if (writeflag)
@@ -745,23 +861,26 @@ int dev_sgi_mec_access(struct cpu *cpu, struct memory *mem,
 		break;
 	case MEC_STATION_ALT:	/*  0xa8  */
 		if (writeflag)
-			debug("[ sgi_mec: setting the ALTERNATIVE MAC address to "
-			    "%02x:%02x:%02x:%02x:%02x:%02x ]\n",
+			debug("[ sgi_mec: setting the ALTERNATIVE MAC address"
+			    " to %02x:%02x:%02x:%02x:%02x:%02x ]\n",
 			    (idata >> 40) & 255, (idata >> 32) & 255,
 			    (idata >> 24) & 255, (idata >> 16) & 255,
 			    (idata >>  8) & 255, (idata >>  0) & 255);
 		break;
 	case MEC_MULTICAST:	/*  0xb0  */
 		if (writeflag)
-			debug("[ sgi_mec: write to MEC_MULTICAST: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_MULTICAST: "
+			    "0x%016llx ]\n", (long long)idata);
 		break;
 	case MEC_TX_RING_BASE:	/*  0xb8  */
 		if (writeflag)
-			debug("[ sgi_mec: write to MEC_TX_RING_BASE: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_TX_RING_BASE: "
+			    "0x%016llx ]\n", (long long)idata);
 		break;
 	case MEC_MCL_RX_FIFO:	/*  0x100  */
 		if (writeflag) {
-			debug("[ sgi_mec: write to MEC_MCL_RX_FIFO: 0x%016llx ]\n", (long long)idata);
+			debug("[ sgi_mec: write to MEC_MCL_RX_FIFO: 0x"
+			    "%016llx ]\n", (long long)idata);
 			d->rx_addr[d->cur_rx_addr_index_write] = idata;
 			d->cur_rx_addr_index_write ++;
 			d->cur_rx_addr_index_write %= N_RX_ADDRESSES;
@@ -769,11 +888,12 @@ int dev_sgi_mec_access(struct cpu *cpu, struct memory *mem,
 		break;
 	default:
 		if (writeflag == MEM_WRITE)
-			fatal("[ sgi_mec: unimplemented write to address 0x%llx, data=0x%016llx ]\n",
+			fatal("[ sgi_mec: unimplemented write to address"
+			    " 0x%llx, data=0x%016llx ]\n",
 			    (long long)relative_addr, (long long)idata);
 		else
-			fatal("[ sgi_mec: unimplemented read from address 0x%llx ]\n",
-			    (long long)relative_addr);
+			fatal("[ sgi_mec: unimplemented read from address"
+			    " 0x%llx ]\n", (long long)relative_addr);
 	}
 
 	if (writeflag == MEM_READ)
@@ -788,24 +908,38 @@ int dev_sgi_mec_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_sgi_mec_init():
  */
-void dev_sgi_mec_init(struct cpu *cpu, struct memory *mem,
-	uint64_t baseaddr, int irq_nr)
+void dev_sgi_mec_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, int irq_nr, unsigned char *macaddr)
 {
+	char *name2;
 	struct sgi_mec_data *d = malloc(sizeof(struct sgi_mec_data));
+
 	if (d == NULL) {
 		fprintf(stderr, "out of memory\n");
 		exit(1);
 	}
 	memset(d, 0, sizeof(struct sgi_mec_data));
 	d->irq_nr = irq_nr;
+	memcpy(d->macaddr, macaddr, 6);
 
 	mec_reset(d);
 
-	memory_device_register(mem, "sgi_mec", baseaddr,
+	name2 = malloc(50);
+	if (name2 == NULL) {
+		fprintf(stderr, "out of memory in dev_sgi_mec_init()\n");
+		exit(1);
+	}
+	sprintf(name2, "mec [%02x:%02x:%02x:%02x:%02x:%02x]",
+	    d->macaddr[0], d->macaddr[1], d->macaddr[2],
+	    d->macaddr[3], d->macaddr[4], d->macaddr[5]);
+
+	memory_device_register(mem, name2, baseaddr,
 	    DEV_SGI_MEC_LENGTH, dev_sgi_mec_access, (void *)d,
 	    MEM_DEFAULT, NULL);
 
-	cpu_add_tickfunction(cpu, dev_sgi_mec_tick, d, MEC_TICK_SHIFT);
+	machine_add_tickfunction(machine, dev_sgi_mec_tick, d, MEC_TICK_SHIFT);
+
+	net_add_nic(machine->emul->net, d, macaddr);
 }
 
 
@@ -974,7 +1108,9 @@ int dev_sgi_mte_access(struct cpu *cpu, struct memory *mem,
 						fill_len = sizeof(zerobuf);
 					else
 						fill_len = zerobuflen;
-					memory_rw(cpu, mem, fill_addr, zerobuf, fill_len, MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
+					cpu->memory_rw(cpu, mem, fill_addr,
+					    zerobuf, fill_len, MEM_WRITE,
+					    NO_EXCEPTIONS | PHYSICAL);
 					fill_addr += fill_len;
 					zerobuflen -= sizeof(zerobuf);
 				}
