@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sgi_mardigras.c,v 1.9 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_sgi_mardigras.c,v 1.15 2005/01/30 12:54:43 debug Exp $
  *  
  *  "MardiGras" graphics controller on SGI IP30 (Octane).
  *
@@ -38,9 +38,10 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "misc.h"
-#include "memory.h"
+#include "cpu.h"
 #include "devices.h"
+#include "memory.h"
+#include "misc.h"
 
 
 #define debug fatal
@@ -229,6 +230,15 @@ int dev_sgi_mardigras_access(struct cpu *cpu, struct memory *mem,
 	}
 
 	switch (relative_addr) {
+	case 0x00004:
+		/*  xtalk data:  (according to Linux/IP30)  */
+		/*  (mfgr & 0x7ff) << 1  */
+		/*  (part & 0xffff) << 12  */
+		/*  (rev  & 0xf) << 28  */
+		odata = (2 << 28) | (0xc003 << 12) | (0x2aa << 1);
+		break;
+	case 0x20008:	/*  Fifo status  */
+		break;
 	case 0x20200:
 		break;
 	case 0x20400:
@@ -281,7 +291,8 @@ int dev_sgi_mardigras_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_sgi_mardigras_init():
  */
-void dev_sgi_mardigras_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr)
+void dev_sgi_mardigras_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr)
 {
 	struct sgi_mardigras_data *d;
 
@@ -292,7 +303,7 @@ void dev_sgi_mardigras_init(struct cpu *cpu, struct memory *mem, uint64_t basead
 	}
 	memset(d, 0, sizeof(struct sgi_mardigras_data));
 
-	d->fb = dev_fb_init(cpu, mem, MARDIGRAS_FAKE_OFFSET,
+	d->fb = dev_fb_init(machine, mem, MARDIGRAS_FAKE_OFFSET,
 	    VFB_GENERIC, mardigras_xsize, mardigras_ysize,
 	    mardigras_xsize, mardigras_ysize, 24, "SGI MardiGras", 1);
 	if (d->fb == NULL) {
