@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sii.c,v 1.10 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_sii.c,v 1.13 2005/02/11 09:53:48 debug Exp $
  *  
  *  SII SCSI controller, used in some DECstation systems.
  *
@@ -36,13 +36,18 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "devices.h"
 
 #include "siireg.h"
 
 
+#define	SII_TICK_SHIFT		14
+
+
 struct sii_data {
 	int		irq_nr;
 	uint64_t	buf_start;
@@ -129,12 +134,13 @@ void dev_sii_tick(struct cpu *cpu, void *extra)
 		d->connected = 0;
 		d->connected_to_id = 0;
 
-		/*  Is the target available for selection?   TODO: make this nicer  */
+		/*  Is the target available for selection?
+		    TODO: make this nicer  */
 #if 0
 		if ((d->siiregs.slcsr & 7) == 0) {
 			d->siiregs.cstat |=  SII_CON;	/*  Connected  */
 			d->siiregs.cstat |=  SII_SCH;	/*  State change  */
-			d->siiregs.cstat &= ~SII_SIP;	/*  Selection in progress  */
+			d->siiregs.cstat &= ~SII_SIP;	/*  Sel. in progress  */
 
 			d->connected = 1;
 			d->connected_to_id = 0;
@@ -144,15 +150,19 @@ void dev_sii_tick(struct cpu *cpu, void *extra)
 
 	if (d->siiregs.comm & SII_INXFER
 	    && (d->siiregs.comm & 0x70) == (d->siiregs.cstat & 0x70) &&
-	    (d->siiregs.comm & 0x03) == (d->siiregs.dstat & 0x03) && !(d->siiregs.cstat & SII_SIP)) {	/*  (I,T)  */
-		debug("[ sii: command INXFER to scsiid=%i ]\n", d->siiregs.slcsr);
+	    (d->siiregs.comm & 0x03) == (d->siiregs.dstat & 0x03) &&
+	    !(d->siiregs.cstat & SII_SIP)) {	/*  (I,T)  */
+		debug("[ sii: command INXFER to scsiid=%i ]\n",
+		    d->siiregs.slcsr);
 		if (d->siiregs.comm & SII_DMA)
 			debug("[ sii DMA: TODO ]\n");
 		else {
-			debug("[ sii: transmitting byte 0x%02x using PIO mode ]\n", d->siiregs.data);
+			debug("[ sii: transmitting byte 0x%02x using "
+			    "PIO mode ]\n", d->siiregs.data);
 			d->siiregs.comm &= ~SII_INXFER;
 
-/*			d->siiregs.dstat |= SII_DNE; */	/*  Done, only for DMA?  */
+/*			d->siiregs.dstat |= SII_DNE; */
+			    /*  Done, only for DMA?  */
 			d->siiregs.dstat |= SII_TBE;	/*  Buffer empty?  */
 		}
 	}
@@ -191,18 +201,22 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 	switch (relative_addr) {
 	case 0x00:				/*  SII_SDB: Diagnostic  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from SDB (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from SDB (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  SDB (data=0x%04x) ]\n", idata);
+			debug("[ sii: write to  SDB (data=0x%04x) ]\n",
+			    (int)idata);
 			d->regs[regnr] = idata;
 			return 1;
 		}
 		break;
 	case 0x0c:				/*  SII_CSR: Control/status  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from CSR (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from CSR (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  CSR (data=0x%04x: %s %s %s %s %s) ]\n", idata,
+			debug("[ sii: write to  CSR (data=0x%04x: %s %s "
+			    "%s %s %s) ]\n", (int)idata,
 			    idata & SII_HPM? "HPM" : "!hpm",
 			    idata & SII_RSE? "RSE" : "!rse",
 			    idata & SII_SLE? "SLE" : "!sle",
@@ -214,37 +228,48 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 		break;
 	case 0x10:				/*  SII_ID: SCSI ID  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from ID (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from ID (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  ID (data=0x%04x: scsi id %i) ]\n", idata, idata & 7);
+			debug("[ sii: write to  ID (data=0x%04x: scsi id %i)"
+			    " ]\n", (int)idata, (int)(idata & 7));
 			if (!(idata & SII_ID_IO))
-				debug("WARNING: sii ID bit SII_ID_IO not set on write!\n");
+				debug("WARNING: sii ID bit SII_ID_IO not "
+				    "set on write!\n");
 			idata &= ~SII_ID_IO;
 			if ((idata & ~0x7) != 0)
-				debug("WARNING: sii ID bits that should be zero are not zero!\n");
+				debug("WARNING: sii ID bits that should "
+				    "be zero are not zero!\n");
 			idata &= 0x7;
 			d->regs[regnr] = idata & 0x7;
 			return 1;
 		}
 		break;
-	case 0x14:				/*  SII_SLCSR: Selector control  */
+	case 0x14:			/*  SII_SLCSR: Selector control  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from SLCSR (data=0x%04x: scsi_id=%i) ]\n", d->regs[regnr], d->regs[regnr] & 7);
+			debug("[ sii: read from SLCSR (data=0x%04x: "
+			    "scsi_id=%i) ]\n", d->regs[regnr],
+			    d->regs[regnr] & 7);
 		} else {
-			debug("[ sii: write to  SLCSR (data=0x%04x: scsi_id=%i) ]\n", idata, idata & 7);
+			debug("[ sii: write to  SLCSR (data=0x%04x: "
+			    "scsi_id=%i) ]\n", (int)idata, (int)(idata & 7));
 			if ((idata & ~0x7) != 0)
-				debug("WARNING: sii SLCSR bits that should be zero are not zero!\n");
+				debug("WARNING: sii SLCSR bits that should "
+				    "be zero are not zero!\n");
 			idata &= 0x7;
 			d->regs[regnr] = idata & 0x7;
 			return 1;
 		}
 		break;
-	case 0x18:				/*  SII_DESTAT: Selection detector status  */
+	case 0x18:		/*  SII_DESTAT: Selection detector status  */
 		if (writeflag == MEM_READ) {
-			/*  TODO: set the DESTAT register from somewhere else?  */
-			debug("[ sii: read from DESTAT (data=0x%04x: scsi_id=%i) ]\n", d->regs[regnr], d->regs[regnr] & 7);
+			/*  TODO: set DESTAT from somewhere else?  */
+			debug("[ sii: read from DESTAT (data=0x%04x: "
+			    "scsi_id=%i) ]\n", d->regs[regnr],
+			    d->regs[regnr] & 7);
 		} else {
-			debug("[ sii: write to  DESTAT (data=0x%04x: scsi_id=%i) ]\n", idata, idata & 7);
+			debug("[ sii: write to  DESTAT (data=0x%04x: "
+			    "scsi_id=%i) ]\n", (int)idata, (int)(idata & 7));
 			debug("WARNING: sii DESTAT is read-only!\n");
 			return 1;
 		}
@@ -252,10 +277,12 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 	case 0x20:				/*  SII_DATA: Data register  */
 		if (writeflag == MEM_READ) {
 			/*  TODO  */
-			debug("[ sii: read from DATA (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from DATA (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
 			/*  TODO  */
-			debug("[ sii: write to  DATA (data=0x%04x) ]\n", idata);
+			debug("[ sii: write to  DATA (data=0x%04x) ]\n",
+			    (int)idata);
 			idata &= 0xff;
 			d->regs[regnr] = idata;
 			return 1;
@@ -263,24 +290,29 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 		break;
 	case 0x24:				/*  SII_DMCTRL: DMA control  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from DMCTRL (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from DMCTRL (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  DMCTRL (data=0x%04x: %s) ]\n", idata,
-			    (idata & 3)==0? "async" : "sync");
+			debug("[ sii: write to  DMCTRL (data=0x%04x: %s) ]\n",
+			    (int)idata, (idata & 3)==0? "async" : "sync");
 			if ((idata & ~0x3) != 0)
-				debug("WARNING: sii DMCTRL bits that should be zero are not zero!\n");
+				debug("WARNING: sii DMCTRL bits that "
+				    "should be zero are not zero!\n");
 			idata &= 0x3;
 			d->regs[regnr] = idata;
 			return 1;
 		}
 		break;
-	case 0x48:				/*  SII_CSTAT: Connection status  */
+	case 0x48:			/*  SII_CSTAT: Connection status  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from CSTAT (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from CSTAT (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  CSTAT (data=0x%04x) ]\n", idata);
+			debug("[ sii: write to  CSTAT (data=0x%04x) ]\n",
+			    (int)idata);
 
-			/*  readonly / writeoncetoclear bits according to page 21 in the DS3100 manual:  */
+			/*  readonly / writeoncetoclear bits according
+			    to page 21 in the DS3100 manual:  */
 			if (idata & (1<<13)) {
 				idata &= ~(1<<13); d->regs[regnr] &= ~(1<<13);
 			}
@@ -288,13 +320,16 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 				idata &= ~(1<<12); d->regs[regnr] &= ~(1<<12);
 			}
 			if (idata & (1<<11)) {
-				idata &= ~(1<<11); d->regs[regnr] &= ~(1<<11);		/*  is this write-1-to-clear?  */
+				/*  is this actually write-1-to-clear?  */
+				idata &= ~(1<<11); d->regs[regnr] &= ~(1<<11);
 			}
 			if (idata & (1<<9)) {
-				idata &= ~(1<<9); d->regs[regnr] &= ~(1<<9);		/*  ?  */
+				/*  ?  */
+				idata &= ~(1<<9); d->regs[regnr] &= ~(1<<9);
 			}
 			if (idata & (1<<8)) {
-				idata &= ~(1<<8); d->regs[regnr] &= ~(1<<8);		/*  ?  */
+				/*  ?  */
+				idata &= ~(1<<8); d->regs[regnr] &= ~(1<<8);
 			}
 			if (idata & (1<<7)) {
 				idata &= ~(1<<7); d->regs[regnr] &= ~(1<<7);
@@ -303,37 +338,45 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 				idata &= ~(1<<3); d->regs[regnr] &= ~(1<<3);
 			}
 
-			idata &= ~0x3bf7;		/*  Read-only bits are taken from the old register.  */
+			/*  Read-only bits are taken from the old register:  */
+			idata &= ~0x3bf7;
 			idata |= d->regs[regnr] & 0x3bf7;
 
 			d->regs[regnr] = idata;
 			return 1;
 		}
 		break;
-	case 0x4c:				/*  SII_DSTAT: Data transfer status  */
+	case 0x4c:			/*  SII_DSTAT: Data transfer status  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from DSTAT (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from DSTAT (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  DSTAT (data=0x%04x) ]\n", idata);
+			debug("[ sii: write to  DSTAT (data=0x%04x) ]\n",
+			    (int)idata);
 
-			/*  readonly / writeoncetoclear bits according to page 22 in the DS3100 manual:  */
+			/*  readonly / writeoncetoclear bits
+			    according to page 22 in the DS3100 manual:  */
 			if (idata & (1<<13)) {
 				idata &= ~(1<<13); d->regs[regnr] &= ~(1<<13);
 			}
 			if (idata & (1<<11)) {
-				idata &= ~(1<<11); d->regs[regnr] &= ~(1<<11);		/*  is this write-1-to-clear?  */
+				/*  is this write-1-to-clear?  */
+				idata &= ~(1<<11); d->regs[regnr] &= ~(1<<11);
 			}
 			if (idata & (1<<10)) {
-				idata &= ~(1<<10); d->regs[regnr] &= ~(1<<10);		/*  is this write-1-to-clear?  */
+				/*  is this write-1-to-clear?  */
+				idata &= ~(1<<10); d->regs[regnr] &= ~(1<<10);
 			}
 			if (idata & (1<<4)) {
-				idata &= ~(1<<4); d->regs[regnr] &= ~(1<<4);		/*  is this write-1-to-clear?  */
+				/*  is this write-1-to-clear?  */
+				idata &= ~(1<<4); d->regs[regnr] &= ~(1<<4);
 			}
 			if (idata & (1<<3)) {
 				idata &= ~(1<<3); d->regs[regnr] &= ~(1<<3);
 			}
 
-			idata &= ~0x0c17;	/*  Read-only bits are taken from the old register.  */
+			/*  Read-only bits are taken from the old register:  */
+			idata &= ~0x0c17;
 			idata |= d->regs[regnr] & 0x0c17;
 
 			d->regs[regnr] = idata;
@@ -342,14 +385,18 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 		break;
 	case 0x50:				/*  SII_COMM: Command  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from COMM (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from COMM (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  COMM (data=0x%04x: %s %s %s command=0x%02x rest=0x%02x) ]\n", idata,
+			debug("[ sii: write to  COMM (data=0x%04x: %s %s "
+			    "%s command=0x%02x rest=0x%02x) ]\n", (int)idata,
 			    idata & SII_DMA?    "DMA" : "!dma",
 			    idata & SII_DO_RST? "RST" : "!rst",
 			    idata & SII_RSL?    "RSL" : "!rsl",
-			    (idata >> 7) & 0x1f,		/*  command, 5 bits  */
-			    idata & 0x3f);			/*  rest, 7 bits  */
+			    /*  command, 5 bits:  */
+			    (int)((idata >> 7) & 0x1f),
+			    /*  rest, 7 bits:  */
+			    (int)(idata & 0x3f));
 
 			if (idata & SII_DO_RST) {
 				/*  Reset: TODO  */
@@ -362,23 +409,28 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 			return 1;
 		}
 		break;
-	case 0x54:				/*  SII_DICTRL: Diagnostics control  */
+	case 0x54:			/*  SII_DICTRL: Diagnostics control  */
 		if (writeflag == MEM_READ) {
-			debug("[ sii: read from DICTRL (data=0x%04x) ]\n", d->regs[regnr]);
+			debug("[ sii: read from DICTRL (data=0x%04x) ]\n",
+			    d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  DICTRL (data=0x%04x: port=%s) ]\n",
-			    idata, idata & SII_PRE? "enabled" : "disabled");
+			debug("[ sii: write to  DICTRL (data=0x%04x: "
+			    "port=%s) ]\n", (int)idata,
+			    idata & SII_PRE? "enabled" : "disabled");
 			if ((idata & ~0xf) != 0)
-				debug("WARNING: sii DICTRL bits that should be zero are not zero!\n");
+				debug("WARNING: sii DICTRL bits that "
+				    "should be zero are not zero!\n");
 			d->regs[regnr] = idata;
 			return 1;
 		}
 		break;
 	default:
 		if (writeflag==MEM_READ) {
-			debug("[ sii: read from %08lx (data=0x%04x) ]\n", (long)relative_addr, d->regs[regnr]);
+			debug("[ sii: read from %08lx (data=0x%04x) ]\n",
+			    (long)relative_addr, d->regs[regnr]);
 		} else {
-			debug("[ sii: write to  %08lx (data=0x%04x) ]\n", (long)relative_addr, idata);
+			debug("[ sii: write to  %08lx (data=0x%04x) ]\n",
+			    (long)relative_addr, (int)idata);
 			d->regs[regnr] = idata;
 		}
 	}
@@ -393,8 +445,8 @@ int dev_sii_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_sii_init():
  */
-void dev_sii_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	uint64_t buf_start, uint64_t buf_end, int irq_nr)
+void dev_sii_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, uint64_t buf_start, uint64_t buf_end, int irq_nr)
 {
 	struct sii_data *d = malloc(sizeof(struct sii_data));
 	if (d == NULL) {
@@ -412,6 +464,6 @@ void dev_sii_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	memory_device_register(mem, "sii", baseaddr, DEV_SII_LENGTH,
 	    dev_sii_access, (void *)d, MEM_DEFAULT, NULL);
 
-	cpu_add_tickfunction(cpu, dev_sii_tick, d, 10);
+	machine_add_tickfunction(machine, dev_sii_tick, d, SII_TICK_SHIFT);
 }
 
