@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ssc.c,v 1.14 2005/01/09 04:17:18 debug Exp $
+ *  $Id: dev_ssc.c,v 1.21 2005/02/11 09:29:48 debug Exp $
  *  
  *  Serial controller on DECsystem 5400 and 5800.
  *  Known as System Support Chip on VAX 3600 (KA650).
@@ -37,10 +37,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "console.h"
+#include "cpu.h"
+#include "devices.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "console.h"
-#include "devices.h"
 
 
 #define	RX_INT_ENABLE	0x40
@@ -58,6 +60,7 @@
 
 struct ssc_data {
 	int		irq_nr;
+	int		console_handle;
 	int		use_fb;
 
 	int		rx_ctl;
@@ -77,7 +80,7 @@ void dev_ssc_tick(struct cpu *cpu, void *extra)
 	d->tx_ctl |= TX_READY;	/*  transmitter always ready  */
 
 	d->rx_ctl &= ~RX_AVAIL;
-	if (console_charavail())
+	if (console_charavail(d->console_handle))
 		d->rx_ctl |= RX_AVAIL;
 
 	/*  rx interrupts enabled, and char avail?  */
@@ -87,7 +90,8 @@ void dev_ssc_tick(struct cpu *cpu, void *extra)
 		if (d->csrp != NULL) {
 			unsigned char txvector = 0xf8;
 			(*d->csrp) |= 0x10000000;
-			memory_rw(cpu, cpu->mem, 0x40000050, &txvector, 1, MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
+			cpu->memory_rw(cpu, cpu->mem, 0x40000050, &txvector,
+			    1, MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
 			cpu_interrupt(cpu, 2);
 		}
 	}
@@ -99,7 +103,8 @@ void dev_ssc_tick(struct cpu *cpu, void *extra)
 		if (d->csrp != NULL) {
 			unsigned char txvector = 0xfc;
 			(*d->csrp) |= 0x10000000;
-			memory_rw(cpu, cpu->mem, 0x40000050, &txvector, 1, MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
+			cpu->memory_rw(cpu, cpu->mem, 0x40000050, &txvector,
+			    1, MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
 			cpu_interrupt(cpu, 2);
 		}
 	}
@@ -146,8 +151,8 @@ int dev_ssc_access(struct cpu *cpu, struct memory *mem,
 #ifdef SSC_DEBUG_TXRX
 			debug("[ ssc: read from 0x%08lx ]\n", (long)relative_addr);
 #endif
-			if (console_charavail())
-				odata = console_readchar();
+			if (console_charavail(d->console_handle))
+				odata = console_readchar(d->console_handle);
 		} else {
 #ifdef SSC_DEBUG_TXRX
 			debug("[ ssc: write to 0x%08lx: 0x%02x ]\n", (long)relative_addr, idata);
@@ -180,7 +185,7 @@ int dev_ssc_access(struct cpu *cpu, struct memory *mem,
 			debug("[ ssc: read from 0x%08lx ]\n", (long)relative_addr);
 		} else {
 			/*  debug("[ ssc: write to 0x%08lx: 0x%02x ]\n", (long)relative_addr, idata);  */
-			console_putchar(idata);
+			console_putchar(d->console_handle, idata);
 		}
 
 		break;
@@ -227,8 +232,8 @@ int dev_ssc_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_ssc_init():
  */
-void dev_ssc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	int irq_nr, int use_fb, uint32_t *csrp)
+void dev_ssc_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, int irq_nr, int use_fb, uint32_t *csrp)
 {
 	struct ssc_data *d;
 
@@ -241,10 +246,11 @@ void dev_ssc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	d->irq_nr = irq_nr;
 	d->use_fb = use_fb;
 	d->csrp   = csrp;
+	d->console_handle = console_start_slave(machine, "SSC");
 
 	memory_device_register(mem, "ssc", baseaddr, DEV_SSC_LENGTH,
 	    dev_ssc_access, d, MEM_DEFAULT, NULL);
 
-	cpu_add_tickfunction(cpu, dev_ssc_tick, d, 14);
+	machine_add_tickfunction(machine, dev_ssc_tick, d, 14);
 }
 
