@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_wdsc.c,v 1.15 2005/01/09 01:55:25 debug Exp $
+ *  $Id: dev_wdsc.c,v 1.23 2005/01/30 13:14:12 debug Exp $
  *  
  *  WDSC SCSI (WD33C93) controller.
  *  (For SGI-IP22. See sys/arch/sgimips/hpc/sbic* in NetBSD for details.)
@@ -37,11 +37,13 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "console.h"
+#include "cpu.h"
+#include "devices.h"
+#include "diskimage.h"
+#include "machine.h"
 #include "memory.h"
 #include "misc.h"
-#include "console.h"
-#include "diskimage.h"
-#include "devices.h"
 
 #include "wdsc_sbicreg.h"
 
@@ -196,7 +198,8 @@ static void dev_wdsc_regwrite(struct cpu *cpu, struct wdsc_data *d, int idata)
 			debug("SEL_ATN");
 			d->irq_pending = 1;
 			d->reg[SBIC_csr] = SBIC_CSR_SEL_TIMEO;
-			if (d->controller_nr == 0 && diskimage_exist(d->reg[SBIC_selid] & SBIC_SID_IDMASK)) {
+			if (d->controller_nr == 0 && diskimage_exist(
+			    cpu->machine, d->reg[SBIC_selid] & SBIC_SID_IDMASK)) {
 				if (d->xfer != NULL)
 					scsi_transfer_free(d->xfer);
 				d->xfer = scsi_transfer_alloc();
@@ -381,8 +384,8 @@ int dev_wdsc_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_wdsc_init():
  */
-void dev_wdsc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
-	int controller_nr, int irq_nr)
+void dev_wdsc_init(struct machine *machine, struct memory *mem,
+	uint64_t baseaddr, int controller_nr, int irq_nr)
 {
 	struct wdsc_data *d;
 
@@ -398,6 +401,6 @@ void dev_wdsc_init(struct cpu *cpu, struct memory *mem, uint64_t baseaddr,
 	memory_device_register(mem, "wdsc", baseaddr, DEV_WDSC_LENGTH,
 	    dev_wdsc_access, d, MEM_DEFAULT, NULL);
 
-	cpu_add_tickfunction(cpu, dev_wdsc_tick, d, 13);
+	machine_add_tickfunction(machine, dev_wdsc_tick, d, 14);
 }
 
