@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: pci_dec21143.c,v 1.5 2005/01/09 01:55:26 debug Exp $
+ *  $Id: pci_dec21143.c,v 1.6 2005/01/23 13:43:02 debug Exp $
  *
  *  DEC 21143 PCI ethernet.
  *
@@ -69,7 +69,7 @@ uint32_t pci_dec21143_rr(int reg)
 /*
  *  pci_dec21143_init():
  */
-void pci_dec21143_init(struct cpu *cpu, struct memory *mem)
+void pci_dec21143_init(struct machine *machine, struct memory *mem)
 {
 }
 
