@@ -2,7 +2,7 @@
 #define	PCI_BUS_H
 
 /*
- *  Copyright (C) 2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,16 +28,18 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bus_pci.h,v 1.8 2005/01/09 01:55:28 debug Exp $
+ *  $Id: bus_pci.h,v 1.10 2005/01/25 08:38:27 debug Exp $
  */
 
-#include "memory.h"
 #include "misc.h"
 
+struct machine;
+struct memory;
+
 struct pci_device {
 	int		bus, device, function;
 
-	void		(*init)(struct cpu *, struct memory *mem);
+	void		(*init)(struct machine *, struct memory *mem);
 	uint32_t	(*read_register)(int reg);
 
 	struct pci_device *next;
@@ -61,9 +63,10 @@ struct pci_data {
 
 /*  bus_pci.c:  */
 int bus_pci_access(struct cpu *cpu, struct memory *mem, uint64_t relative_addr, uint64_t *data, int writeflag, struct pci_data *pci_data);
-void bus_pci_add(struct cpu *cpu, struct pci_data *pci_data, struct memory *mem,
+void bus_pci_add(struct machine *machine, struct pci_data *pci_data, struct memory *mem,
 	int bus, int device, int function,
-	void (*init)(struct cpu *, struct memory *), uint32_t (*read_register)(int reg));
+	void (*init)(struct machine *, struct memory *),
+	uint32_t (*read_register)(int reg));
 struct pci_data *bus_pci_init(struct memory *mem, int irq_nr);
 
 
@@ -73,21 +76,21 @@ struct pci_data *bus_pci_init(struct memory *mem, int irq_nr);
 
 /*  ahc:  */
 uint32_t pci_ahc_rr(int reg);
-void pci_ahc_init(struct cpu *, struct memory *mem);
+void pci_ahc_init(struct machine *, struct memory *mem);
 
 /*  dec21030:  */
 uint32_t pci_dec21030_rr(int reg);
-void pci_dec21030_init(struct cpu *, struct memory *mem);
+void pci_dec21030_init(struct machine *, struct memory *mem);
 
 /*  dec21143:  */
 uint32_t pci_dec21143_rr(int reg);
-void pci_dec21143_init(struct cpu *, struct memory *mem);
+void pci_dec21143_init(struct machine *, struct memory *mem);
 
 /*  vt82c586:  */
 uint32_t pci_vt82c586_isa_rr(int reg);
-void pci_vt82c586_isa_init(struct cpu *, struct memory *mem);
+void pci_vt82c586_isa_init(struct machine *, struct memory *mem);
 uint32_t pci_vt82c586_ide_rr(int reg);
-void pci_vt82c586_ide_init(struct cpu *, struct memory *mem);
+void pci_vt82c586_ide_init(struct machine *, struct memory *mem);
 
 
 #endif	/*  PCI_BUS_H  */
