@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bintrans.c,v 1.133 2005/01/10 22:30:41 debug Exp $
+ *  $Id: bintrans.c,v 1.151 2005/02/09 20:36:08 debug Exp $
  *
  *  Dynamic binary translation.
  *
@@ -35,7 +35,8 @@
  *	o)  Keep a translation cache of a certain number of blocks.
  *
  *	o)  Only translate simple instructions. (For example, the 'tlbwr'
- *	    instruction is not translated.)
+ *	    instruction is not actually translated, converted into a call
+ *	    to C code.)
  *
  *	o)  Translate code in physical ram, not virtual. This will keep
  *	    things translated over process switches, and TLB updates.
@@ -49,8 +50,8 @@
  *
  *	o)  Do not translate over MIPS page boundaries (4 KB).
  *	    (TODO: Perhaps it would be possible if we're running in kernel
- *	    space? But it this would then require special checks at the
- *	    end of each page.)
+ *	    space? But that would require special checks at the end of
+ *	    each page.)
  *
  *	o)  If memory is overwritten, any translated block for that page
  *	    must be invalidated. (It is removed from the cache so that it
@@ -68,7 +69,8 @@
  *	    directly from MIPS machine code to target machine code.
  *
  *	o)  Theoretical support for multiple target architectures (Alpha,
- *	    i386, sparc, mips :-), ...), but only Alpha implemented so far.
+ *	    i386, sparc, mips :-), ...), but only Alpha and i386 implemented
+ *	    so far.
  *
  *	o)  Load/stores: TODO: Comment.
  *
@@ -87,7 +89,6 @@
  *  non-safe.
  */
 
-
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -95,17 +96,19 @@
 #include <sys/types.h>
 #include <sys/mman.h>
 
-#include "misc.h"
-
 #include "bintrans.h"
 #include "cop0.h"
-#include "cpu_types.h"
-#include "emul.h"
+#include "cpu.h"
+#include "cpu_mips.h"
+#include "machine.h"
 #include "memory.h"
-#include "opcodes.h"
+#include "mips_cpu_types.h"
+#include "misc.h"
+#include "opcodes_mips.h"
 
 
 #ifndef BINTRANS
+
 /*
  *  No bintrans, then let's supply dummy functions:
  */
@@ -114,9 +117,10 @@ int bintrans_pc_is_in_cache(struct cpu *cpu, uint64_t pc) { return 0; }
 void bintrans_invalidate(struct cpu *cpu, uint64_t paddr) { }
 int bintrans_attempt_translate(struct cpu *cpu, uint64_t paddr) { return 0; }
 void bintrans_init_cpu(struct cpu *cpu) { }
-void bintrans_init(void)
+void bintrans_init(struct machine *machine, struct memory *mem)
 {
-	fatal("NOT starting bintrans, as mips64emul was compiled without such support!\n");
+	fatal("\n***  NOT starting bintrans, as mips64emul "
+	    "was compiled without such support!\n\n");
 }
 
 #else
@@ -128,105 +132,59 @@ static void bintrans_host_cacheinvalidate(unsigned char *p, size_t len);
 static void bintrans_write_chunkreturn(unsigned char **addrp);
 static void bintrans_write_chunkreturn_fail(unsigned char **addrp);
 static void bintrans_write_pc_inc(unsigned char **addrp);
-/*  static void bintrans_runchunk(struct cpu *cpu, unsigned char *code);  */
-static void bintrans_write_quickjump(unsigned char *quickjump_code, uint32_t chunkoffset);
+static void bintrans_write_quickjump(struct memory *mem, unsigned char *quickjump_code, uint32_t chunkoffset);
 static int bintrans_write_instruction__addiu_etc(unsigned char **addrp, int rt, int rs, int imm, int instruction_type);
 static int bintrans_write_instruction__addu_etc(unsigned char **addrp, int rd, int rs, int rt, int sa, int instruction_type);
 static int bintrans_write_instruction__branch(unsigned char **addrp, int instruction_type, int regimm_type, int rt, int rs, int imm);
 static int bintrans_write_instruction__jr(unsigned char **addrp, int rs, int rd, int special);
 static int bintrans_write_instruction__jal(unsigned char **addrp, int imm, int link);
-static int bintrans_write_instruction__delayedbranch(unsigned char **addrp, uint32_t *potential_chunk_p, uint32_t *chunks, int only_care_about_chunk_p, int p, int forward);
-static int bintrans_write_instruction__loadstore(unsigned char **addrp, int rt, int imm, int rs, int instruction_type, int bigendian);
+static int bintrans_write_instruction__delayedbranch(struct memory *mem, unsigned char **addrp, uint32_t *potential_chunk_p, uint32_t *chunks, int only_care_about_chunk_p, int p, int forward);
+static int bintrans_write_instruction__loadstore(struct memory *mem, unsigned char **addrp, int rt, int imm, int rs, int instruction_type, int bigendian);
 static int bintrans_write_instruction__lui(unsigned char **addrp, int rt, int imm);
 static int bintrans_write_instruction__mfmthilo(unsigned char **addrp, int rd, int from_flag, int hi_flag);
-static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc_nr, int flag64bit, int rt, int rd, int mtcflag);
+static int bintrans_write_instruction__mfc_mtc(struct memory *mem, unsigned char **addrp, int coproc_nr, int flag64bit, int rt, int rd, int mtcflag);
 static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp, int itype);
 
-#define	TLB_TLBWI	0
-#define	TLB_TLBWR	1
-#define	TLB_TLBP	2
-#define	TLB_TLBR	3
-#define	TLB_RFE		4
-#define	TLB_ERET	5
-#define	TLB_BREAK	6
-#define	TLB_SYSCALL	7
+#define	CALL_TLBWI	0
+#define	CALL_TLBWR	1
+#define	CALL_TLBP	2
+#define	CALL_TLBR	3
+#define	CALL_RFE	4
+#define	CALL_ERET	5
+#define	CALL_BREAK	6
+#define	CALL_SYSCALL	7
 
 
-#define	BINTRANS_CACHE_N_INDEX_BITS	15
-#define	CACHE_INDEX_MASK		((1 << BINTRANS_CACHE_N_INDEX_BITS) - 1)
-#define	PADDR_TO_INDEX(p)		((p >> 12) & CACHE_INDEX_MASK)
-
-#ifndef BINTRANS_SIZE_IN_MB
-#define BINTRANS_SIZE_IN_MB		16
-#endif
-
-#define	CODE_CHUNK_SPACE_SIZE		(BINTRANS_SIZE_IN_MB * 1048576)
-#define	CODE_CHUNK_SPACE_MARGIN		65536
-
-/*
- *  translation_code_chunk_space is a large chunk of (linear) memory where
- *  translated code chunks and translation_entrys are stored. When this is
- *  filled, we restart from scratch (by resetting
- *  translation_code_chunk_space_head to 0, and removing all translation
- *  entries).
- *
- *  (Using a static memory region like this is somewhat inspired by the QEMU
- *  web pages, http://fabrice.bellard.free.fr/qemu/qemu-tech.html#SEC13)
- */
-unsigned char *translation_code_chunk_space;
-size_t translation_code_chunk_space_head;
-
-struct translation_page_entry {
-	struct translation_page_entry	*next;
-
-	uint64_t			paddr;
-
-	int				page_is_potentially_in_use;
-
-	uint32_t			chunk[1024];
-	char				flags[1024];
-};
-#define	UNTRANSLATABLE		0x01
-
-static struct translation_page_entry *translation_page_entry_array[
-    (1 << BINTRANS_CACHE_N_INDEX_BITS) ];
-
-
-#define	MAX_QUICK_JUMPS		8
-static unsigned char *quick_jump_host_address[MAX_QUICK_JUMPS];
-static int quick_jump_page_offset[MAX_QUICK_JUMPS];
-static int n_quick_jumps;
-static int quick_jumps_index;		/*  where to write the next quick jump  */
-
-static void bintrans_register_potential_quick_jump(unsigned char *a, int p)
+static void bintrans_register_potential_quick_jump(struct memory *mem,
+	unsigned char *a, int p)
 {
-	/*  printf("%02i: a=%016llx p=%i\n", quick_jumps_index, a, p);  */
-	quick_jump_host_address[quick_jumps_index] = a;
-	quick_jump_page_offset[quick_jumps_index] = p;
-	quick_jumps_index ++;
-	if (quick_jumps_index > n_quick_jumps)
-		n_quick_jumps = quick_jumps_index;
-	if (quick_jumps_index >= MAX_QUICK_JUMPS)
-		quick_jumps_index = 0;
+	/*  printf("%02i: a=%016llx p=%i\n", mem->quick_jumps_index, a, p);  */
+	mem->quick_jump_host_address[mem->quick_jumps_index] = a;
+	mem->quick_jump_page_offset[mem->quick_jumps_index] = p;
+	mem->quick_jumps_index ++;
+	if (mem->quick_jumps_index > mem->n_quick_jumps)
+		mem->n_quick_jumps = mem->quick_jumps_index;
+	if (mem->quick_jumps_index >= MAX_QUICK_JUMPS)
+		mem->quick_jumps_index = 0;
 }
 
 
-/*  Set to non-zero for R3000 and similar cpus.  */
-static int bintrans_32bit_only = 0;
-
-
 /*  Include host architecture specific bintrans code:  */
 
 #ifdef ALPHA
+#define BACKEND_NAME "Alpha"
 #include "bintrans_alpha.c"
 #else
 #ifdef I386
+#define BACKEND_NAME "i386"
 #include "bintrans_i386.c"
 #else
 #ifdef MIPS
+#define BACKEND_NAME "MIPS"
 #include "bintrans_mips.c"
 #else
 #ifdef SPARCV9
+#define BACKEND_NAME "UltraSPARC"
 #include "bintrans_sparcv9.c"
 #else
 #error Unsupported host architecture for bintrans.
@@ -250,7 +208,7 @@ struct translation_page_entry *prev = NULL;
 #endif
 	uint64_t paddr_page = paddr & ~0xfff;
 
-	tep = translation_page_entry_array[entry_index];
+	tep = cpu->mem->translation_page_entry_array[entry_index];
 	while (tep != NULL) {
 		if (tep->paddr == paddr_page)
 #if 1
@@ -258,7 +216,7 @@ struct translation_page_entry *prev = NULL;
 #else
 {
 			if (prev == NULL)
-				translation_page_entry_array[entry_index] = tep->next;
+				cpu->mem->translation_page_entry_array[entry_index] = tep->next;
 			else
 				prev->next = tep->next;
 			return;
@@ -288,11 +246,11 @@ static void enter_chunks_into_tables(struct cpu *cpu, uint64_t vaddr, uint32_t *
 	int a, b;
 	struct vth32_table *tbl1;
 
-	switch (cpu->cpu_type.mmu_model) {
+	switch (cpu->cd.mips.cpu_type.mmu_model) {
 	case MMU3K:
 		a = (vaddr >> 22) & 0x3ff;
 		b = (vaddr >> 12) & 0x3ff;
-		tbl1 = cpu->vaddr_to_hostaddr_table0_kernel[a];
+		tbl1 = cpu->cd.mips.vaddr_to_hostaddr_table0_kernel[a];
 		if (tbl1->haddr_entry[b] != NULL)
 			tbl1->bintrans_chunks[b] = chunk0;
 		break;
@@ -335,26 +293,27 @@ int bintrans_attempt_translate(struct cpu *cpu, uint64_t paddr)
 
 
 	/*  Abort if the current "environment" isn't safe enough:  */
-	if (cpu->delay_slot || cpu->nullify_next || (paddr & 3) != 0)
-		return cpu->bintrans_instructions_executed;
+	if (cpu->cd.mips.delay_slot || cpu->cd.mips.nullify_next || (paddr & 3) != 0)
+		return cpu->cd.mips.bintrans_instructions_executed;
 
-	bintrans_32bit_only = cpu->cpu_type.mmu_model == MMU3K;
+	cpu->mem->bintrans_32bit_only = (cpu->cd.mips.cpu_type.isa_level <= 2 ||
+	    cpu->cd.mips.cpu_type.isa_level == 32);
 	byte_order_cached_bigendian = (cpu->byte_order == EMUL_BIG_ENDIAN);
 
 	/*  Is this a part of something that is already translated?  */
 	paddr_page = paddr & ~0xfff;
 	offset_within_page = (paddr & 0xfff) / 4;
 	entry_index = PADDR_TO_INDEX(paddr);
-	tep = translation_page_entry_array[entry_index];
+	tep = cpu->mem->translation_page_entry_array[entry_index];
 	while (tep != NULL) {
 		if (tep->paddr == paddr_page) {
 			if (tep->chunk[offset_within_page] != 0) {
 				f = (size_t)tep->chunk[offset_within_page] +
-				    translation_code_chunk_space;
+				    cpu->mem->translation_code_chunk_space;
 				goto run_it;	/*  see further down  */
 			}
 			if (tep->flags[offset_within_page] & UNTRANSLATABLE)
-				return cpu->bintrans_instructions_executed;
+				return cpu->cd.mips.bintrans_instructions_executed;
 			break;
 		}
 		tep = tep->next;
@@ -367,54 +326,55 @@ quick_attempt_translate_again:
 #endif
 /*printf("B: ");
 printf("v=%016llx p=%016llx h=%p paddr=%016llx\n",
-(long long)cpu->pc_last_virtual_page,
-(long long)cpu->pc_last_physical_page,
-cpu->pc_last_host_4k_page,(long long)paddr);
+(long long)cpu->cd.mips.pc_last_virtual_page,
+(long long)cpu->cd.mips.pc_last_physical_page,
+cpu->cd.mips.pc_last_host_4k_page,(long long)paddr);
 */
 	/*
 	 *  If the chunk space is all used up, we need to start over from
 	 *  an empty chunk space.
 	 */
-	if (translation_code_chunk_space_head >= CODE_CHUNK_SPACE_SIZE) {
+	if (cpu->mem->translation_code_chunk_space_head >=
+	    cpu->machine->bintrans_size) {
 		int i, n = 1 << BINTRANS_CACHE_N_INDEX_BITS;
 		for (i=0; i<n; i++)
-			translation_page_entry_array[i] = NULL;
-		translation_code_chunk_space_head = 0;
-		n_quick_jumps = 0;
+			cpu->mem->translation_page_entry_array[i] = NULL;
+		cpu->mem->translation_code_chunk_space_head = 0;
+		cpu->mem->n_quick_jumps = 0;
 		tep = NULL;
 		debug("bintrans: Starting over!\n");
 		clear_all_chunks_from_all_tables(cpu);
 	}
 
 
-	host_mips_page = cpu->pc_bintrans_host_4kpage;
+	host_mips_page = cpu->cd.mips.pc_bintrans_host_4kpage;
 	if (host_mips_page == NULL)
-		return cpu->bintrans_instructions_executed;
+		return cpu->cd.mips.bintrans_instructions_executed;
 
 
 	if (tep == NULL) {
 		/*  Allocate a new translation page entry:  */
-		tep = (void *)(size_t) (translation_code_chunk_space +
-		    translation_code_chunk_space_head);
-		translation_code_chunk_space_head += sizeof(struct translation_page_entry);
+		tep = (void *)(size_t) (cpu->mem->translation_code_chunk_space +
+		    cpu->mem->translation_code_chunk_space_head);
+		cpu->mem->translation_code_chunk_space_head += sizeof(struct translation_page_entry);
 
 		/*  ... and align again:  */
-		translation_code_chunk_space_head =
-		    ((translation_code_chunk_space_head - 1) | 63) + 1;
+		cpu->mem->translation_code_chunk_space_head =
+		    ((cpu->mem->translation_code_chunk_space_head - 1) | 63) + 1;
 
 		/*  Add the entry to the array:  */
 		memset(tep, 0, sizeof(struct translation_page_entry));
-		tep->next = translation_page_entry_array[entry_index];
-		translation_page_entry_array[entry_index] = tep;
+		tep->next = cpu->mem->translation_page_entry_array[entry_index];
+		cpu->mem->translation_page_entry_array[entry_index] = tep;
 		tep->paddr = paddr_page;
 	}
 
 	/*  printf("translation_page_entry_array[%i] = %p, ofs = %i\n",
-	    entry_index, translation_page_entry_array[entry_index], offset_within_page);  */
+	    entry_index, cpu->mem->translation_page_entry_array[entry_index], offset_within_page);  */
 
 	/*  ca is the "chunk address"; where to start generating a chunk:  */
-	ca = translation_code_chunk_space
-	    + translation_code_chunk_space_head;
+	ca = cpu->mem->translation_code_chunk_space
+	    + cpu->mem->translation_code_chunk_space_head;
 
 
 	/*
@@ -434,7 +394,7 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 	stop_after_delayed_branch = 0;
 	delayed_branch_new_p = 0;
 	rt = 0;
-	n_quick_jumps = quick_jumps_index = 0;
+	cpu->mem->n_quick_jumps = cpu->mem->quick_jumps_index = 0;
 
 	while (try_to_translate) {
 		ca_justdid = ca;
@@ -473,10 +433,16 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 				break;
 			case SPECIAL_SYSCALL:
 			case SPECIAL_BREAK:
-				translated = bintrans_write_instruction__tlb_rfe_etc(&ca,
-				    special6 == SPECIAL_BREAK? TLB_BREAK : TLB_SYSCALL);
-				n_translated += translated;
- 				try_to_translate = 0;
+				if (cpu->machine->userland_emul != NULL) {
+					bintrans_write_chunkreturn_fail(&ca);
+					tep->flags[prev_p] |= UNTRANSLATABLE;
+					try_to_translate = 0;
+				} else {
+					translated = bintrans_write_instruction__tlb_rfe_etc(&ca,
+					    special6 == SPECIAL_BREAK? CALL_BREAK : CALL_SYSCALL);
+					n_translated += translated;
+	 				try_to_translate = 0;
+				}
 				break;
 			case SPECIAL_ADDU:
 			case SPECIAL_DADDU:
@@ -615,53 +581,53 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 		case HI6_COP0:
 			if (instr[3] == 0x42 && instr[2] == 0x00 && instr[1] == 0x00 && instr[0] == 0x10) {
 				/*  rfe:  */
-				translated = bintrans_write_instruction__tlb_rfe_etc(&ca, TLB_RFE);
+				translated = bintrans_write_instruction__tlb_rfe_etc(&ca, CALL_RFE);
 				n_translated += translated;
  				try_to_translate = 0;
 			} else if (instr[3] == 0x42 && instr[2] == 0x00 && instr[1] == 0x00 && instr[0] == 0x18) {
 				/*  eret:  */
-				translated = bintrans_write_instruction__tlb_rfe_etc(&ca, TLB_ERET);
+				translated = bintrans_write_instruction__tlb_rfe_etc(&ca, CALL_ERET);
 				n_translated += translated;
  				try_to_translate = 0;
 			} else if (instr[3] == 0x40 && (instr[2] & 0xe0)==0 && (instr[1]&7)==0 && instr[0]==0) {
 				/*  mfc0:  */
 				rt = instr[2] & 31;
 				rd = (instr[1] >> 3) & 31;
-				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(&ca, 0, 0, rt, rd, 0);
+				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(cpu->mem, &ca, 0, 0, rt, rd, 0);
 				n_translated += translated;
 			} else if (instr[3] == 0x40 && (instr[2] & 0xe0)==0x20 && (instr[1]&7)==0 && instr[0]==0) {
 				/*  dmfc0:  */
 				rt = instr[2] & 31;
 				rd = (instr[1] >> 3) & 31;
-				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(&ca, 0, 1, rt, rd, 0);
+				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(cpu->mem, &ca, 0, 1, rt, rd, 0);
 				n_translated += translated;
 			} else if (instr[3] == 0x40 && (instr[2] & 0xe0)==0x80 && (instr[1]&7)==0 && instr[0]==0) {
 				/*  mtc0:  */
 				rt = instr[2] & 31;
 				rd = (instr[1] >> 3) & 31;
-				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(&ca, 0, 0, rt, rd, 1);
+				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(cpu->mem, &ca, 0, 0, rt, rd, 1);
 				n_translated += translated;
 			} else if (instr[3] == 0x40 && (instr[2] & 0xe0)==0xa0 && (instr[1]&7)==0 && instr[0]==0) {
 				/*  dmtc0:  */
 				rt = instr[2] & 31;
 				rd = (instr[1] >> 3) & 31;
-				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(&ca, 0, 1, rt, rd, 1);
+				translated = try_to_translate = bintrans_write_instruction__mfc_mtc(cpu->mem, &ca, 0, 1, rt, rd, 1);
 				n_translated += translated;
 			} else if (instr[3] == 0x42 && instr[2] == 0 && instr[1] == 0 && instr[0] == 2) {
 				/*  tlbwi:  */
-				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, TLB_TLBWI);
+				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, CALL_TLBWI);
 				n_translated += translated;
 			} else if (instr[3] == 0x42 && instr[2] == 0 && instr[1] == 0 && instr[0] == 6) {
 				/*  tlbwr:  */
-				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, TLB_TLBWR);
+				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, CALL_TLBWR);
 				n_translated += translated;
 			} else if (instr[3] == 0x42 && instr[2] == 0 && instr[1] == 0 && instr[0] == 8) {
 				/*  tlbp:  */
-				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, TLB_TLBP);
+				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, CALL_TLBP);
 				n_translated += translated;
 			} else if (instr[3] == 0x42 && instr[2] == 0 && instr[1] == 0 && instr[0] == 1) {
 				/*  tlbr:  */
-				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, TLB_TLBR);
+				translated = try_to_translate = bintrans_write_instruction__tlb_rfe_etc(&ca, CALL_TLBR);
 				n_translated += translated;
 			} else
 				try_to_translate = 0;
@@ -693,7 +659,7 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 			imm = (instr[1] << 8) + instr[0];
 			if (imm >= 32768)
 				imm -= 65536;
-			translated = try_to_translate = bintrans_write_instruction__loadstore(&ca, rt, imm, rs, hi6, byte_order_cached_bigendian);
+			translated = try_to_translate = bintrans_write_instruction__loadstore(cpu->mem, &ca, rt, imm, rs, hi6, byte_order_cached_bigendian);
 			n_translated += translated;
 			break;
 
@@ -729,7 +695,8 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 
 				forward = delayed_branch_new_p > p;
 
-				bintrans_write_instruction__delayedbranch(&ca,
+				bintrans_write_instruction__delayedbranch(
+				    cpu->mem, &ca,
 				    potential_chunk_p, &tep->chunk[0], 0,
 				    delayed_branch_new_p & 0xfff, forward);
 
@@ -744,14 +711,14 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 			if (tep->chunk[prev_p] == 0)
 				tep->chunk[prev_p] = (uint32_t)
 				    ((size_t)ca_justdid -
-				    (size_t)translation_code_chunk_space);
+				    (size_t)cpu->mem->translation_code_chunk_space);
 
 			/*  Quickjump to the translated instruction from some
 			    previous instruction?  */
-			for (i=0; i<n_quick_jumps; i++)
-				if (quick_jump_page_offset[i] == p)
-					bintrans_write_quickjump(
-					    quick_jump_host_address[i],
+			for (i=0; i<cpu->mem->n_quick_jumps; i++)
+				if (cpu->mem->quick_jump_page_offset[i] == p)
+					bintrans_write_quickjump(cpu->mem,
+					    cpu->mem->quick_jump_host_address[i],
 					    tep->chunk[prev_p]);
 		}
 
@@ -759,14 +726,14 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 		if (translated && try_to_translate &&
 		    prev_p < 1023 && tep->chunk[prev_p+1] != 0
 		    && !delayed_branch) {
-			bintrans_write_instruction__delayedbranch(
+			bintrans_write_instruction__delayedbranch(cpu->mem,
 			    &ca, &tep->chunk[prev_p+1], NULL, 1, p+4, 1);
 			try_to_translate = 0;
 		}
 
 		if (translated && try_to_translate && n_translated > 80
 		    && prev_p < 1023 && !delayed_branch) {
-			bintrans_write_instruction__delayedbranch(
+			bintrans_write_instruction__delayedbranch(cpu->mem,
 			    &ca, &tep->chunk[prev_p+1], NULL, 1, p+4, 1);
 			try_to_translate = 0;
 		}
@@ -788,12 +755,12 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 	/*  Not enough translated? Then abort.  */
 	if (n_translated < 1) {
 		tep->flags[offset_within_page] |= UNTRANSLATABLE;
-		return cpu->bintrans_instructions_executed;
+		return cpu->cd.mips.bintrans_instructions_executed;
 	}
 
 	/*  ca2 = ptr to the head of the new code chunk  */
-	ca2 = translation_code_chunk_space +
-	    translation_code_chunk_space_head;
+	ca2 = cpu->mem->translation_code_chunk_space +
+	    cpu->mem->translation_code_chunk_space_head;
 
 	/*  Add return code:  */
 	bintrans_write_chunkreturn(&ca);
@@ -804,11 +771,11 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 	/*  Invalidate the host's instruction cache, if necessary:  */
 	bintrans_host_cacheinvalidate(ca2, chunk_len);
 
-	translation_code_chunk_space_head += chunk_len;
+	cpu->mem->translation_code_chunk_space_head += chunk_len;
 
 	/*  Align the code chunk space:  */
-	translation_code_chunk_space_head =
-	    ((translation_code_chunk_space_head - 1) | 63) + 1;
+	cpu->mem->translation_code_chunk_space_head =
+	    ((cpu->mem->translation_code_chunk_space_head - 1) | 63) + 1;
 
 
 	/*  RUN the code chunk:  */
@@ -816,30 +783,30 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 
 run_it:
 	/*  printf("BEFORE: pc=%016llx r31=%016llx\n",
-	    (long long)cpu->pc, (long long)cpu->gpr[31]); */
+	    (long long)cpu->cd.mips.pc, (long long)cpu->gpr[31]); */
 
-	enter_chunks_into_tables(cpu, cpu->pc, &tep->chunk[0]);
+	enter_chunks_into_tables(cpu, cpu->cd.mips.pc, &tep->chunk[0]);
 
-	old_n_executed = cpu->bintrans_instructions_executed;
+	old_n_executed = cpu->cd.mips.bintrans_instructions_executed;
 
 	bintrans_runchunk(cpu, f);
 
 	/*  printf("AFTER:  pc=%016llx r31=%016llx\n",
-	    (long long)cpu->pc, (long long)cpu->gpr[31]);  */
+	    (long long)cpu->cd.mips.pc, (long long)cpu->gpr[31]);  */
 
-	if (!cpu->delay_slot && !cpu->nullify_next &&
-	    cpu->bintrans_instructions_executed < N_SAFE_BINTRANS_LIMIT
-	    && (cpu->pc & 3) == 0
-	    && cpu->bintrans_instructions_executed != old_n_executed) {
+	if (!cpu->cd.mips.delay_slot && !cpu->cd.mips.nullify_next &&
+	    cpu->cd.mips.bintrans_instructions_executed < N_SAFE_BINTRANS_LIMIT
+	    && (cpu->cd.mips.pc & 3) == 0
+	    && cpu->cd.mips.bintrans_instructions_executed != old_n_executed) {
 		int ok = 0, a, b;
 		struct vth32_table *tbl1;
 
-		if (bintrans_32bit_only ||
-		    (cpu->pc & 0xffffffff80000000ULL) == 0 ||
-		    (cpu->pc & 0xffffffff80000000ULL) == 0xffffffff80000000ULL) {
+		if (cpu->mem->bintrans_32bit_only ||
+		    (cpu->cd.mips.pc & 0xffffffff80000000ULL) == 0 ||
+		    (cpu->cd.mips.pc & 0xffffffff80000000ULL) == 0xffffffff80000000ULL) {
 			/*  32-bit special case:  */
-			a = (cpu->pc >> 22) & 0x3ff;
-			b = (cpu->pc >> 12) & 0x3ff;
+			a = (cpu->cd.mips.pc >> 22) & 0x3ff;
+			b = (cpu->cd.mips.pc >> 12) & 0x3ff;
 
 			/*  TODO: There is a bug here; if caches are disabled, and
 			    for some reason the code jumps to a different page, then
@@ -847,11 +814,11 @@ run_it:
 			    to check for cache isolation, and if so, use the
 			    kernel table. Otherwise use table0.  */
 
-			/*  tbl1 = cpu->vaddr_to_hostaddr_table0_kernel[a];  */
+			/*  tbl1 = cpu->cd.mips.vaddr_to_hostaddr_table0_kernel[a];  */
 
-			tbl1 = cpu->vaddr_to_hostaddr_table0[a];
+			tbl1 = cpu->cd.mips.vaddr_to_hostaddr_table0[a];
 			if (tbl1->haddr_entry[b] != NULL) {
-				paddr = tbl1->paddr_entry[b] | (cpu->pc & 0xfff);
+				paddr = tbl1->paddr_entry[b] | (cpu->cd.mips.pc & 0xfff);
 				ok = 1;
 			}
 		}
@@ -859,16 +826,16 @@ run_it:
 		/*  General case, or if the special case above failed:  */
 		/*  (This may cause exceptions.)  */
 		if (!ok) {
-			uint64_t old_pc = cpu->pc_last = cpu->pc;
-			ok = cpu->translate_address(cpu, cpu->pc, &paddr, FLAG_INSTR);
+			uint64_t old_pc = cpu->cd.mips.pc_last = cpu->cd.mips.pc;
+			ok = cpu->translate_address(cpu, cpu->cd.mips.pc, &paddr, FLAG_INSTR);
 
-			if (!ok && old_pc != cpu->pc) {
+			if (!ok && old_pc != cpu->cd.mips.pc) {
 				/*  pc is something like ...0080 or ...0000 or so.  */
-				paddr = cpu->pc & 0xfff;
+				paddr = cpu->cd.mips.pc & 0xfff;
 				ok = 1;
 
-				cpu->pc_last_host_4k_page = NULL;
-				cpu->pc_bintrans_host_4kpage = NULL;
+				cpu->cd.mips.pc_last_host_4k_page = NULL;
+				cpu->cd.mips.pc_bintrans_host_4kpage = NULL;
 			}
 		}
 
@@ -876,16 +843,16 @@ run_it:
 			paddr_page = paddr & ~0xfff;
 			offset_within_page = (paddr & 0xfff) / 4;
 			entry_index = PADDR_TO_INDEX(paddr);
-			tep = translation_page_entry_array[entry_index];
+			tep = cpu->mem->translation_page_entry_array[entry_index];
 			while (tep != NULL) {
 				if (tep->paddr == paddr_page) {
 					if (tep->chunk[offset_within_page] != 0) {
 						f = (size_t)tep->chunk[offset_within_page] +
-						    translation_code_chunk_space;
+						    cpu->mem->translation_code_chunk_space;
 						goto run_it;
 					}
 					if (tep->flags[offset_within_page] & UNTRANSLATABLE)
-						return cpu->bintrans_instructions_executed;
+						return cpu->cd.mips.bintrans_instructions_executed;
 					break;
 				}
 				tep = tep->next;
@@ -893,38 +860,38 @@ run_it:
 
 #if 1
 			/*  We have no translation.  */
-			if ((cpu->pc & 0xfff00000) == 0xbfc00000 &&
-			    cpu->emul->prom_emulation)
-				return cpu->bintrans_instructions_executed;
+			if ((cpu->cd.mips.pc & 0xfff00000) == 0xbfc00000 &&
+			    cpu->machine->prom_emulation)
+				return cpu->cd.mips.bintrans_instructions_executed;
 
 			/*  This special hack might make the time spent
 			    in the main cpu_run_instr() lower:  */
-			if (bintrans_32bit_only) {
-			/*  TODO: This doesn't seem to work with R4000 etc?
-			    || (cpu->pc & 0xffffffff80000000ULL) == 0 ||
-			    (cpu->pc & 0xffffffff80000000ULL) == 0xffffffff80000000ULL) {  */
+			/*  TODO: This doesn't seem to work with R4000 etc?  */
+			if (cpu->mem->bintrans_32bit_only) {
+			    /* || (cpu->cd.mips.pc & 0xffffffff80000000ULL) == 0 ||
+			    (cpu->cd.mips.pc & 0xffffffff80000000ULL) == 0xffffffff80000000ULL) {  */
 				int ok = 1;
 				/*  32-bit special case:  */
-				a = (cpu->pc >> 22) & 0x3ff;
-				b = (cpu->pc >> 12) & 0x3ff;
-				if (cpu->vaddr_to_hostaddr_table0 !=
-				    cpu->vaddr_to_hostaddr_table0_kernel)
+				a = (cpu->cd.mips.pc >> 22) & 0x3ff;
+				b = (cpu->cd.mips.pc >> 12) & 0x3ff;
+				if (cpu->cd.mips.vaddr_to_hostaddr_table0 !=
+				    cpu->cd.mips.vaddr_to_hostaddr_table0_kernel)
 					ok = 0;
-				tbl1 = cpu->vaddr_to_hostaddr_table0_kernel[a];
+				tbl1 = cpu->cd.mips.vaddr_to_hostaddr_table0_kernel[a];
 				if (ok && tbl1->haddr_entry[b] != NULL) {
-					cpu->pc_last_virtual_page = cpu->pc & ~0xfff;
-					cpu->pc_last_physical_page = paddr & ~0xfff;
-					cpu->pc_last_host_4k_page = (unsigned char *)
+					cpu->cd.mips.pc_last_virtual_page = cpu->cd.mips.pc & ~0xfff;
+					cpu->cd.mips.pc_last_physical_page = paddr & ~0xfff;
+					cpu->cd.mips.pc_last_host_4k_page = (unsigned char *)
 					    (((size_t)tbl1->haddr_entry[b]) & ~1);
-					cpu->pc_bintrans_host_4kpage = cpu->pc_last_host_4k_page;
-					cpu->pc_bintrans_paddr = paddr;
+					cpu->cd.mips.pc_bintrans_host_4kpage = cpu->cd.mips.pc_last_host_4k_page;
+					cpu->cd.mips.pc_bintrans_paddr = paddr;
 
 /*
 printf("C: ");
 printf("v=%016llx p=%016llx h=%p paddr=%016llx\n",
-(long long)cpu->pc_last_virtual_page,
-(long long)cpu->pc_last_physical_page,
-cpu->pc_last_host_4k_page,(long long)paddr);
+(long long)cpu->cd.mips.pc_last_virtual_page,
+(long long)cpu->cd.mips.pc_last_physical_page,
+cpu->cd.mips.pc_last_host_4k_page,(long long)paddr);
 */
 					goto quick_attempt_translate_again;
 				}
@@ -935,7 +902,7 @@ cpu->pc_last_host_4k_page,(long long)paddr);
 		}
 	}
 
-	return cpu->bintrans_instructions_executed;
+	return cpu->cd.mips.bintrans_instructions_executed;
 }
 
 
@@ -950,59 +917,59 @@ void bintrans_init_cpu(struct cpu *cpu)
 {
 	int i, offset;
 
-	cpu->chunk_base_address        = translation_code_chunk_space;
-	cpu->bintrans_loadstore_32bit  = bintrans_loadstore_32bit;
-	cpu->bintrans_jump_to_32bit_pc = bintrans_jump_to_32bit_pc;
-	cpu->bintrans_fast_tlbwri      = coproc_tlbwri;
-	cpu->bintrans_fast_tlbpr       = coproc_tlbpr;
-	cpu->bintrans_fast_rfe         = coproc_rfe;
-	cpu->bintrans_fast_eret        = coproc_eret;
-	cpu->bintrans_simple_exception = cpu_cause_simple_exception;
-	cpu->fast_vaddr_to_hostaddr    = fast_vaddr_to_hostaddr;
+	cpu->cd.mips.chunk_base_address        = cpu->mem->translation_code_chunk_space;
+	cpu->cd.mips.bintrans_loadstore_32bit  = bintrans_loadstore_32bit;
+	cpu->cd.mips.bintrans_jump_to_32bit_pc = bintrans_jump_to_32bit_pc;
+	cpu->cd.mips.bintrans_fast_tlbwri      = coproc_tlbwri;
+	cpu->cd.mips.bintrans_fast_tlbpr       = coproc_tlbpr;
+	cpu->cd.mips.bintrans_fast_rfe         = coproc_rfe;
+	cpu->cd.mips.bintrans_fast_eret        = coproc_eret;
+	cpu->cd.mips.bintrans_simple_exception = mips_cpu_cause_simple_exception;
+	cpu->cd.mips.fast_vaddr_to_hostaddr    = fast_vaddr_to_hostaddr;
 
 	/*  Initialize vaddr->hostaddr translation tables:  */
-	cpu->vaddr_to_hostaddr_nulltable =
+	cpu->cd.mips.vaddr_to_hostaddr_nulltable =
 	    zeroed_alloc(sizeof(struct vth32_table));
 
 	/*  Data cache:  */
 	offset = 0;
-	cpu->vaddr_to_hostaddr_r2k3k_dcachetable =
+	cpu->cd.mips.vaddr_to_hostaddr_r2k3k_dcachetable =
 	    zeroed_alloc(sizeof(struct vth32_table));
 	for (i=0; i<1024; i++) {
-		cpu->vaddr_to_hostaddr_r2k3k_dcachetable->haddr_entry[i] =
-		    (void *)(((size_t)cpu->cache[0]+offset) | 1);
-		offset = (offset + 4096) % cpu->cache_size[0];
+		cpu->cd.mips.vaddr_to_hostaddr_r2k3k_dcachetable->haddr_entry[i] =
+		    (void *)(((size_t)cpu->cd.mips.cache[0]+offset) | 1);
+		offset = (offset + 4096) % cpu->cd.mips.cache_size[0];
 	}
-	cpu->vaddr_to_hostaddr_r2k3k_dcachetable->refcount = 1024;
+	cpu->cd.mips.vaddr_to_hostaddr_r2k3k_dcachetable->refcount = 1024;
 
 	/*  Instruction cache:  */
 	offset = 0;
-	cpu->vaddr_to_hostaddr_r2k3k_icachetable =
+	cpu->cd.mips.vaddr_to_hostaddr_r2k3k_icachetable =
 	    zeroed_alloc(sizeof(struct vth32_table));
 	for (i=0; i<1024; i++) {
-		cpu->vaddr_to_hostaddr_r2k3k_icachetable->haddr_entry[i] =
-		    (void *)(((size_t)cpu->cache[1]+offset) | 1);
-		offset = (offset + 4096) % cpu->cache_size[1];
+		cpu->cd.mips.vaddr_to_hostaddr_r2k3k_icachetable->haddr_entry[i] =
+		    (void *)(((size_t)cpu->cd.mips.cache[1]+offset) | 1);
+		offset = (offset + 4096) % cpu->cd.mips.cache_size[1];
 	}
-	cpu->vaddr_to_hostaddr_r2k3k_icachetable->refcount = 1024;
+	cpu->cd.mips.vaddr_to_hostaddr_r2k3k_icachetable->refcount = 1024;
 
-	cpu->vaddr_to_hostaddr_table0_kernel =
+	cpu->cd.mips.vaddr_to_hostaddr_table0_kernel =
 	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
-	cpu->vaddr_to_hostaddr_table0_user =
+	cpu->cd.mips.vaddr_to_hostaddr_table0_user =
 	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
-	cpu->vaddr_to_hostaddr_table0_cacheisol_i =
+	cpu->cd.mips.vaddr_to_hostaddr_table0_cacheisol_i =
 	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
-	cpu->vaddr_to_hostaddr_table0_cacheisol_d =
+	cpu->cd.mips.vaddr_to_hostaddr_table0_cacheisol_d =
 	    zeroed_alloc(1024 * sizeof(struct vth32_table *));
 
 	for (i=0; i<1024; i++) {
-		cpu->vaddr_to_hostaddr_table0_kernel[i] = cpu->vaddr_to_hostaddr_nulltable;
-		cpu->vaddr_to_hostaddr_table0_user[i] = cpu->vaddr_to_hostaddr_nulltable;
-		cpu->vaddr_to_hostaddr_table0_cacheisol_i[i] = cpu->vaddr_to_hostaddr_r2k3k_icachetable;
-		cpu->vaddr_to_hostaddr_table0_cacheisol_d[i] = cpu->vaddr_to_hostaddr_r2k3k_dcachetable;
+		cpu->cd.mips.vaddr_to_hostaddr_table0_kernel[i] = cpu->cd.mips.vaddr_to_hostaddr_nulltable;
+		cpu->cd.mips.vaddr_to_hostaddr_table0_user[i] = cpu->cd.mips.vaddr_to_hostaddr_nulltable;
+		cpu->cd.mips.vaddr_to_hostaddr_table0_cacheisol_i[i] = cpu->cd.mips.vaddr_to_hostaddr_r2k3k_icachetable;
+		cpu->cd.mips.vaddr_to_hostaddr_table0_cacheisol_d[i] = cpu->cd.mips.vaddr_to_hostaddr_r2k3k_dcachetable;
 	}
 
-	cpu->vaddr_to_hostaddr_table0 = cpu->vaddr_to_hostaddr_table0_kernel;
+	cpu->cd.mips.vaddr_to_hostaddr_table0 = cpu->cd.mips.vaddr_to_hostaddr_table0_kernel;
 }
 
 
@@ -1011,36 +978,42 @@ void bintrans_init_cpu(struct cpu *cpu)
  *
  *  Should be called before any other bintrans_*() function is used.
  */
-void bintrans_init(void)
+void bintrans_init(struct machine *machine, struct memory *mem)
 {
 	int res, i, n = 1 << BINTRANS_CACHE_N_INDEX_BITS;
 	size_t s;
 
-	debug("bintrans: EXPERIMENTAL!\n");
+	mem->translation_page_entry_array = malloc(sizeof(
+	    struct translation_page_entry *) *
+	    (1 << BINTRANS_CACHE_N_INDEX_BITS));
+	if (mem->translation_page_entry_array == NULL) {
+		fprintf(stderr, "bintrans_init(): out of memory\n");
+		exit(1);
+	}
 
 	/*
 	 *  The entry array must be NULLed, as these are pointers to
 	 *  translation page entries.
 	 */
 	for (i=0; i<n; i++)
-		translation_page_entry_array[i] = NULL;
+		mem->translation_page_entry_array[i] = NULL;
 
 	/*  Allocate the large code chunk space:  */
-	s = CODE_CHUNK_SPACE_SIZE + CODE_CHUNK_SPACE_MARGIN;
-	translation_code_chunk_space = (unsigned char *) mmap(NULL, s,
+	s = machine->bintrans_size + CODE_CHUNK_SPACE_MARGIN;
+	mem->translation_code_chunk_space = (unsigned char *) mmap(NULL, s,
 	    PROT_READ | PROT_WRITE | PROT_EXEC, MAP_ANON | MAP_PRIVATE, -1, 0);
 
 	/*  If mmap() failed, try malloc():  */
-	if (translation_code_chunk_space == NULL) {
-		translation_code_chunk_space = malloc(s);
-		if (translation_code_chunk_space == NULL) {
+	if (mem->translation_code_chunk_space == NULL) {
+		mem->translation_code_chunk_space = malloc(s);
+		if (mem->translation_code_chunk_space == NULL) {
 			fprintf(stderr, "bintrans_init(): out of memory (2)\n");
 			exit(1);
 		}
 	}
 
-	debug("bintrans: translation_code_chunk_space = %i MB at %p\n",
-	    (int)(s/1048576), translation_code_chunk_space);
+	debug("bintrans: "BACKEND_NAME", %i MB translation cache at %p\n",
+	    (int)(s/1048576), mem->translation_code_chunk_space);
 
 	/*
 	 *  The translation_code_chunk_space does not need to be zeroed,
@@ -1048,7 +1021,7 @@ void bintrans_init(void)
 	 *  add new chunks must be initialized to the beginning of the
 	 *  chunk space.
 	 */
-	translation_code_chunk_space_head = 0;
+	mem->translation_code_chunk_space_head = 0;
 
 	/*
 	 *  Some operating systems (for example OpenBSD using the default
@@ -1061,11 +1034,13 @@ void bintrans_init(void)
 	 *  of memory obtained from mmap(2).".  If malloc() isn't implemented
 	 *  using mmap(), then this could be a problem.
 	 */
-	res = mprotect((void *)translation_code_chunk_space,
+	res = mprotect((void *)mem->translation_code_chunk_space,
 	    s, PROT_READ | PROT_WRITE | PROT_EXEC);
 	if (res)
 		debug("warning: mprotect() failed with errno %i."
 		    " this usually doesn't really matter...\n", errno);
+
+	bintrans_backend_init();
 }
 
 #endif	/*  BINTRANS  */
