@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bintrans_alpha.c,v 1.104 2005/01/10 22:30:42 debug Exp $
+ *  $Id: bintrans_alpha.c,v 1.112 2005/02/02 20:42:40 debug Exp $
  *
  *  Alpha specific code for dynamic binary translation.
  *
@@ -116,7 +116,7 @@ static int map_MIPS_to_Alpha[32] = {
 
 
 struct cpu dummy_cpu;
-struct coproc dummy_coproc;
+struct mips_coproc dummy_coproc;
 struct vth32_table dummy_vth32_table;
 
 unsigned char bintrans_alpha_imb[32] = {
@@ -176,165 +176,24 @@ static void bintrans_host_cacheinvalidate(unsigned char *p, size_t len)
  *  lda sp,128(sp)
  *  ret
  */
-#define ofs_pc	(((size_t)&dummy_cpu.pc) - ((size_t)&dummy_cpu))
-#define ofs_pc_last	(((size_t)&dummy_cpu.pc_last) - ((size_t)&dummy_cpu))
-#define ofs_n	(((size_t)&dummy_cpu.bintrans_instructions_executed) - ((size_t)&dummy_cpu))
-#define ofs_ds	(((size_t)&dummy_cpu.delay_slot) - ((size_t)&dummy_cpu))
-#define ofs_ja	(((size_t)&dummy_cpu.delay_jmpaddr) - ((size_t)&dummy_cpu))
-#define ofs_sp	(((size_t)&dummy_cpu.gpr[GPR_SP]) - ((size_t)&dummy_cpu))
-#define ofs_ra	(((size_t)&dummy_cpu.gpr[GPR_RA]) - ((size_t)&dummy_cpu))
-#define ofs_a0	(((size_t)&dummy_cpu.gpr[GPR_A0]) - ((size_t)&dummy_cpu))
-#define ofs_a1	(((size_t)&dummy_cpu.gpr[GPR_A1]) - ((size_t)&dummy_cpu))
-#define ofs_t0	(((size_t)&dummy_cpu.gpr[GPR_T0]) - ((size_t)&dummy_cpu))
-#define ofs_t1	(((size_t)&dummy_cpu.gpr[GPR_T1]) - ((size_t)&dummy_cpu))
-#define ofs_t2	(((size_t)&dummy_cpu.gpr[GPR_T2]) - ((size_t)&dummy_cpu))
-#define ofs_v0	(((size_t)&dummy_cpu.gpr[GPR_V0]) - ((size_t)&dummy_cpu))
-#define ofs_s0	(((size_t)&dummy_cpu.gpr[GPR_S0]) - ((size_t)&dummy_cpu))
-#define ofs_tbl0 (((size_t)&dummy_cpu.vaddr_to_hostaddr_table0) - ((size_t)&dummy_cpu))
-static unsigned char bintrans_alpha_runchunk[200] = {
-	0x80, 0xff, 0xde, 0x23,		/*  lda     sp,-128(sp)  */
-	0x00, 0x00, 0x5e, 0xb7,		/*  stq     ra,0(sp)  */
-	0x08, 0x00, 0x3e, 0xb5,		/*  stq     s0,8(sp)  */
-	0x10, 0x00, 0x5e, 0xb5,		/*  stq     s1,16(sp)  */
-	0x18, 0x00, 0x7e, 0xb5,		/*  stq     s2,24(sp)  */
-	0x20, 0x00, 0x9e, 0xb5,		/*  stq     s3,32(sp)  */
-	0x28, 0x00, 0xbe, 0xb5,		/*  stq     s4,40(sp)  */
-	0x30, 0x00, 0xde, 0xb5,		/*  stq     s5,48(sp)  */
-	0x38, 0x00, 0xfe, 0xb5,		/*  stq     s6,56(sp)  */
-	0x78, 0x00, 0xbe, 0xb7,		/*  stq     gp,120(sp)  */
-
-	ofs_pc&255,ofs_pc>>8,0xd0,0xa4,	/*  ldq     t5,"pc"(a0)  */
-	ofs_n&255,ofs_n>>8,0xf0,0xa0,	/*  ldl     t6,"bintrans_instructions_executed"(a0)  */
-	ofs_a0&255,ofs_a0>>8,0x10,0xa5,	/*  ldq     t7,"a0"(a0)  */
-	ofs_a1&255,ofs_a1>>8,0xd0,0xa6,	/*  ldq     t8,"a1"(a0)  */
-	ofs_s0&255,ofs_s0>>8,0xf0,0xa6,	/*  ldq     t9,"s0"(a0)  */
-	ofs_ds&255,ofs_ds>>8,0x30,0xa1,	/*  ldl     s0,"delay_slot"(a0)  */
-	ofs_ja&255,ofs_ja>>8,0x50,0xa5,	/*  ldq     s1,"delay_jmpaddr"(a0)  */
-	ofs_sp&255,ofs_sp>>8,0x70,0xa5,	/*  ldq     s2,"gpr[sp]"(a0)  */
-	ofs_ra&255,ofs_ra>>8,0x90,0xa5,	/*  ldq     s3,"gpr[ra]"(a0)  */
-	ofs_t0&255,ofs_t0>>8,0xb0,0xa5,	/*  ldq     s4,"gpr[t0]"(a0)  */
-	ofs_t1&255,ofs_t1>>8,0xd0,0xa5,	/*  ldq     s5,"gpr[t1]"(a0)  */
-	ofs_t2&255,ofs_t2>>8,0xf0,0xa5,	/*  ldq     s6,"gpr[t2]"(a0)  */
-	ofs_tbl0&255,ofs_tbl0>>8,0x10,0xa7,/*  ldq     t10,table0(a0)  */
-	ofs_v0&255,ofs_v0>>8,0x30,0xa7,	/*  ldq     t11,"gpr[v0]"(a0)  */
-
-	0x00, 0x40, 0x51, 0x6b,		/*  jsr     ra,(a1),<back>  */
-
-	ofs_pc&255,ofs_pc>>8,0xd0,0xb4,	/*  stq     t5,"pc"(a0)  */
-	ofs_n&255,ofs_n>>8,0xf0,0xb0,	/*  stl     t6,"bintrans_instructions_executed"(a0)  */
-	ofs_a0&255,ofs_a0>>8,0x10,0xb5,	/*  stq     t7,"a0"(a0)  */
-	ofs_a1&255,ofs_a1>>8,0xd0,0xb6,	/*  stq     t8,"a1"(a0)  */
-	ofs_s0&255,ofs_s0>>8,0xf0,0xb6,	/*  stq     t9,"s0"(a0)  */
-	ofs_ds&255,ofs_ds>>8,0x30,0xb1,	/*  stl     s0,"delay_slot"(a0)  */
-	ofs_ja&255,ofs_ja>>8,0x50,0xb5,	/*  stq     s1,"delay_jmpaddr"(a0)  */
-	ofs_sp&255,ofs_sp>>8,0x70,0xb5,	/*  stq     s2,"gpr[sp]"(a0)  */
-	ofs_ra&255,ofs_ra>>8,0x90,0xb5,	/*  stq     s3,"gpr[ra]"(a0)  */
-	ofs_t0&255,ofs_t0>>8,0xb0,0xb5,	/*  stq     s4,"gpr[t0]"(a0)  */
-	ofs_t1&255,ofs_t1>>8,0xd0,0xb5,	/*  stq     s5,"gpr[t1]"(a0)  */
-	ofs_t2&255,ofs_t2>>8,0xf0,0xb5,	/*  stq     s6,"gpr[t2]"(a0)  */
-	ofs_v0&255,ofs_v0>>8,0x30,0xb7,	/*  stq     t11,"gpr[v0]"(a0)  */
-
-	0x00, 0x00, 0x5e, 0xa7,		/*  ldq     ra,0(sp)  */
-	0x08, 0x00, 0x3e, 0xa5,		/*  ldq     s0,8(sp)  */
-	0x10, 0x00, 0x5e, 0xa5,		/*  ldq     s1,16(sp)  */
-	0x18, 0x00, 0x7e, 0xa5,		/*  ldq     s2,24(sp)  */
-	0x20, 0x00, 0x9e, 0xa5,		/*  ldq     s3,32(sp)  */
-	0x28, 0x00, 0xbe, 0xa5,		/*  ldq     s4,40(sp)  */
-	0x30, 0x00, 0xde, 0xa5,		/*  ldq     s5,48(sp)  */
-	0x38, 0x00, 0xfe, 0xa5,		/*  ldq     s6,56(sp)  */
-	0x78, 0x00, 0xbe, 0xa7,		/*  ldq     gp,120(sp)  */
-	0x80, 0x00, 0xde, 0x23,		/*  lda     sp,128(sp)  */
-	0x01, 0x80, 0xfa, 0x6b		/*  ret   */
-};
-
-static unsigned char bintrans_alpha_jump_to_32bit_pc[25 * 4] = {
-	/*  Don't execute too many instructions. (see comment below)  */
-	(N_SAFE_BINTRANS_LIMIT-1)&255, ((N_SAFE_BINTRANS_LIMIT-1) >> 8)&255,
-		0x5f, 0x20,		/*  lda t1,safe limit - 1 */
-	0xa1, 0x0d, 0xe2, 0x40,		/*  cmple t6,t1,t0  */
-	0x01, 0x00, 0x20, 0xf4,		/*  bne  */
-	0x01, 0x80, 0xfa, 0x6b,		/*  ret  */
-
-	0x11, 0x14, 0xc0, 0x40,		/*  addq t5,0,a1  */
-
-	/*
-	 *  Special case for 32-bit addressing:
-	 *
-	 *  t1 = 1023;
-	 *  t2 = ((a1 >> 22) & t1) * sizeof(void *);
-	 *  t3 = ((a1 >> 12) & t1) * sizeof(void *);
-	 *  t1 = a1 & 4095;
-	 *
-	 *  f8 1f 5f 20     lda     t1,1023 * 8
-	 *  83 76 22 4a     srl     a1,19,t2
-	 *  84 36 21 4a     srl     a1, 9,t3
-	 *  03 00 62 44     and     t2,t1,t2
-	 */
-	0xf8, 0x1f, 0x5f, 0x20,
-	0x83, 0x76, 0x22, 0x4a,
-	0x84, 0x36, 0x21, 0x4a,
-	0x03, 0x00, 0x62, 0x44,
-
-	/*
-	 *  t10 is vaddr_to_hostaddr_table0
-	 *
-	 *  a3 = tbl0[t2]  (load entry from tbl0)
-	 *  12 04 03 43     addq    t10,t2,a2
-	 */
-	0x12, 0x04, 0x03, 0x43,
-
-	/*  04 00 82 44     and     t3,t1,t3  */
-	0x04, 0x00, 0x82, 0x44,
-
-	/*  00 00 72 a6     ldq     a3,0(a2)  */
-	0x00, 0x00, 0x72, 0xa6,
-
-	/*  fc 0f 5f 20     lda     t1,0xffc  */
-	0xfc, 0x0f, 0x5f, 0x20,
-
-	/*
-	 *  a3 = tbl1[t3]  (load entry from tbl1 (whic is a3))
-	 *  13 04 64 42     addq    a3,t3,a3
-	 */
-	0x13, 0x04, 0x64, 0x42,
-
-	/*  02 00 22 46     and     a1,t1,t1  */
-	0x02, 0x00, 0x22, 0x46,
-
-	/*  00 00 73 a6     ldq     a3,chunks[0](a3)  */
+#define ofs_pc	(((size_t)&dummy_cpu.cd.mips.pc) - ((size_t)&dummy_cpu))
+#define ofs_pc_last	(((size_t)&dummy_cpu.cd.mips.pc_last) - ((size_t)&dummy_cpu))
+#define ofs_n	(((size_t)&dummy_cpu.cd.mips.bintrans_instructions_executed) - ((size_t)&dummy_cpu))
+#define ofs_ds	(((size_t)&dummy_cpu.cd.mips.delay_slot) - ((size_t)&dummy_cpu))
+#define ofs_ja	(((size_t)&dummy_cpu.cd.mips.delay_jmpaddr) - ((size_t)&dummy_cpu))
+#define ofs_sp	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_SP]) - ((size_t)&dummy_cpu))
+#define ofs_ra	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_RA]) - ((size_t)&dummy_cpu))
+#define ofs_a0	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_A0]) - ((size_t)&dummy_cpu))
+#define ofs_a1	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_A1]) - ((size_t)&dummy_cpu))
+#define ofs_t0	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_T0]) - ((size_t)&dummy_cpu))
+#define ofs_t1	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_T1]) - ((size_t)&dummy_cpu))
+#define ofs_t2	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_T2]) - ((size_t)&dummy_cpu))
+#define ofs_v0	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_V0]) - ((size_t)&dummy_cpu))
+#define ofs_s0	(((size_t)&dummy_cpu.cd.mips.gpr[MIPS_GPR_S0]) - ((size_t)&dummy_cpu))
+#define ofs_tbl0 (((size_t)&dummy_cpu.cd.mips.vaddr_to_hostaddr_table0) - ((size_t)&dummy_cpu))
 #define ofs_c0	((size_t)&dummy_vth32_table.bintrans_chunks[0] - (size_t)&dummy_vth32_table)
-	ofs_c0 & 255, (ofs_c0 >> 8) & 255, 0x73, 0xa6,
+#define ofs_cb (((size_t)&dummy_cpu.cd.mips.chunk_base_address) - (size_t)&dummy_cpu)
 
-	/*
-	 *  NULL? Then just return.
-	 *  01 00 60 f6     bne     a3,f8 <okzz>
-	 */
-	0x01, 0x00, 0x60, 0xf6,
-	0x01, 0x80, 0xfa, 0x6b,		/*  ret  */
-
-	/*
-	 *  02 04 53 40     addq    t1,a3,t1
-	 *  00 00 22 a0     ldl     t0,0(t1)
-	 */
-	0x02, 0x04, 0x53, 0x40,
-	0x00, 0x00, 0x22, 0xa0,
-
-	/*  No translation? Then return.  */
-	0x03, 0x00, 0x20, 0xe4,		/*  beq t0,<skip>  */
-
-	/*  ldq t2,chunk_base_address(a0)  */
-#define ofs_cb (((size_t)&dummy_cpu.chunk_base_address) - (size_t)&dummy_cpu)
-	(ofs_cb & 255), (ofs_cb >> 8) & 255, 0x70, 0xa4,
-
-	/*  addq t0,t2,t0  */
-	0x01, 0x04, 0x23, 0x40,
-
-	/*  00 00 e1 6b     jmp     (t0)  */
-	0x00, 0x00, 0xe1, 0x6b,		/*  jmp (t0)  */
-
-	/*  Return to the main translation loop.  */
-	0x01, 0x80, 0xfa, 0x6b		/*  ret  */
-};
 
 static uint32_t bintrans_alpha_loadstore_32bit[19] = {
 	/*
@@ -408,11 +267,9 @@ static uint32_t bintrans_alpha_loadstore_32bit[19] = {
 	0x6be50000		/*  jmp (t4)  */
 };
 
-static const void (*bintrans_runchunk)
-    (struct cpu *, unsigned char *) = (void *)bintrans_alpha_runchunk;
+static void (*bintrans_runchunk)(struct cpu *, unsigned char *);
 
-static void (*bintrans_jump_to_32bit_pc)
-    (struct cpu *) = (void *)bintrans_alpha_jump_to_32bit_pc;
+static void (*bintrans_jump_to_32bit_pc)(struct cpu *);
 
 static void (*bintrans_loadstore_32bit)
     (struct cpu *) = (void *)bintrans_alpha_loadstore_32bit;
@@ -421,12 +278,12 @@ static void (*bintrans_loadstore_32bit)
 /*
  *  bintrans_write_quickjump():
  */
-static void bintrans_write_quickjump(unsigned char *quickjump_code,
-	uint32_t chunkoffset)
+static void bintrans_write_quickjump(struct memory *mem,
+	unsigned char *quickjump_code, uint32_t chunkoffset)
 {
 	int ofs;
 	uint64_t alpha_addr = chunkoffset +
-	    (size_t)translation_code_chunk_space;
+	    (size_t)mem->translation_code_chunk_space;
 	uint32_t *a = (uint32_t *)quickjump_code;
 
 	ofs = (alpha_addr - ((size_t)a+4)) / 4;
@@ -490,7 +347,7 @@ static void bintrans_move_MIPS_reg_into_Alpha_reg(unsigned char **addrp, int mip
 	default:
 		alpha_mips_reg = map_MIPS_to_Alpha[mipsreg];
 		if (alpha_mips_reg < 0) {
-			ofs = ((size_t)&dummy_cpu.gpr[mipsreg]) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.gpr[mipsreg]) - (size_t)&dummy_cpu;
 			/*  ldq alphareg,gpr[mipsreg](a0)  */
 			*a++ = 0xa4100000 | (alphareg << 21) | ofs;
 		} else {
@@ -529,7 +386,7 @@ static void bintrans_move_Alpha_reg_into_MIPS_reg(unsigned char **addrp, int alp
 		alpha_mips_reg = map_MIPS_to_Alpha[mipsreg];
 		if (alpha_mips_reg < 0) {
 			/*  stq alphareg,gpr[mipsreg](a0)  */
-			ofs = ((size_t)&dummy_cpu.gpr[mipsreg]) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.gpr[mipsreg]) - (size_t)&dummy_cpu;
 			*a++ = 0xb4100000 | (alphareg << 21) | ofs;
 		} else {
 			/*  addq alphareg,0,alpha_mips_reg  */
@@ -893,13 +750,13 @@ static int bintrans_write_instruction__addu_etc(unsigned char **addrp,
 		/*  01 10 60 40     addl    t2,0,t0  */
 		*a++ = 0x01; *a++ = 0x10; *a++ = 0x60; *a++ = 0x40;
 
-		ofs = ((size_t)&dummy_cpu.lo) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.lo) - (size_t)&dummy_cpu;
 		*a++ = (ofs & 255); *a++ = (ofs >> 8); *a++ = 0x30; *a++ = 0xb4;
 
 		/*  81 17 64 48     sra     t2,0x20,t0  */
 		*a++ = 0x81; *a++ = 0x17; *a++ = 0x64; *a++ = 0x48;
 		*a++ = 0x01; *a++ = 0x00; *a++ = 0x3f; *a++ = 0x40;	/*  addl t0,0,t0  */
-		ofs = ((size_t)&dummy_cpu.hi) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.hi) - (size_t)&dummy_cpu;
 		*a++ = (ofs & 255); *a++ = (ofs >> 8); *a++ = 0x30; *a++ = 0xb4;
 		break;
 	case SPECIAL_MOVZ:
@@ -1051,10 +908,10 @@ static int bintrans_write_instruction__branch(unsigned char **addrp,
 		if (b != NULL)
 			*((unsigned char *)b) = ((size_t)a - (size_t)b - 4) / 4;
 
-		/*  cpu->nullify_next = 1;  */
+		/*  cpu->cd.mips.nullify_next = 1;  */
 		/*  01 00 3f 20     lda     t0,1  */
 		*a++ = 0x203f0001;
-		ofs = (size_t)&dummy_cpu.nullify_next - (size_t)&dummy_cpu;
+		ofs = (size_t)&dummy_cpu.cd.mips.nullify_next - (size_t)&dummy_cpu;
 		*a++ = 0xb0300000 | (ofs & 0xffff);
 
 		/*  fail, so that the next instruction is handled manually:  */
@@ -1167,7 +1024,8 @@ static int bintrans_write_instruction__jal(unsigned char **addrp,
 /*
  *  bintrans_write_instruction__delayedbranch():
  */
-static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
+static int bintrans_write_instruction__delayedbranch(
+	struct memory *mem, unsigned char **addrp,
 	uint32_t *potential_chunk_p, uint32_t *chunks,
 	int only_care_about_chunk_p, int p, int forward)
 {
@@ -1195,9 +1053,9 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 	}
 
 	if (potential_chunk_p == NULL) {
-		if (bintrans_32bit_only) {
+		if (mem->bintrans_32bit_only) {
 			/*  34 12 70 a7     ldq     t12,4660(a0)  */
-			ofs = (size_t)&dummy_cpu.bintrans_jump_to_32bit_pc - (size_t)&dummy_cpu;
+			ofs = (size_t)&dummy_cpu.cd.mips.bintrans_jump_to_32bit_pc - (size_t)&dummy_cpu;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = 0x70; *a++ = 0xa7;
 
 			/*  00 00 fb 6b     jmp     (t12)  */
@@ -1222,7 +1080,7 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 			*a++ = 0x01; *a++ = 0x00; *a++ = 0xe0; *a++ = 0xc3;
 
 			/*  34 12 70 a7     ldq     t12,4660(a0)  */
-			ofs = (size_t)&dummy_cpu.bintrans_jump_to_32bit_pc - (size_t)&dummy_cpu;
+			ofs = (size_t)&dummy_cpu.cd.mips.bintrans_jump_to_32bit_pc - (size_t)&dummy_cpu;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = 0x70; *a++ = 0xa7;
 
 			/*  00 00 fb 6b     jmp     (t12)  */
@@ -1320,7 +1178,7 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 			*a++ = 0x03; *a++ = 0x00; *a++ = 0x20; *a++ = 0xe4;	/*  beq t0,<skip>  */
 
 			/*  ldq t2,chunk_base_address(a0)  */
-			ofs = ((size_t)&dummy_cpu.chunk_base_address) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.chunk_base_address) - (size_t)&dummy_cpu;
 			*a++ = (ofs & 255); *a++ = (ofs >> 8); *a++ = 0x70; *a++ = 0xa4;
 			/*  addq t0,t2,t0  */
 			*a++ = 0x01; *a++ = 0x04; *a++ = 0x23; *a++ = 0x40;
@@ -1373,7 +1231,7 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 
 		/*  Case 1:  */
 		/*  printf("%08x ", *potential_chunk_p);  */
-		alpha_addr = *potential_chunk_p + (size_t)translation_code_chunk_space;
+		alpha_addr = *potential_chunk_p + (size_t)mem->translation_code_chunk_space;
 		ofs = (alpha_addr - ((size_t)a+4)) / 4;
 		/*  printf("%016llx %016llx %i\n", (long long)alpha_addr, (long long)a, ofs);  */
 
@@ -1382,7 +1240,7 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 		} else {
 			/*  Case 2:  */
 
-			bintrans_register_potential_quick_jump(a, p);
+			bintrans_register_potential_quick_jump(mem, a, p);
 
 			/*  15 bits at a time, which means max 60 bits, but
 			    that should be enough. the top 4 bits are probably
@@ -1424,7 +1282,7 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 			*a++ = 0x03; *a++ = 0x00; *a++ = 0x20; *a++ = 0xe4;	/*  beq t0,<skip>  */
 
 			/*  ldq t2,chunk_base_address(a0)  */
-			ofs = ((size_t)&dummy_cpu.chunk_base_address) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.chunk_base_address) - (size_t)&dummy_cpu;
 			*a++ = (ofs & 255); *a++ = (ofs >> 8); *a++ = 0x70; *a++ = 0xa4;
 			/*  addq t0,t2,t0  */
 			*a++ = 0x01; *a++ = 0x04; *a++ = 0x23; *a++ = 0x40;
@@ -1451,7 +1309,8 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 /*
  *  bintrans_write_instruction__loadstore():
  */
-static int bintrans_write_instruction__loadstore(unsigned char **addrp,
+static int bintrans_write_instruction__loadstore(
+	struct memory *mem, unsigned char **addrp,
 	int rt, int imm, int rs, int instruction_type, int bigendian)
 {
 	unsigned char *a, *fail, *generic64bit = NULL, *generic64bitA = NULL;
@@ -1574,10 +1433,10 @@ static int bintrans_write_instruction__loadstore(unsigned char **addrp,
 
 	alpha_rt = map_MIPS_to_Alpha[rt];
 
-	if (bintrans_32bit_only) {
+	if (mem->bintrans_32bit_only) {
 		/*  Special case for 32-bit addressing:  */
 
-		ofs = ((size_t)&dummy_cpu.bintrans_loadstore_32bit) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_loadstore_32bit) - (size_t)&dummy_cpu;
 		/*  ldq t12,bintrans_loadstore_32bit(a0)  */
 		*a++ = ofs; *a++ = ofs >> 8; *a++ = 0x70; *a++ = 0xa7;
 
@@ -1619,7 +1478,7 @@ static int bintrans_write_instruction__loadstore(unsigned char **addrp,
 		generic64bitA = a;
 		*a++ = 0x04; *a++ = 0x00; *a++ = 0xe0; *a++ = 0xc3;	/*  br <generic>  */
 
-		ofs = ((size_t)&dummy_cpu.bintrans_loadstore_32bit) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_loadstore_32bit) - (size_t)&dummy_cpu;
 		/*  ldq t12,bintrans_loadstore_32bit(a0)  */
 		*a++ = ofs; *a++ = ofs >> 8; *a++ = 0x70; *a++ = 0xa7;
 
@@ -1671,7 +1530,7 @@ static int bintrans_write_instruction__loadstore(unsigned char **addrp,
 		*b++ = 0xb6de0038;		/*  stq t8,56(sp)  */
 		*b++ = 0xb6fe0040;		/*  stq t9,64(sp)  */
 
-		ofs = ((size_t)&dummy_cpu.fast_vaddr_to_hostaddr) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.fast_vaddr_to_hostaddr) - (size_t)&dummy_cpu;
 
 		*b++ = 0xa7700000 | ofs;	/*  ldq t12,0(a0)  */
 
@@ -2392,9 +2251,9 @@ static int bintrans_write_instruction__mfmthilo(unsigned char **addrp,
 		if (rd != 0) {
 			/*  mfhi or mflo  */
 			if (hi_flag)
-				ofs = ((size_t)&dummy_cpu.hi) - (size_t)&dummy_cpu;
+				ofs = ((size_t)&dummy_cpu.cd.mips.hi) - (size_t)&dummy_cpu;
 			else
-				ofs = ((size_t)&dummy_cpu.lo) - (size_t)&dummy_cpu;
+				ofs = ((size_t)&dummy_cpu.cd.mips.lo) - (size_t)&dummy_cpu;
 			*a++ = (ofs & 255); *a++ = (ofs >> 8); *a++ = 0x30; *a++ = 0xa4;
 
 			bintrans_move_Alpha_reg_into_MIPS_reg(&a, ALPHA_T0, rd);
@@ -2404,9 +2263,9 @@ static int bintrans_write_instruction__mfmthilo(unsigned char **addrp,
 		bintrans_move_MIPS_reg_into_Alpha_reg(&a, rd, ALPHA_T0);
 
 		if (hi_flag)
-			ofs = ((size_t)&dummy_cpu.hi) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.hi) - (size_t)&dummy_cpu;
 		else
-			ofs = ((size_t)&dummy_cpu.lo) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.lo) - (size_t)&dummy_cpu;
 		*a++ = (ofs & 255); *a++ = (ofs >> 8); *a++ = 0x30; *a++ = 0xb4;
 	}
 
@@ -2419,7 +2278,9 @@ static int bintrans_write_instruction__mfmthilo(unsigned char **addrp,
 /*
  *  bintrans_write_instruction__mfc_mtc():
  */
-static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc_nr, int flag64bit, int rt, int rd, int mtcflag)
+static int bintrans_write_instruction__mfc_mtc(struct memory *mem,
+	unsigned char **addrp, int coproc_nr, int flag64bit, int rt,
+	int rd, int mtcflag)
 {
 	uint32_t *a, *jump;
 	int ofs;
@@ -2445,7 +2306,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 
 	a = (uint32_t *) *addrp;
 
-	ofs = ((size_t)&dummy_cpu.coproc[0]) - (size_t)&dummy_cpu;
+	ofs = ((size_t)&dummy_cpu.cd.mips.coproc[0]) - (size_t)&dummy_cpu;
 	*a++ = 0xa4300000 | (ofs & 0xffff);		/*  ldq t0,coproc[0](a0)  */
 
 	ofs = ((size_t)&dummy_coproc.reg[rd]) - (size_t)&dummy_coproc;
@@ -2508,7 +2369,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 			/*  Only allow updates to the status register if
 			    the interrupt enable bits were changed, but no
 			    other bits!  */
-			if (bintrans_32bit_only) {
+			if (mem->bintrans_32bit_only) {
 				/*  R3000 etc.  */
 				/*  t4 = 0x0fe70000;  */
 				*a++ = 0x20bf0000;
@@ -2536,7 +2397,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 
 			/*  If enabling interrupt bits would cause an
 			    exception, then don't do it:  */
-			ofs = ((size_t)&dummy_cpu.coproc[0]) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.coproc[0]) - (size_t)&dummy_cpu;
 			*a++ = 0xa4900000 | (ofs & 0xffff);		/*  ldq t3,coproc[0](a0)  */
 			ofs = ((size_t)&dummy_coproc.reg[COP0_CAUSE]) - (size_t)&dummy_coproc;
 			*a++ = 0xa4a40000 | (ofs & 0xffff);		/*  ldq t4,reg_rd(t3)  */
@@ -2570,7 +2431,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 
 		*a++ = 0x40201402;	/*  addq    t0,0,t1  */
 
-		ofs = ((size_t)&dummy_cpu.coproc[0]) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.coproc[0]) - (size_t)&dummy_cpu;
 		*a++ = 0xa4300000 | (ofs & 0xffff);		/*  ldq t0,coproc[0](a0)  */
 		ofs = ((size_t)&dummy_coproc.reg[rd]) - (size_t)&dummy_coproc;
 		*a++ = 0xb4410000 | (ofs & 0xffff);		/*  stq t1,reg_rd(t0)  */
@@ -2602,14 +2463,14 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	int ofs = 0;
 
 	switch (itype) {
-	case TLB_TLBWI:
-	case TLB_TLBWR:
-	case TLB_TLBP:
-	case TLB_TLBR:
-	case TLB_RFE:
-	case TLB_ERET:
-	case TLB_BREAK:
-	case TLB_SYSCALL:
+	case CALL_TLBWI:
+	case CALL_TLBWR:
+	case CALL_TLBP:
+	case CALL_TLBR:
+	case CALL_RFE:
+	case CALL_ERET:
+	case CALL_BREAK:
+	case CALL_SYSCALL:
 		break;
 	default:
 		return 0;
@@ -2620,19 +2481,19 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	/*  a0 = pointer to the cpu struct  */
 
 	switch (itype) {
-	case TLB_TLBWI:
-	case TLB_TLBWR:
+	case CALL_TLBWI:
+	case CALL_TLBWR:
 		/*  a1 = 0 for indexed, 1 for random  */
-		*a++ = 0x223f0000 | (itype == TLB_TLBWR);
+		*a++ = 0x223f0000 | (itype == CALL_TLBWR);
 		break;
-	case TLB_TLBP:
-	case TLB_TLBR:
+	case CALL_TLBP:
+	case CALL_TLBR:
 		/*  a1 = 0 for probe, 1 for read  */
-		*a++ = 0x223f0000 | (itype == TLB_TLBR);
+		*a++ = 0x223f0000 | (itype == CALL_TLBR);
 		break;
-	case TLB_BREAK:
-	case TLB_SYSCALL:
-		*a++ = 0x223f0000 | (itype == TLB_BREAK? EXCEPTION_BP : EXCEPTION_SYS);
+	case CALL_BREAK:
+	case CALL_SYSCALL:
+		*a++ = 0x223f0000 | (itype == CALL_BREAK? EXCEPTION_BP : EXCEPTION_SYS);
 		break;
 	}
 
@@ -2653,23 +2514,23 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	*a++ = 0xb6fe0040;		/*  stq t9,64(sp)  */
 
 	switch (itype) {
-	case TLB_TLBP:
-	case TLB_TLBR:
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_tlbpr) - (size_t)&dummy_cpu;
+	case CALL_TLBP:
+	case CALL_TLBR:
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_tlbpr) - (size_t)&dummy_cpu;
 		break;
-	case TLB_TLBWR:
-	case TLB_TLBWI:
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_tlbwri) - (size_t)&dummy_cpu;
+	case CALL_TLBWR:
+	case CALL_TLBWI:
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_tlbwri) - (size_t)&dummy_cpu;
 		break;
-	case TLB_RFE:
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_rfe) - (size_t)&dummy_cpu;
+	case CALL_RFE:
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_rfe) - (size_t)&dummy_cpu;
 		break;
-	case TLB_ERET:
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_eret) - (size_t)&dummy_cpu;
+	case CALL_ERET:
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_eret) - (size_t)&dummy_cpu;
 		break;
-	case TLB_BREAK:
-	case TLB_SYSCALL:
-		ofs = ((size_t)&dummy_cpu.bintrans_simple_exception) - (size_t)&dummy_cpu;
+	case CALL_BREAK:
+	case CALL_SYSCALL:
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_simple_exception) - (size_t)&dummy_cpu;
 		break;
 	}
 
@@ -2696,9 +2557,9 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	*addrp = (unsigned char *) a;
 
 	switch (itype) {
-	case TLB_ERET:
-	case TLB_BREAK:
-	case TLB_SYSCALL:
+	case CALL_ERET:
+	case CALL_BREAK:
+	case CALL_SYSCALL:
 		break;
 	default:
 		bintrans_write_pc_inc(addrp);
@@ -2707,3 +2568,165 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	return 1;
 }
 
+
+/*
+ *  bintrans_backend_init():
+ *
+ *  This is neccessary for broken 2.95.4 compilers on FreeBSD/Alpha 4.9,
+ *  and probably a few others. (For Compaq's CC, and for gcc 3.x, this
+ *  wouldn't be neccessary, and the old code would have worked.)
+ */
+static void bintrans_backend_init(void)
+{ 
+	int size;
+	uint32_t *p;
+
+
+	/*  "runchunk":  */
+	size = 256;		/*  NOTE: This MUST be enough, or we fail  */
+	p = (uint32_t *)mmap(NULL, size, PROT_READ | PROT_WRITE | PROT_EXEC,
+	    MAP_ANON | MAP_PRIVATE, -1, 0);
+
+	/*  If mmap() failed, try malloc():  */
+	if (p == NULL) {
+		p = malloc(size);
+		if (p == NULL) {
+			fprintf(stderr, "bintrans_backend_init(): out of memory\n");
+			exit(1);
+		}
+	}
+
+	bintrans_runchunk = (void *)p;
+
+	*p++ = 0x23deffa0;		/*  lda     sp,-0x60(sp)  */
+	*p++ = 0xb75e0000;		/*  stq     ra,0(sp)  */
+	*p++ = 0xb53e0008;		/*  stq     s0,8(sp)  */
+	*p++ = 0xb55e0010;		/*  stq     s1,16(sp)  */
+	*p++ = 0xb57e0018;		/*  stq     s2,24(sp)  */
+	*p++ = 0xb59e0020;		/*  stq     s3,32(sp)  */
+	*p++ = 0xb5be0028;		/*  stq     s4,40(sp)  */
+	*p++ = 0xb5de0030;		/*  stq     s5,48(sp)  */
+	*p++ = 0xb5fe0038;		/*  stq     s6,56(sp)  */
+	*p++ = 0xb7be0058;		/*  stq     gp,0x58(sp)  */
+
+	*p++ = 0xa4d00000 | ofs_pc;	/*  ldq     t5,"pc"(a0)  */
+	*p++ = 0xa0f00000 | ofs_n;	/*  ldl     t6,"bintrans_instructions_executed"(a0)  */
+	*p++ = 0xa5100000 | ofs_a0;	/*  ldq     t7,"a0"(a0)  */
+	*p++ = 0xa6d00000 | ofs_a1;	/*  ldq     t8,"a1"(a0)  */
+	*p++ = 0xa6f00000 | ofs_s0;	/*  ldq     t9,"s0"(a0)  */
+	*p++ = 0xa1300000 | ofs_ds;	/*  ldl     s0,"delay_slot"(a0)  */
+	*p++ = 0xa5500000 | ofs_ja;	/*  ldq     s1,"delay_jmpaddr"(a0)  */
+	*p++ = 0xa5700000 | ofs_sp;	/*  ldq     s2,"gpr[sp]"(a0)  */
+	*p++ = 0xa5900000 | ofs_ra;	/*  ldq     s3,"gpr[ra]"(a0)  */
+	*p++ = 0xa5b00000 | ofs_t0;	/*  ldq     s4,"gpr[t0]"(a0)  */
+	*p++ = 0xa5d00000 | ofs_t1;	/*  ldq     s5,"gpr[t1]"(a0)  */
+	*p++ = 0xa5f00000 | ofs_t2;	/*  ldq     s6,"gpr[t2]"(a0)  */
+	*p++ = 0xa7100000 | ofs_tbl0;	/*  ldq     t10,table0(a0)  */
+	*p++ = 0xa7300000 | ofs_v0;	/*  ldq     t11,"gpr[v0]"(a0)  */
+
+	*p++ = 0x6b514000;		/*  jsr     ra,(a1),<back>  */
+
+	*p++ = 0xb4d00000 | ofs_pc;	/*  stq     t5,"pc"(a0)  */
+	*p++ = 0xb0f00000 | ofs_n;	/*  stl     t6,"bintrans_instructions_executed"(a0)  */
+	*p++ = 0xb5100000 | ofs_a0;	/*  stq     t7,"a0"(a0)  */
+	*p++ = 0xb6d00000 | ofs_a1;	/*  stq     t8,"a1"(a0)  */
+	*p++ = 0xb6f00000 | ofs_s0;	/*  stq     t9,"s0"(a0)  */
+	*p++ = 0xb1300000 | ofs_ds;	/*  stl     s0,"delay_slot"(a0)  */
+	*p++ = 0xb5500000 | ofs_ja;	/*  stq     s1,"delay_jmpaddr"(a0)  */
+	*p++ = 0xb5700000 | ofs_sp;	/*  stq     s2,"gpr[sp]"(a0)  */
+	*p++ = 0xb5900000 | ofs_ra;	/*  stq     s3,"gpr[ra]"(a0)  */
+	*p++ = 0xb5b00000 | ofs_t0;	/*  stq     s4,"gpr[t0]"(a0)  */
+	*p++ = 0xb5d00000 | ofs_t1;	/*  stq     s5,"gpr[t1]"(a0)  */
+	*p++ = 0xb5f00000 | ofs_t2;	/*  stq     s6,"gpr[t2]"(a0)  */
+	*p++ = 0xb7300000 | ofs_v0;	/*  stq     t11,"gpr[v0]"(a0)  */
+
+	*p++ = 0xa75e0000;		/*  ldq     ra,0(sp)  */
+	*p++ = 0xa53e0008;		/*  ldq     s0,8(sp)  */
+	*p++ = 0xa55e0010;		/*  ldq     s1,16(sp)  */
+	*p++ = 0xa57e0018;		/*  ldq     s2,24(sp)  */
+	*p++ = 0xa59e0020;		/*  ldq     s3,32(sp)  */
+	*p++ = 0xa5be0028;		/*  ldq     s4,40(sp)  */
+	*p++ = 0xa5de0030;		/*  ldq     s5,48(sp)  */
+	*p++ = 0xa5fe0038;		/*  ldq     s6,56(sp)  */
+	*p++ = 0xa7be0058;		/*  ldq     gp,0x58(sp)  */
+	*p++ = 0x23de0060;		/*  lda     sp,0x60(sp)  */
+	*p++ = 0x6bfa8001;		/*  ret   */
+
+
+	/*  "jump to 32bit pc":  */
+	size = 128;	/*  WARNING! Don't make this too small.  */
+	p = (uint32_t *)mmap(NULL, size, PROT_READ | PROT_WRITE | PROT_EXEC,
+	    MAP_ANON | MAP_PRIVATE, -1, 0);
+
+	/*  If mmap() failed, try malloc():  */
+	if (p == NULL) {
+		p = malloc(size);
+		if (p == NULL) {
+			fprintf(stderr, "bintrans_backend_init(): out of memory\n");
+			exit(1);
+		}
+	}
+
+	bintrans_jump_to_32bit_pc = (void *)p;
+
+	/*  Don't execute too many instructions:  */
+	*p++ = 0x205f0000 | (N_SAFE_BINTRANS_LIMIT-1);  /*  lda t1,safe-1 */
+
+	*p++ = 0x40e20da1;		/*  cmple t6,t1,t0  */
+	*p++ = 0xf4200001;		/*  bne  */
+	*p++ = 0x6bfa8001;		/*  ret  */
+
+	*p++ = 0x40c01411;		/*  addq t5,0,a1  */
+
+	/*
+	 *  Special case for 32-bit addressing:
+	 *
+	 *  t1 = 1023;
+	 *  t2 = ((a1 >> 22) & t1) * sizeof(void *);
+	 *  t3 = ((a1 >> 12) & t1) * sizeof(void *);
+	 *  t1 = a1 & 4095;
+	 */
+	*p++ = 0x205f1ff8;	/*  lda     t1,1023 * 8  */
+	*p++ = 0x4a227683;	/*  srl     a1,19,t2  */
+	*p++ = 0x4a213684;	/*  srl     a1, 9,t3  */
+	*p++ = 0x44620003;	/*  and     t2,t1,t2  */
+
+	/*
+	 *  t10 is vaddr_to_hostaddr_table0
+	 *
+	 *  a3 = tbl0[t2]  (load entry from tbl0)
+	 */
+	*p++ = 0x43030412;	/*  addq    t10,t2,a2  */
+	*p++ = 0x44820004;	/*  and     t3,t1,t3  */
+	*p++ = 0xa6720000;	/*  ldq     a3,0(a2)  */
+	*p++ = 0x205f0ffc;	/*  lda     t1,0xffc  */
+
+	/*
+	 *  a3 = tbl1[t3]  (load entry from tbl1 (whic is a3))
+	 */
+	*p++ = 0x42640413;	/*  addq    a3,t3,a3  */
+	*p++ = 0x46220002;	/*  and     a1,t1,t1  */
+
+	*p++ = 0xa6730000 | ofs_c0;	/*  ldq a3,chunks[0](a3)  */
+
+	/*
+	 *  NULL? Then just return.
+	 */
+	*p++ = 0xf6600001;	/*  bne     a3,<ok>  */
+	*p++ = 0x6bfa8001;	/*  ret  */
+
+	*p++ = 0x40530402;	/*  addq    t1,a3,t1  */
+	*p++ = 0xa0220000;	/*  ldl     t0,0(t1)  */
+
+	/*  No translation? Then return.  */
+	*p++ = 0xe4200003;	/*  beq t0,<skip>  */
+
+	*p++ = 0xa4700000 | ofs_cb;	/*  ldq t2,chunk_base_address(a0)  */
+
+	*p++ = 0x40230401;	/*  addq t0,t2,t0  */
+	*p++ = 0x6be10000;	/*  jmp (t0)  */
+
+	/*  Return to the main translation loop.  */
+	*p++ = 0x6bfa8001;		/*  ret  */
+} 
+
