@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: bintrans_i386.c,v 1.63 2005/01/10 03:06:32 debug Exp $
+ *  $Id: bintrans_i386.c,v 1.72 2005/02/02 20:12:45 debug Exp $
  *
  *  i386 specific code for dynamic binary translation.
  *  See bintrans.c for more information.  Included from bintrans.c.
@@ -39,7 +39,7 @@
 
 
 struct cpu dummy_cpu;
-struct coproc dummy_coproc;
+struct mips_coproc dummy_coproc;
 struct vth32_table dummy_vth32_table;
 
 
@@ -55,9 +55,9 @@ static void bintrans_host_cacheinvalidate(unsigned char *p, size_t len)
 }
 
 
-#define ofs_i		(((size_t)&dummy_cpu.bintrans_instructions_executed) - ((size_t)&dummy_cpu))
-#define ofs_pc		(((size_t)&dummy_cpu.pc) - ((size_t)&dummy_cpu))
-#define ofs_pc_last	(((size_t)&dummy_cpu.pc_last) - ((size_t)&dummy_cpu))
+#define ofs_i		(((size_t)&dummy_cpu.cd.mips.bintrans_instructions_executed) - ((size_t)&dummy_cpu))
+#define ofs_pc		(((size_t)&dummy_cpu.cd.mips.pc) - ((size_t)&dummy_cpu))
+#define ofs_pc_last	(((size_t)&dummy_cpu.cd.mips.pc_last) - ((size_t)&dummy_cpu))
 
 
 unsigned char bintrans_i386_runchunk[41] = {
@@ -120,7 +120,7 @@ static unsigned char bintrans_i386_jump_to_32bit_pc[76] = {
 	 *
 	 *  8b 8e 34 12 00 00       mov    0x1234(%esi),%ecx
 	 */
-#define ofs_tabl0	(((size_t)&dummy_cpu.vaddr_to_hostaddr_table0) - ((size_t)&dummy_cpu))
+#define ofs_tabl0	(((size_t)&dummy_cpu.cd.mips.vaddr_to_hostaddr_table0) - ((size_t)&dummy_cpu))
 	0x8b, 0x8e,
 	ofs_tabl0 & 255, (ofs_tabl0 >> 8) & 255, (ofs_tabl0 >> 16) & 255, (ofs_tabl0 >> 24) & 255,
 
@@ -143,7 +143,7 @@ static unsigned char bintrans_i386_jump_to_32bit_pc[76] = {
 	0x81, 0xe3, 0xfc, 0x0f, 0, 0,
 
 	/*
-	 *  ecx = vaddr_to_hostaddr_table0[a][b].chunks
+	 *  ecx = vaddr_to_hostaddr_table0[a][b].cd.mips.chunks
 	 *
 	 *  8b 8c 19 56 34 12 00    mov    0x123456(%ecx,%ebx,1),%ecx
 	 */
@@ -182,7 +182,7 @@ static unsigned char bintrans_i386_jump_to_32bit_pc[76] = {
 
 	/*  03 86 78 56 34 12       add    0x12345678(%esi),%eax  */
 	/*  ff e0                   jmp    *%eax  */
-#define ofs_chunkbase	((size_t)&dummy_cpu.chunk_base_address - (size_t)&dummy_cpu)
+#define ofs_chunkbase	((size_t)&dummy_cpu.cd.mips.chunk_base_address - (size_t)&dummy_cpu)
 	0x03, 0x86,
 	    ofs_chunkbase & 255, (ofs_chunkbase >> 8) & 255, (ofs_chunkbase >> 16) & 255, (ofs_chunkbase >> 24) & 255,
 	0xff, 0xe0
@@ -250,13 +250,13 @@ static void (*bintrans_loadstore_32bit)
 /*
  *  bintrans_write_quickjump():
  */
-static void bintrans_write_quickjump(unsigned char *quickjump_code,
-	uint32_t chunkoffset)
+static void bintrans_write_quickjump(struct memory *mem,
+	unsigned char *quickjump_code, uint32_t chunkoffset)
 {
 	uint32_t i386_addr;
 	unsigned char *a = quickjump_code;
 
-	i386_addr = chunkoffset + (size_t)translation_code_chunk_space;
+	i386_addr = chunkoffset + (size_t)mem->translation_code_chunk_space;
 	i386_addr = i386_addr - ((size_t)a + 5);
 
         /*  printf("chunkoffset=%i, %08x %08x %i\n",
@@ -303,14 +303,15 @@ static void bintrans_write_chunkreturn_fail(unsigned char **addrp)
 static void bintrans_write_pc_inc(unsigned char **addrp)
 {
 	unsigned char *a = *addrp;
-	int ofs;
 
 	/*  83 c7 04                add    $0x4,%edi  */
 	*a++ = 0x83; *a++ = 0xc7; *a++ = 4;
 
+#if 0
 	if (!bintrans_32bit_only) {
+		int ofs;
 		/*  83 96 zz zz zz zz 00    adcl   $0x0,zz(%esi)  */
-		ofs = ((size_t)&dummy_cpu.pc) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.pc) - (size_t)&dummy_cpu;
 		ofs += 4;
 		*a++ = 0x83; *a++ = 0x96;
 		*a++ = ofs & 255;
@@ -319,6 +320,7 @@ static void bintrans_write_pc_inc(unsigned char **addrp)
 		*a++ = (ofs >> 24) & 255;
 		*a++ = 0;
 	}
+#endif
 
 	/*  45   inc %ebp  */
 	*a++ = 0x45;
@@ -338,11 +340,14 @@ static void load_pc_into_eax_edx(unsigned char **addrp)
 	/*  89 f8                   mov    %edi,%eax  */
 	*a++ = 0x89; *a++ = 0xf8;
 
+#if 0
 	if (bintrans_32bit_only) {
 		/*  99                      cltd   */
 		*a++ = 0x99;
-	} else {
-		int ofs = ((size_t)&dummy_cpu.pc) - (size_t)&dummy_cpu;
+	} else
+#endif
+ {
+		int ofs = ((size_t)&dummy_cpu.cd.mips.pc) - (size_t)&dummy_cpu;
 		/*  8b 96 3c 30 00 00       mov    0x303c(%esi),%edx  */
 		ofs += 4;
 		*a++ = 0x8b; *a++ = 0x96;
@@ -359,7 +364,7 @@ static void load_pc_into_eax_edx(unsigned char **addrp)
 static void store_eax_edx_into_pc(unsigned char **addrp)
 {
 	unsigned char *a;
-	int ofs = ((size_t)&dummy_cpu.pc) - (size_t)&dummy_cpu;
+	int ofs = ((size_t)&dummy_cpu.cd.mips.pc) - (size_t)&dummy_cpu;
 	a = *addrp;
 
 	/*  89 c7                   mov    %eax,%edi  */
@@ -377,7 +382,7 @@ static void store_eax_edx_into_pc(unsigned char **addrp)
 /*
  *  load_into_eax_edx():
  *
- *  Usage:    load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);   etc.
+ *  Usage:    load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);   etc.
  */
 static void load_into_eax_edx(unsigned char **addrp, void *p)
 {
@@ -389,10 +394,13 @@ static void load_into_eax_edx(unsigned char **addrp, void *p)
 	*a++ = 0x8b; *a++ = 0x86;
 	*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 
+#if 0
 	if (bintrans_32bit_only) {
 		/*  99                      cltd   */
 		*a++ = 0x99;
-	} else {
+	} else
+#endif
+ {
 		/*  8b 96 3c 30 00 00       mov    0x303c(%esi),%edx  */
 		ofs += 4;
 		*a++ = 0x8b; *a++ = 0x96;
@@ -406,7 +414,7 @@ static void load_into_eax_edx(unsigned char **addrp, void *p)
 /*
  *  load_into_eax_and_sign_extend_into_edx():
  *
- *  Usage:    load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.gpr[rs]);   etc.
+ *  Usage:    load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);   etc.
  */
 static void load_into_eax_and_sign_extend_into_edx(unsigned char **addrp, void *p)
 {
@@ -428,7 +436,7 @@ static void load_into_eax_and_sign_extend_into_edx(unsigned char **addrp, void *
 /*
  *  load_into_eax_dont_care_about_edx():
  *
- *  Usage:    load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rs]);   etc.
+ *  Usage:    load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);   etc.
  */
 static void load_into_eax_dont_care_about_edx(unsigned char **addrp, void *p)
 {
@@ -447,7 +455,7 @@ static void load_into_eax_dont_care_about_edx(unsigned char **addrp, void *p)
 /*
  *  store_eax_edx():
  *
- *  Usage:    store_eax_edx(&a, &dummy_cpu.gpr[rs]);   etc.
+ *  Usage:    store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);   etc.
  */
 static void store_eax_edx(unsigned char **addrp, void *p)
 {
@@ -486,7 +494,7 @@ static int bintrans_write_instruction__lui(unsigned char **addrp, int rt, int im
 	/*  99                      cltd   */
 	*a++ = 0x99;
 
-	store_eax_edx(&a, &dummy_cpu.gpr[rt]);
+	store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 	*addrp = a;
 
 rt0:
@@ -498,7 +506,8 @@ rt0:
 /*
  *  bintrans_write_instruction__jr():
  */
-static int bintrans_write_instruction__jr(unsigned char **addrp, int rs, int rd, int special)
+static int bintrans_write_instruction__jr(unsigned char **addrp,
+	int rs, int rd, int special)
 {
 	unsigned char *a;
 	int ofs;
@@ -511,26 +520,31 @@ static int bintrans_write_instruction__jr(unsigned char **addrp, int rs, int rd,
 	 */
 
 	/*  c7 86 38 30 00 00 01 00 00 00    movl   $0x1,0x3038(%esi)  */
-	ofs = ((size_t)&dummy_cpu.delay_slot) - (size_t)&dummy_cpu;
+	ofs = ((size_t)&dummy_cpu.cd.mips.delay_slot) - (size_t)&dummy_cpu;
 	*a++ = 0xc7; *a++ = 0x86;
 	*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 	*a++ = TO_BE_DELAYED; *a++ = 0; *a++ = 0; *a++ = 0;
 
+#if 0
 	if (bintrans_32bit_only)
-		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 	else
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);
+#endif
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 
-	store_eax_edx(&a, &dummy_cpu.delay_jmpaddr);
+	store_eax_edx(&a, &dummy_cpu.cd.mips.delay_jmpaddr);
 
 	if (special == SPECIAL_JALR && rd != 0) {
 		/*  gpr[rd] = retaddr    (pc + 8)  */
 
+#if 0
 		if (bintrans_32bit_only) {
 			load_pc_into_eax_edx(&a);
 			/*  83 c0 08                add    $0x8,%eax  */
 			*a++ = 0x83; *a++ = 0xc0; *a++ = 0x08;
-		} else {
+		} else
+#endif
+ {
 			load_pc_into_eax_edx(&a);
 			/*  83 c0 08                add    $0x8,%eax  */
 			/*  83 d2 00                adc    $0x0,%edx  */
@@ -538,7 +552,7 @@ static int bintrans_write_instruction__jr(unsigned char **addrp, int rs, int rd,
 			*a++ = 0x83; *a++ = 0xd2; *a++ = 0x00;
 		}
 
-		store_eax_edx(&a, &dummy_cpu.gpr[rd]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rd]);
 	}
 
 	*addrp = a;
@@ -561,18 +575,18 @@ static int bintrans_write_instruction__mfmthilo(unsigned char **addrp,
 		if (rd != 0) {
 			/*  mfhi or mflo  */
 			if (hi_flag)
-				load_into_eax_edx(&a, &dummy_cpu.hi);
+				load_into_eax_edx(&a, &dummy_cpu.cd.mips.hi);
 			else
-				load_into_eax_edx(&a, &dummy_cpu.lo);
-			store_eax_edx(&a, &dummy_cpu.gpr[rd]);
+				load_into_eax_edx(&a, &dummy_cpu.cd.mips.lo);
+			store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rd]);
 		}
 	} else {
 		/*  mthi or mtlo  */
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rd]);
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rd]);
 		if (hi_flag)
-			store_eax_edx(&a, &dummy_cpu.hi);
+			store_eax_edx(&a, &dummy_cpu.cd.mips.hi);
 		else
-			store_eax_edx(&a, &dummy_cpu.lo);
+			store_eax_edx(&a, &dummy_cpu.cd.mips.lo);
 	}
 
 	*addrp = a;
@@ -606,22 +620,24 @@ static int bintrans_write_instruction__addiu_etc(unsigned char **addrp,
 
 	if (uimm == 0 && (instruction_type == HI6_ADDIU ||
 	    instruction_type == HI6_ADDI)) {
-		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.gpr[rs]);
-		store_eax_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 		goto rt0;
 	}
 
 	if (uimm == 0 && (instruction_type == HI6_DADDIU ||
 	    instruction_type == HI6_DADDI || instruction_type == HI6_ORI)) {
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);
-		store_eax_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 		goto rt0;
 	}
 
+#if 0
 	if (bintrans_32bit_only)
-		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 	else
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);
+#endif
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 
 	switch (instruction_type) {
 	case HI6_ADDIU:
@@ -657,18 +673,10 @@ static int bintrans_write_instruction__addiu_etc(unsigned char **addrp,
 	case HI6_ORI:
 		/*  0d 34 12 00 00          or     $0x1234,%eax  */
 		*a++ = 0xd; *a++ = uimm; *a++ = uimm >> 8; *a++ = 0; *a++ = 0;
-		if (bintrans_32bit_only) {
-			/*  99                      cltd   */
-			*a++ = 0x99;
-		}
 		break;
 	case HI6_XORI:
 		/*  35 34 12 00 00          xor    $0x1234,%eax  */
 		*a++ = 0x35; *a++ = uimm; *a++ = uimm >> 8; *a++ = 0; *a++ = 0;
-		if (bintrans_32bit_only) {
-			/*  99                      cltd   */
-			*a++ = 0x99;
-		}
 		break;
 	case HI6_SLTIU:
 		/*  set if less than, unsigned. (compare edx:eax to ecx:ebx)  */
@@ -677,10 +685,12 @@ static int bintrans_write_instruction__addiu_etc(unsigned char **addrp,
 		/*  b9 ff ff ff ff          mov    $0xffffffff,%ecx  */
 		/*  or  */
 		/*  29 c9                   sub    %ecx,%ecx  */
+#if 0
 		if (bintrans_32bit_only) {
 			/*  99                      cltd   */
 			*a++ = 0x99;
 		}
+#endif
 		*a++ = 0xbb; *a++ = uimm; *a++ = uimm >> 8;
 		if (uimm & 0x8000) {
 			*a++ = 0xff; *a++ = 0xff;
@@ -720,10 +730,12 @@ static int bintrans_write_instruction__addiu_etc(unsigned char **addrp,
 		/*  b9 ff ff ff ff          mov    $0xffffffff,%ecx  */
 		/*  or  */
 		/*  29 c9                   sub    %ecx,%ecx  */
+#if 0
 		if (bintrans_32bit_only) {
 			/*  99                      cltd   */
 			*a++ = 0x99;
 		}
+#endif
 		*a++ = 0xbb; *a++ = uimm; *a++ = uimm >> 8;
 		if (uimm & 0x8000) {
 			*a++ = 0xff; *a++ = 0xff;
@@ -763,7 +775,7 @@ static int bintrans_write_instruction__addiu_etc(unsigned char **addrp,
 		break;
 	}
 
-	store_eax_edx(&a, &dummy_cpu.gpr[rt]);
+	store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 
 rt0:
 	*addrp = a;
@@ -775,7 +787,8 @@ rt0:
 /*
  *  bintrans_write_instruction__jal():
  */
-static int bintrans_write_instruction__jal(unsigned char **addrp, int imm, int link)
+static int bintrans_write_instruction__jal(unsigned char **addrp,
+	int imm, int link)
 {
 	unsigned char *a;
 	uint32_t subimm;
@@ -787,12 +800,15 @@ static int bintrans_write_instruction__jal(unsigned char **addrp, int imm, int l
 
 	if (link) {
 		/*  gpr[31] = pc + 8  */
+#if 0
 		if (bintrans_32bit_only) {
 			/*  50             push  %eax */
 			/*  83 c0 08       add   $0x8,%eax  */
 			*a++ = 0x50;
 			*a++ = 0x83; *a++ = 0xc0; *a++ = 0x08;
-		} else {
+		} else
+#endif
+ {
 			/*  50             push  %eax */
 			/*  52             push  %edx */
 			/*  83 c0 08                add    $0x8,%eax  */
@@ -802,11 +818,14 @@ static int bintrans_write_instruction__jal(unsigned char **addrp, int imm, int l
 			*a++ = 0x83; *a++ = 0xc0; *a++ = 0x08;
 			*a++ = 0x83; *a++ = 0xd2; *a++ = 0x00;
 		}
-		store_eax_edx(&a, &dummy_cpu.gpr[31]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[31]);
+#if 0
 		if (bintrans_32bit_only) {
 			/*  58     pop %eax  */
 			*a++ = 0x58;
-		} else {
+		} else
+#endif
+ {
 			/*  5a     pop %edx  */
 			/*  58     pop %eax  */
 			*a++ = 0x5a;
@@ -835,10 +854,10 @@ static int bintrans_write_instruction__jal(unsigned char **addrp, int imm, int l
 	*a++ = 0x0d; *a++ = subimm; *a++ = subimm >> 8;
 	*a++ = subimm >> 16; *a++ = subimm >> 24;
 
-	store_eax_edx(&a, &dummy_cpu.delay_jmpaddr);
+	store_eax_edx(&a, &dummy_cpu.cd.mips.delay_jmpaddr);
 
 	/*  c7 86 38 30 00 00 01 00 00 00    movl   $0x1,0x3038(%esi)  */
-	ofs = ((size_t)&dummy_cpu.delay_slot) - (size_t)&dummy_cpu;
+	ofs = ((size_t)&dummy_cpu.cd.mips.delay_slot) - (size_t)&dummy_cpu;
 	*a++ = 0xc7; *a++ = 0x86;
 	*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 	*a++ = TO_BE_DELAYED; *a++ = 0; *a++ = 0; *a++ = 0;
@@ -913,27 +932,27 @@ static int bintrans_write_instruction__addu_etc(unsigned char **addrp,
 	if ((instruction_type == SPECIAL_ADDU || instruction_type == SPECIAL_DADDU
 	    || instruction_type == SPECIAL_OR) && rt == 0) {
 		if (load64)
-			load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);
+			load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 		else
-			load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.gpr[rs]);
-		store_eax_edx(&a, &dummy_cpu.gpr[rd]);
+			load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rd]);
 		*addrp = a;
 		goto rd0;
 	}
 
 	/*  edx:eax = rs, ecx:ebx = rt  */
 	if (load64) {
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 		/*  89 c3                   mov    %eax,%ebx  */
 		/*  89 d1                   mov    %edx,%ecx  */
 		*a++ = 0x89; *a++ = 0xc3; *a++ = 0x89; *a++ = 0xd1;
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 	} else {
-		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 		/*  89 c3                   mov    %eax,%ebx  */
 		/*  89 d1                   mov    %edx,%ecx  */
 		*a++ = 0x89; *a++ = 0xc3; *a++ = 0x89; *a++ = 0xd1;
-		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_and_sign_extend_into_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 	}
 
 	switch (instruction_type) {
@@ -1139,13 +1158,13 @@ static int bintrans_write_instruction__addu_etc(unsigned char **addrp,
 		*a++ = 0x89; *a++ = 0xd7;
 		*a++ = 0x99;
 		/*  here: edi=hi, edx:eax = sign-extended lo  */
-		store_eax_edx(&a, &dummy_cpu.lo);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.lo);
 		/*  89 f8                   mov    %edi,%eax  */
 		/*  99                      cltd   */
 		*a++ = 0x89; *a++ = 0xf8;
 		*a++ = 0x99;
 		/*  here: edx:eax = sign-extended hi  */
-		store_eax_edx(&a, &dummy_cpu.hi);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.hi);
 		/*  5f    pop %edi  */
 		*a++ = 0x5f;
 		do_store = 0;
@@ -1178,13 +1197,13 @@ static int bintrans_write_instruction__addu_etc(unsigned char **addrp,
 		*a++ = 0x89; *a++ = 0xd7;
 		*a++ = 0x99;
 		/*  here: edi=hi, edx:eax = sign-extended lo  */
-		store_eax_edx(&a, &dummy_cpu.lo);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.lo);
 		/*  89 f8                   mov    %edi,%eax  */
 		/*  99                      cltd   */
 		*a++ = 0x89; *a++ = 0xf8;
 		*a++ = 0x99;
 		/*  here: edx:eax = sign-extended hi  */
-		store_eax_edx(&a, &dummy_cpu.hi);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.hi);
 		/*  5f    pop %edi  */
 		*a++ = 0x5f;
 		do_store = 0;
@@ -1219,7 +1238,7 @@ static int bintrans_write_instruction__addu_etc(unsigned char **addrp,
 	}
 
 	if (do_store)
-		store_eax_edx(&a, &dummy_cpu.gpr[rd]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rd]);
 
 	*addrp = a;
 rd0:
@@ -1231,13 +1250,15 @@ rd0:
 /*
  *  bintrans_write_instruction__mfc_mtc():
  */
-static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc_nr, int flag64bit, int rt, int rd, int mtcflag)
+static int bintrans_write_instruction__mfc_mtc(struct memory *mem,
+	unsigned char **addrp, int coproc_nr, int flag64bit, int rt,
+	int rd, int mtcflag)
 {
 	unsigned char *a, *failskip;
 	int ofs;
 
 	if (mtcflag && flag64bit) {
-		/*  mtc: */
+		/*  dmtc  */
 		return 0;
 	}
 
@@ -1262,7 +1283,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 	 *************************************************************/
 
 	/*  8b 96 3c 30 00 00       mov    0x303c(%esi),%edx  */
-	ofs = ((size_t)&dummy_cpu.coproc[0]) - (size_t)&dummy_cpu;
+	ofs = ((size_t)&dummy_cpu.cd.mips.coproc[0]) - (size_t)&dummy_cpu;
 	*a++ = 0x8b; *a++ = 0x96;
 	*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 
@@ -1278,7 +1299,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 		*a++ = 0x8b; *a++ = 0x9a;
 		*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 
 		/*
 		 *  Here:  eax contains the value in register rt,
@@ -1289,6 +1310,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 		 */
 
 		switch (rd) {
+
 		case COP0_INDEX:
 			break;
 
@@ -1313,13 +1335,13 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 			*a++ = 0x89; *a++ = 0xc1;
 			*a++ = 0x89; *a++ = 0xda;
 			*a++ = 0x81; *a++ = 0xe1; *a++ = 0x00; *a++ = 0x00;
-			if (bintrans_32bit_only) {
+			if (mem->bintrans_32bit_only) {
 				*a++ = 0xe7; *a++ = 0x0f;
 			} else {
 				*a++ = 0xff; *a++ = 0xff;
 			}
 			*a++ = 0x81; *a++ = 0xe2; *a++ = 0x00; *a++ = 0x00;
-			if (bintrans_32bit_only) {
+			if (mem->bintrans_32bit_only) {
 				*a++ = 0xe7; *a++ = 0x0f;
 			} else {
 				*a++ = 0xff; *a++ = 0xff;
@@ -1333,7 +1355,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 			    an interrupt exception:  */
 
 			/*  8b 96 3c 30 00 00       mov    0x303c(%esi),%edx  */
-			ofs = ((size_t)&dummy_cpu.coproc[0]) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.coproc[0]) - (size_t)&dummy_cpu;
 			*a++ = 0x8b; *a++ = 0x96;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 
@@ -1366,7 +1388,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 		}
 
 		/*  8b 96 3c 30 00 00       mov    0x303c(%esi),%edx  */
-		ofs = ((size_t)&dummy_cpu.coproc[0]) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.coproc[0]) - (size_t)&dummy_cpu;
 		*a++ = 0x8b; *a++ = 0x96;
 		*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 
@@ -1403,7 +1425,7 @@ static int bintrans_write_instruction__mfc_mtc(unsigned char **addrp, int coproc
 			*a++ = 0x99;
 		}
 
-		store_eax_edx(&a, &dummy_cpu.gpr[rt]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 	}
 
 	*addrp = a;
@@ -1447,12 +1469,12 @@ static int bintrans_write_instruction__branch(unsigned char **addrp,
 	switch (instruction_type) {
 	case HI6_BEQ:
 	case HI6_BNE:
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 		/*  89 c3                   mov    %eax,%ebx  */
 		/*  89 d1                   mov    %edx,%ecx  */
 		*a++ = 0x89; *a++ = 0xc3; *a++ = 0x89; *a++ = 0xd1;
 	}
-	load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);
+	load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 
 	if (instruction_type == HI6_BEQ && rt != rs) {
 		/*  If rt != rs, then skip.  */
@@ -1462,7 +1484,10 @@ static int bintrans_write_instruction__branch(unsigned char **addrp,
 		/*  75 01                   jne    155 <skip>  */
 		*a++ = 0x39; *a++ = 0xc3;
 		*a++ = 0x75; skip1 = a; *a++ = 0x00;
-		if (!bintrans_32bit_only) {
+#if 0
+		if (!bintrans_32bit_only)
+#endif
+ {
 			*a++ = 0x39; *a++ = 0xd1;
 			*a++ = 0x75; skip2 = a; *a++ = 0x00;
 		}
@@ -1470,12 +1495,15 @@ static int bintrans_write_instruction__branch(unsigned char **addrp,
 
 	if (instruction_type == HI6_BNE) {
 		/*  If rt != rs, then ok. Otherwise skip.  */
+#if 0
 		if (bintrans_32bit_only) {
 			/*  39 c3                   cmp    %eax,%ebx  */
 			/*  74 xx                   je     <skip>  */
 			*a++ = 0x39; *a++ = 0xc3;
 			*a++ = 0x74; skip2 = a; *a++ = 0x00;
-		} else {
+		} else
+#endif
+ {
 			/*  39 c3                   cmp    %eax,%ebx  */
 			/*  75 06                   jne    156 <bra>  */
 			/*  39 d1                   cmp    %edx,%ecx  */
@@ -1552,7 +1580,7 @@ static int bintrans_write_instruction__branch(unsigned char **addrp,
 	 */
 
 	/*  c7 86 38 30 00 00 01 00 00 00    movl   $0x1,0x3038(%esi)  */
-	ofs = ((size_t)&dummy_cpu.delay_slot) - (size_t)&dummy_cpu;
+	ofs = ((size_t)&dummy_cpu.cd.mips.delay_slot) - (size_t)&dummy_cpu;
 	*a++ = 0xc7; *a++ = 0x86;
 	*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 	*a++ = TO_BE_DELAYED; *a++ = 0; *a++ = 0; *a++ = 0;
@@ -1570,7 +1598,7 @@ static int bintrans_write_instruction__branch(unsigned char **addrp,
 	} else {
 		*a++ = 0x83; *a++ = 0xd2; *a++ = 0xff;
 	}
-	store_eax_edx(&a, &dummy_cpu.delay_jmpaddr);
+	store_eax_edx(&a, &dummy_cpu.cd.mips.delay_jmpaddr);
 
 	if (skip1 != NULL)
 		*skip1 = (size_t)a - (size_t)skip1 - 1;
@@ -1586,8 +1614,8 @@ static int bintrans_write_instruction__branch(unsigned char **addrp,
 /*
  *  bintrans_write_instruction__delayedbranch():
  */
-static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
-	uint32_t *potential_chunk_p, uint32_t *chunks,
+static int bintrans_write_instruction__delayedbranch(struct memory *mem,
+	unsigned char **addrp, uint32_t *potential_chunk_p, uint32_t *chunks,
 	int only_care_about_chunk_p, int p, int forward)
 {
 	unsigned char *a, *skip=NULL, *failskip;
@@ -1600,7 +1628,7 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 		goto try_chunk_p;
 
 	/*  Skip all of this if there is no branch:  */
-	ofs = ((size_t)&dummy_cpu.delay_slot) - (size_t)&dummy_cpu;
+	ofs = ((size_t)&dummy_cpu.cd.mips.delay_slot) - (size_t)&dummy_cpu;
 
 	/*  8b 86 38 30 00 00       mov    0x3038(%esi),%eax  */
 	*a++ = 0x8b; *a++ = 0x86;
@@ -1617,7 +1645,7 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 	 */
 
 	/*  c7 86 38 30 00 00 00 00 00 00    movl   $0x0,0x3038(%esi)  */
-	ofs = ((size_t)&dummy_cpu.delay_slot) - (size_t)&dummy_cpu;
+	ofs = ((size_t)&dummy_cpu.cd.mips.delay_slot) - (size_t)&dummy_cpu;
 	*a++ = 0xc7; *a++ = 0x86;
 	*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 	*a++ = 0; *a++ = 0; *a++ = 0; *a++ = 0;
@@ -1628,17 +1656,17 @@ static int bintrans_write_instruction__delayedbranch(unsigned char **addrp,
 	/*  89 d1                   mov    %edx,%ecx  */
 	*a++ = 0x89; *a++ = 0xc3;
 	*a++ = 0x89; *a++ = 0xd1;
-	load_into_eax_edx(&a, &dummy_cpu.delay_jmpaddr);
+	load_into_eax_edx(&a, &dummy_cpu.cd.mips.delay_jmpaddr);
 	store_eax_edx_into_pc(&a);
 
 try_chunk_p:
 
 	if (potential_chunk_p == NULL) {
-		if (bintrans_32bit_only) {
+		if (mem->bintrans_32bit_only) {
 #if 1
 			/*  8b 86 78 56 34 12       mov    0x12345678(%esi),%eax  */
 			/*  ff e0                   jmp    *%eax  */
-			ofs = ((size_t)&dummy_cpu.bintrans_jump_to_32bit_pc) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_jump_to_32bit_pc) - (size_t)&dummy_cpu;
 			*a++ = 0x8b; *a++ = 0x86;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 			*a++ = 0xff; *a++ = 0xe0;
@@ -1671,7 +1699,7 @@ try_chunk_p:
 			 *
 			 *  8b 8e 34 12 00 00       mov    0x1234(%esi),%ecx
 			 */
-			ofs = ((size_t)&dummy_cpu.vaddr_to_hostaddr_table0) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.vaddr_to_hostaddr_table0) - (size_t)&dummy_cpu;
 			*a++ = 0x8b; *a++ = 0x8e;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 
@@ -1694,11 +1722,11 @@ try_chunk_p:
 			*a++ = 0x81; *a++ = 0xe3; *a++ = 0xfc; *a++ = 0x0f; *a++ = 0; *a++ = 0;
 
 			/*
-			 *  ecx = vaddr_to_hostaddr_table0[a][b].chunks
+			 *  ecx = vaddr_to_hostaddr_table0[a][b].cd.mips.chunks
 			 *
 			 *  8b 8c 19 56 34 12 00    mov    0x123456(%ecx,%ebx,1),%ecx
 			 */
-			ofs = (size_t)&dummy_vth32_table.bintrans_chunks[0]
+			ofs = (size_t)&dummy_vth32_table.cd.mips.bintrans_chunks[0]
 			    - (size_t)&dummy_vth32_table;
 
 			*a++ = 0x8b; *a++ = 0x8c;  *a++ = 0x19;
@@ -1737,7 +1765,7 @@ try_chunk_p:
 
 			/*  03 86 78 56 34 12       add    0x12345678(%esi),%eax  */
 			/*  ff e0                   jmp    *%eax  */
-			ofs = ((size_t)&dummy_cpu.chunk_base_address) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.chunk_base_address) - (size_t)&dummy_cpu;
 			*a++ = 0x03; *a++ = 0x86;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 			*a++ = 0xff; *a++ = 0xe0;
@@ -1808,7 +1836,7 @@ try_chunk_p:
 
 			/*  03 86 78 56 34 12       add    0x12345678(%esi),%eax  */
 			/*  ff e0                   jmp    *%eax  */
-			ofs = ((size_t)&dummy_cpu.chunk_base_address) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.chunk_base_address) - (size_t)&dummy_cpu;
 			*a++ = 0x03; *a++ = 0x86;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 			*a++ = 0xff; *a++ = 0xe0;
@@ -1854,7 +1882,8 @@ try_chunk_p:
 
 		/*  Case 1:  */
 		/*  printf("%08x ", *potential_chunk_p);  */
-		i386_addr = *potential_chunk_p + (size_t)translation_code_chunk_space;
+		i386_addr = *potential_chunk_p +
+		    (size_t)mem->translation_code_chunk_space;
 		i386_addr = i386_addr - ((size_t)a + 5);
 		if ((*potential_chunk_p) != 0) {
 			*a++ = 0xe9;
@@ -1865,7 +1894,7 @@ try_chunk_p:
 		} else {
 			/*  Case 2:  */
 
-			bintrans_register_potential_quick_jump(a, p);
+			bintrans_register_potential_quick_jump(mem, a, p);
 
 			i386_addr = (size_t)potential_chunk_p;
 
@@ -1888,7 +1917,7 @@ try_chunk_p:
 
 			/*  03 86 78 56 34 12       add    0x12345678(%esi),%eax  */
 			/*  ff e0                   jmp    *%eax  */
-			ofs = ((size_t)&dummy_cpu.chunk_base_address) - (size_t)&dummy_cpu;
+			ofs = ((size_t)&dummy_cpu.cd.mips.chunk_base_address) - (size_t)&dummy_cpu;
 			*a++ = 0x03; *a++ = 0x86;
 			*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 			*a++ = 0xff; *a++ = 0xe0;
@@ -1906,8 +1935,9 @@ try_chunk_p:
 /*
  *  bintrans_write_instruction__loadstore():
  */
-static int bintrans_write_instruction__loadstore(unsigned char **addrp,
-	int rt, int imm, int rs, int instruction_type, int bigendian)
+static int bintrans_write_instruction__loadstore(struct memory *mem,
+	unsigned char **addrp, int rt, int imm, int rs,
+	int instruction_type, int bigendian)
 {
 	unsigned char *a, *retfail, *generic64bit, *doloadstore,
 	    *okret0, *okret1, *okret2, *skip;
@@ -1953,17 +1983,17 @@ static int bintrans_write_instruction__loadstore(unsigned char **addrp,
 
 	a = *addrp;
 
-	if (bintrans_32bit_only)
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rs]);
+	if (mem->bintrans_32bit_only)
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 	else
-		load_into_eax_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 
 	if (imm & 0x8000) {
 		/*  05 34 f2 ff ff          add    $0xfffff234,%eax  */
 		/*  83 d2 ff                adc    $0xffffffff,%edx  */
 		*a++ = 5;
 		*a++ = imm; *a++ = imm >> 8; *a++ = 0xff; *a++ = 0xff;
-		if (!bintrans_32bit_only) {
+		if (!mem->bintrans_32bit_only) {
 			*a++ = 0x83; *a++ = 0xd2; *a++ = 0xff;
 		}
 	} else {
@@ -1971,7 +2001,7 @@ static int bintrans_write_instruction__loadstore(unsigned char **addrp,
 		/*  83 d2 00                adc    $0x0,%edx  */
 		*a++ = 5;
 		*a++ = imm; *a++ = imm >> 8; *a++ = 0; *a++ = 0;
-		if (!bintrans_32bit_only) {
+		if (!mem->bintrans_32bit_only) {
 			*a++ = 0x83; *a++ = 0xd2; *a++ = 0;
 		}
 	}
@@ -2034,7 +2064,7 @@ static int bintrans_write_instruction__loadstore(unsigned char **addrp,
 
 	/*  Here, edx:eax = vaddr  */
 
-	if (bintrans_32bit_only) {
+	if (mem->bintrans_32bit_only) {
 		/*  Call the quick lookup routine:  */
 		ofs = (size_t)bintrans_i386_loadstore_32bit;
 		ofs = ofs - ((size_t)a + 5);
@@ -2170,7 +2200,7 @@ TODO: top 33 bits!!!!!!!
 		*a++ = 0x56;
 
 		/*  eax = points to the right function  */
-		ofs = ((size_t)&dummy_cpu.fast_vaddr_to_hostaddr) - (size_t)&dummy_cpu;
+		ofs = ((size_t)&dummy_cpu.cd.mips.fast_vaddr_to_hostaddr) - (size_t)&dummy_cpu;
 		*a++ = 0x8b; *a++ = 0x86;
 		*a++ = ofs; *a++ = ofs >> 8; *a++ = ofs >> 16; *a++ = ofs >> 24;
 
@@ -2198,9 +2228,9 @@ TODO: top 33 bits!!!!!!!
 
 	if (!load) {
 		if (alignment >= 7)
-			load_into_eax_edx(&a, &dummy_cpu.gpr[rt]);
+			load_into_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 		else
-			load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rt]);
+			load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 	}
 
 	switch (instruction_type) {
@@ -2258,7 +2288,7 @@ TODO: top 33 bits!!!!!!!
 		break;
 
 	case HI6_LWL:
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 		/*  05 34 f2 ff ff          add    $0xfffff234,%eax  */
 		*a++ = 5;
 		*a++ = imm; *a++ = imm >> 8; *a++ = 0xff; *a++ = 0xff;
@@ -2267,7 +2297,7 @@ TODO: top 33 bits!!!!!!!
 		/*  89 c3                   mov    %eax,%ebx  */
 		*a++ = 0x89; *a++ = 0xc3;
 
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 
 		/*  ALIGNED LOAD:  */
 		/*  8b 11                   mov    (%ecx),%edx  */
@@ -2357,7 +2387,7 @@ TODO: top 33 bits!!!!!!!
 		break;
 
 	case HI6_LWR:
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 		/*  05 34 f2 ff ff          add    $0xfffff234,%eax  */
 		*a++ = 5;
 		*a++ = imm; *a++ = imm >> 8; *a++ = 0xff; *a++ = 0xff;
@@ -2366,7 +2396,7 @@ TODO: top 33 bits!!!!!!!
 		/*  89 c3                   mov    %eax,%ebx  */
 		*a++ = 0x89; *a++ = 0xc3;
 
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 
 		/*  ALIGNED LOAD:  */
 		/*  8b 11                   mov    (%ecx),%edx  */
@@ -2475,7 +2505,7 @@ TODO: top 33 bits!!!!!!!
 		break;
 
 	case HI6_SWL:
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 		/*  05 34 f2 ff ff          add    $0xfffff234,%eax  */
 		*a++ = 5;
 		*a++ = imm; *a++ = imm >> 8; *a++ = 0xff; *a++ = 0xff;
@@ -2484,7 +2514,7 @@ TODO: top 33 bits!!!!!!!
 		/*  89 c3                   mov    %eax,%ebx  */
 		*a++ = 0x89; *a++ = 0xc3;
 
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 
 		/*  ALIGNED LOAD:  */
 		/*  8b 11                   mov    (%ecx),%edx  */
@@ -2578,7 +2608,7 @@ TODO: top 33 bits!!!!!!!
 		break;
 
 	case HI6_SWR:
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rs]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rs]);
 		/*  05 34 f2 ff ff          add    $0xfffff234,%eax  */
 		*a++ = 5;
 		*a++ = imm; *a++ = imm >> 8; *a++ = 0xff; *a++ = 0xff;
@@ -2587,7 +2617,7 @@ TODO: top 33 bits!!!!!!!
 		/*  89 c3                   mov    %eax,%ebx  */
 		*a++ = 0x89; *a++ = 0xc3;
 
-		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.gpr[rt]);
+		load_into_eax_dont_care_about_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 
 		/*  ALIGNED LOAD:  */
 		/*  8b 11                   mov    (%ecx),%edx  */
@@ -2686,7 +2716,7 @@ TODO: top 33 bits!!!!!!!
 	}
 
 	if (load && rt != 0)
-		store_eax_edx(&a, &dummy_cpu.gpr[rt]);
+		store_eax_edx(&a, &dummy_cpu.cd.mips.gpr[rt]);
 
 	*addrp = a;
 	bintrans_write_pc_inc(addrp);
@@ -2701,17 +2731,17 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	int itype)
 {
 	unsigned char *a;
-	int ofs = 0;	/*  avoid a compiler warning  */
+	int ofs;
 
 	switch (itype) {
-	case TLB_TLBP:
-	case TLB_TLBR:
-	case TLB_TLBWR:
-	case TLB_TLBWI:
-	case TLB_RFE:
-	case TLB_ERET:
-	case TLB_SYSCALL:
-	case TLB_BREAK:
+	case CALL_TLBP:
+	case CALL_TLBR:
+	case CALL_TLBWR:
+	case CALL_TLBWI:
+	case CALL_RFE:
+	case CALL_ERET:
+	case CALL_SYSCALL:
+	case CALL_BREAK:
 		break;
 	default:
 		return 0;
@@ -2723,34 +2753,48 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	*a++ = 0x89; *a++ = 0xbe; *a++ = ofs_pc&255;
 	*a++ = (ofs_pc>>8)&255; *a++ = (ofs_pc>>16)&255;
 	*a++ = (ofs_pc>>24)&255;	/*  mov    %edi,pc(%esi)  */
+
 	*a++ = 0x89; *a++ = 0xbe; *a++ = ofs_pc_last&255;
 	*a++ = (ofs_pc_last>>8)&255; *a++ = (ofs_pc_last>>16)&255;
 	*a++ = (ofs_pc_last>>24)&255;	/*  mov    %edi,pc_last(%esi)  */
 
+	/*  ... and make sure that the high 32 bits are ALSO in pc_last:  */
+	/*  8b 86 38 12 00 00       mov    0x1238(%esi),%eax  */
+	ofs = ofs_pc + 4;
+	*a++ = 0x8b; *a++ = 0x86; *a++ = ofs&255;
+	*a++ = (ofs>>8)&255; *a++ = (ofs>>16)&255;
+	*a++ = (ofs>>24)&255;	/*  mov    %edi,pc(%esi)  */
+
+	/*  89 86 34 12 00 00       mov    %eax,0x1234(%esi)  */
+	ofs = ofs_pc_last + 4;
+	*a++ = 0x89; *a++ = 0x86; *a++ = ofs&255;
+	*a++ = (ofs>>8)&255; *a++ = (ofs>>16)&255;
+	*a++ = (ofs>>24)&255;	/*  mov    %edi,pc(%esi)  */
+
 	switch (itype) {
-	case TLB_TLBP:
-	case TLB_TLBR:
+	case CALL_TLBP:
+	case CALL_TLBR:
 		/*  push readflag  */
-		*a++ = 0x6a; *a++ = (itype == TLB_TLBR);
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_tlbpr) - (size_t)&dummy_cpu;
+		*a++ = 0x6a; *a++ = (itype == CALL_TLBR);
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_tlbpr) - (size_t)&dummy_cpu;
 		break;
-	case TLB_TLBWR:
-	case TLB_TLBWI:
+	case CALL_TLBWR:
+	case CALL_TLBWI:
 		/*  push randomflag  */
-		*a++ = 0x6a; *a++ = (itype == TLB_TLBWR);
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_tlbwri) - (size_t)&dummy_cpu;
+		*a++ = 0x6a; *a++ = (itype == CALL_TLBWR);
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_tlbwri) - (size_t)&dummy_cpu;
 		break;
-	case TLB_SYSCALL:
-	case TLB_BREAK:
+	case CALL_SYSCALL:
+	case CALL_BREAK:
 		/*  push randomflag  */
-		*a++ = 0x6a; *a++ = (itype == TLB_BREAK? EXCEPTION_BP : EXCEPTION_SYS);
-		ofs = ((size_t)&dummy_cpu.bintrans_simple_exception) - (size_t)&dummy_cpu;
+		*a++ = 0x6a; *a++ = (itype == CALL_BREAK? EXCEPTION_BP : EXCEPTION_SYS);
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_simple_exception) - (size_t)&dummy_cpu;
 		break;
-	case TLB_RFE:
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_rfe) - (size_t)&dummy_cpu;
+	case CALL_RFE:
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_rfe) - (size_t)&dummy_cpu;
 		break;
-	case TLB_ERET:
-		ofs = ((size_t)&dummy_cpu.bintrans_fast_eret) - (size_t)&dummy_cpu;
+	case CALL_ERET:
+		ofs = ((size_t)&dummy_cpu.cd.mips.bintrans_fast_eret) - (size_t)&dummy_cpu;
 		break;
 	}
 
@@ -2765,8 +2809,8 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	*a++ = 0xff; *a++ = 0xd0;
 
 	switch (itype) {
-	case TLB_RFE:
-	case TLB_ERET:
+	case CALL_RFE:
+	case CALL_ERET:
 		/*  83 c4 04                add    $4,%esp  */
 		*a++ = 0x83; *a++ = 0xc4; *a++ = 4;
 		break;
@@ -2784,9 +2828,9 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	*addrp = a;
 
 	switch (itype) {
-	case TLB_ERET:
-	case TLB_SYSCALL:
-	case TLB_BREAK:
+	case CALL_ERET:
+	case CALL_SYSCALL:
+	case CALL_BREAK:
 		break;
 	default:
 		bintrans_write_pc_inc(addrp);
@@ -2795,3 +2839,11 @@ static int bintrans_write_instruction__tlb_rfe_etc(unsigned char **addrp,
 	return 1;
 }
 
+
+/*
+ *  bintrans_backend_init():
+ */
+static void bintrans_backend_init(void)
+{
+}
+
