@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: dec_prom.c,v 1.40 2005/01/09 01:55:30 debug Exp $
+ *  $Id: dec_prom.c,v 1.56 2005/02/18 07:22:31 debug Exp $
  *
  *  DECstation PROM emulation.
  */
@@ -37,12 +37,13 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 
-#include "misc.h"
-
 #include "console.h"
+#include "cpu.h"
+#include "cpu_mips.h"
 #include "diskimage.h"
-#include "emul.h"
+#include "machine.h"
 #include "memory.h"
+#include "misc.h"
 
 #include "dec_prom.h"
 #include "dec_5100.h"
@@ -54,6 +55,21 @@
 extern int quiet_mode;
 
 
+/*
+ *  mem_readchar():
+ *
+ *  Reads a byte from emulated RAM, using a MIPS register as a base address.
+ *  (Helper function.)
+ */
+static unsigned char mem_readchar(struct cpu *cpu, int regbase, int offset)
+	{
+	unsigned char ch;
+	cpu->memory_rw(cpu, cpu->mem, cpu->cd.mips.gpr[regbase] + offset,
+	    &ch, sizeof(ch), MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
+	return ch;
+}
+
+
 /*
  *  dec_jumptable_func():
  *
@@ -84,18 +100,20 @@ int dec_jumptable_func(struct cpu *cpu, int vector)
 	switch (vector) {
 	case 0x0:	/*  reset()  */
 		/*  TODO  */
-		cpu->emul->exit_without_entering_debugger = 1;
+		cpu->machine->exit_without_entering_debugger = 1;
 		cpu->running = 0;
+		cpu->dead = 1;
 		break;
 	case 0x10:	/*  restart()  */
 		/*  TODO  */
-		cpu->emul->exit_without_entering_debugger = 1;
+		cpu->machine->exit_without_entering_debugger = 1;
 		cpu->running = 0;
+		cpu->dead = 1;
 		break;
 	case 0x18:	/*  reinit()  */
 		/*  TODO  */
-		cpu->emul->exit_without_entering_debugger = 1;
-		cpu->gpr[GPR_V0] = 0;
+		cpu->machine->exit_without_entering_debugger = 1;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x30:	/*  open()  */
 		/*
@@ -104,36 +122,43 @@ int dec_jumptable_func(struct cpu *cpu, int vector)
 		 *  is ignored, and a file handle value of 1 is returned.
 		 */
 		if (file_opened) {
-			fatal("\ndec_jumptable_func(): opening more than one file isn't supported yet.\n");
+			fatal("\ndec_jumptable_func(): opening more than one "
+			    "file isn't supported yet.\n");
 			cpu->running = 0;
+			cpu->dead = 1;
 		}
 		file_opened = 1;
-		cpu->gpr[GPR_V0] = 1;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 1;
 		break;
 	case 0x38:	/*  read(handle, ptr, length)  */
-		cpu->gpr[GPR_V0] = -1;
-		if ((int32_t)cpu->gpr[GPR_A2] > 0) {
-			int disk_id = diskimage_bootdev();
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = -1;
+		if ((int32_t)cpu->cd.mips.gpr[MIPS_GPR_A2] > 0) {
+			int disk_id = diskimage_bootdev(cpu->machine);
 			int res;
 			unsigned char *tmp_buf;
 
-			tmp_buf = malloc(cpu->gpr[GPR_A2]);
+			tmp_buf = malloc(cpu->cd.mips.gpr[MIPS_GPR_A2]);
 			if (tmp_buf == NULL) {
-				fprintf(stderr, "[ ***  Out of memory in dec_prom.c, allocating %i bytes ]\n", (int)cpu->gpr[GPR_A2]);
+				fprintf(stderr, "[ ***  Out of memory in "
+				    "dec_prom.c, allocating %i bytes ]\n",
+				    (int)cpu->cd.mips.gpr[MIPS_GPR_A2]);
 				break;
 			}
 
-			res = diskimage_access(disk_id, 0, current_file_offset,
-			    tmp_buf, cpu->gpr[GPR_A2]);
+			res = diskimage_access(cpu->machine, disk_id, 0,
+			    current_file_offset, tmp_buf,
+			    cpu->cd.mips.gpr[MIPS_GPR_A2]);
 
 			/*  If the transfer was successful, transfer the data
 			    to emulated memory:  */
 			if (res) {
-				uint64_t dst = cpu->gpr[GPR_A1];
+				uint64_t dst = cpu->cd.mips.gpr[MIPS_GPR_A1];
 				store_buf(cpu, dst, (char *)tmp_buf,
-				    cpu->gpr[GPR_A2]);
-				cpu->gpr[GPR_V0] = cpu->gpr[GPR_A2];
-				current_file_offset += cpu->gpr[GPR_A2];
+				    cpu->cd.mips.gpr[MIPS_GPR_A2]);
+				cpu->cd.mips.gpr[MIPS_GPR_V0] =
+				    cpu->cd.mips.gpr[MIPS_GPR_A2];
+				current_file_offset +=
+				    cpu->cd.mips.gpr[MIPS_GPR_A2];
 			}
 
 			free(tmp_buf);
@@ -141,25 +166,28 @@ int dec_jumptable_func(struct cpu *cpu, int vector)
 		break;
 	case 0x58:	/*  lseek(handle, offset[, whence])  */
 		/*  TODO  */
-		if (cpu->gpr[GPR_A2] == 0)
-			current_file_offset = cpu->gpr[GPR_A1];
+		if (cpu->cd.mips.gpr[MIPS_GPR_A2] == 0)
+			current_file_offset = cpu->cd.mips.gpr[MIPS_GPR_A1];
 		else
-			fatal("WARNING! Unimplemented whence in dec_jumptable_func()\n");
-		cpu->gpr[GPR_V0] = 0;
+			fatal("WARNING! Unimplemented whence in "
+			    "dec_jumptable_func()\n");
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x68:	/*  putchar()  */
-		console_putchar(cpu->gpr[GPR_A0]);
+		console_putchar(cpu->machine->main_console_handle,
+		    cpu->cd.mips.gpr[MIPS_GPR_A0]);
 		break;
 	case 0x88:	/*  printf()  */
 		return 0x30;
 	case 0x108:	/*  getenv2()  */
 		return 0x64;
 	default:
-		cpu_register_dump(cpu, 1, 0x1);
+		cpu_register_dump(cpu->machine, cpu, 1, 0x1);
 		printf("a0 points to: ");
 		for (i=0; i<40; i++) {
 			unsigned char ch = '\0';
-			memory_rw(cpu, cpu->mem, cpu->gpr[GPR_A0] + i, &ch,
+			cpu->memory_rw(cpu, cpu->mem,
+			    cpu->cd.mips.gpr[MIPS_GPR_A0] + i, &ch,
 			    sizeof(ch), MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
 			if (ch >= ' ' && ch < 126)
 				printf("%c", ch);
@@ -167,9 +195,10 @@ int dec_jumptable_func(struct cpu *cpu, int vector)
 				printf("[%02x]", ch);
 		}
 		printf("\n");
-		fatal("PROM emulation: unimplemented JUMP TABLE vector 0x%x (decimal function %i)\n",
-		    vector, vector/8);
+		fatal("PROM emulation: unimplemented JUMP TABLE vector "
+		    "0x%x (decimal function %i)\n", vector, vector/8);
 		cpu->running = 0;
+		cpu->dead = 1;
 	}
 
 	return 0;
@@ -204,11 +233,11 @@ int dec_jumptable_func(struct cpu *cpu, int vector)
  *	0xa8	execute_cmd()
  *	0xac	rex()
  */
-void decstation_prom_emul(struct cpu *cpu)
+int decstation_prom_emul(struct cpu *cpu)
 {
 	int i, j, ch, argreg, argdata;
-	int vector = cpu->pc & 0xfff;
-	int callback = (cpu->pc & 0xf000)? 1 : 0;
+	int vector = cpu->cd.mips.pc & 0xfff;
+	int callback = (cpu->cd.mips.pc & 0xf000)? 1 : 0;
 	unsigned char buf[100];
 	unsigned char ch1, ch2, ch3;
 	uint64_t tmpaddr, slot_base = 0x10000000, slot_size = 0;
@@ -216,7 +245,7 @@ void decstation_prom_emul(struct cpu *cpu)
 	if (!callback) {
 		vector = dec_jumptable_func(cpu, vector);
 		if (vector == 0)
-			return;
+			return 1;
 	} else {
 		/*  Vector number is n*4, PC points to n*8.  */
 		vector /= 2;
@@ -226,35 +255,40 @@ void decstation_prom_emul(struct cpu *cpu)
 	case 0x0c:		/*  strcmp():  */
 		i = j = 0;
 		do {
-			ch1 = read_char_from_memory(cpu, GPR_A0, i++);
-			ch2 = read_char_from_memory(cpu, GPR_A1, j++);
+			ch1 = mem_readchar(cpu, MIPS_GPR_A0, i++);
+			ch2 = mem_readchar(cpu, MIPS_GPR_A1, j++);
 		} while (ch1 == ch2 && ch1 != '\0');
 
 		/*  If ch1=='\0', then strings are equal.  */
 		if (ch1 == '\0')
-			cpu->gpr[GPR_V0] = 0;
+			cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		if ((signed char)ch1 > (signed char)ch2)
-			cpu->gpr[GPR_V0] = 1;
+			cpu->cd.mips.gpr[MIPS_GPR_V0] = 1;
 		if ((signed char)ch1 < (signed char)ch2)
-			cpu->gpr[GPR_V0] = -1;
+			cpu->cd.mips.gpr[MIPS_GPR_V0] = -1;
 		break;
 	case 0x14:		/*  strlen():  */
 		i = 0;
 		do {
-			ch2 = read_char_from_memory(cpu, GPR_A0, i++);
+			ch2 = mem_readchar(cpu, MIPS_GPR_A0, i++);
 		} while (ch2 != 0);
-		cpu->gpr[GPR_V0] = i - 1;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = i - 1;
 		break;
 	case 0x24:		/*  getchar()  */
 		/*  debug("[ DEC PROM getchar() ]\n");  */
-		cpu->gpr[GPR_V0] = console_readchar();
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = console_readchar(
+		    cpu->machine->main_console_handle);
 		break;
 	case 0x28:		/*  gets()  */
 		/*  debug("[ DEC PROM gets() ]\n");  */
-		tmpaddr = cpu->gpr[GPR_A0];
+		tmpaddr = cpu->cd.mips.gpr[MIPS_GPR_A0];
 		i = 0;
+
+		/*  TODO: Make this not hang (block) the entire emulator  */
+
 		do {
-			while ((ch = console_readchar()) < 1)
+			while ((ch = console_readchar(
+			    cpu->machine->main_console_handle)) < 1)
 				;
 			if (ch == '\r')
 				ch = '\n';
@@ -262,12 +296,16 @@ void decstation_prom_emul(struct cpu *cpu)
 
 			if (ch == '\b') {
 				if (i > 0) {
-					console_putchar(ch2);
-					console_putchar(' ');
-					console_putchar(ch2);
+					console_putchar(cpu->machine->
+					    main_console_handle, ch2);
+					console_putchar(cpu->machine->
+					    main_console_handle, ' ');
+					console_putchar(cpu->machine->
+					    main_console_handle, ch2);
 				}
 			} else
-				console_putchar(ch2);
+				console_putchar(cpu->machine->
+				    main_console_handle, ch2);
 
 			fflush(stdout);
 
@@ -275,7 +313,8 @@ void decstation_prom_emul(struct cpu *cpu)
 				/*  It seems that trailing newlines
 				    are not included in the buffer.  */
 			} else if (ch != '\b') {
-				memory_rw(cpu, cpu->mem, cpu->gpr[GPR_A0] + i,
+				cpu->memory_rw(cpu, cpu->mem,
+				    cpu->cd.mips.gpr[MIPS_GPR_A0] + i,
 				    &ch2, sizeof(ch2), MEM_WRITE,
 				    CACHE_DATA | NO_EXCEPTIONS);
 				i++;
@@ -287,100 +326,131 @@ void decstation_prom_emul(struct cpu *cpu)
 
 		/*  Trailing nul-byte:  */
 		ch2 = '\0';
-		memory_rw(cpu, cpu->mem, cpu->gpr[GPR_A0] + i, &ch2,
-		    sizeof(ch2), MEM_WRITE, CACHE_DATA | NO_EXCEPTIONS);
+		cpu->memory_rw(cpu, cpu->mem, cpu->cd.mips.gpr[MIPS_GPR_A0] +
+		    i, &ch2, sizeof(ch2), MEM_WRITE,
+		    CACHE_DATA | NO_EXCEPTIONS);
 
 		/*  Return the input argument:  */
-		cpu->gpr[GPR_V0] = cpu->gpr[GPR_A0];
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = cpu->cd.mips.gpr[MIPS_GPR_A0];
 		break;
 	case 0x2c:		/*  puts()  */
 		i = 0;
-		while ((ch = read_char_from_memory(cpu, GPR_A0, i++)) != '\0')
-			console_putchar(ch);
-		console_putchar('\n');
-		cpu->gpr[GPR_V0] = 0;
+		while ((ch = mem_readchar(cpu, MIPS_GPR_A0, i++)) != '\0')
+			console_putchar(cpu->machine->main_console_handle, ch);
+		console_putchar(cpu->machine->main_console_handle, '\n');
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x30:		/*  printf()  */
-		if (cpu->emul->register_dump || cpu->emul->instruction_trace)
+		if (cpu->machine->register_dump ||
+		    cpu->machine->instruction_trace)
 			debug("PROM printf(0x%08lx): \n",
-			    (long)cpu->gpr[GPR_A0]);
+			    (long)cpu->cd.mips.gpr[MIPS_GPR_A0]);
 
-		i = 0; ch = -1; argreg = GPR_A1;
+		i = 0; ch = -1; argreg = MIPS_GPR_A1;
 		while (ch != '\0') {
-			ch = read_char_from_memory(cpu, GPR_A0, i++);
+			char printfbuf[8000];
+			int x;
+
+			printfbuf[0] = printfbuf[sizeof(printfbuf)-1] = '\0';
+
+			ch = mem_readchar(cpu, MIPS_GPR_A0, i++);
 			switch (ch) {
 			case '%':
 				ch = '0';
 				while (ch >= '0' && ch <= '9')
-					ch = read_char_from_memory(
-					    cpu, GPR_A0, i++);
+					ch = mem_readchar(cpu,
+					    MIPS_GPR_A0, i++);
+
 				switch (ch) {
 				case '%':
-					printf("%%");
+					strcpy(printfbuf, "%%");
 					break;
 				case 'c':
 				case 'd':
 				case 's':
 				case 'x':
 					/*  Get argument:  */
-					if (argreg > GPR_A3) {
+					if (argreg > MIPS_GPR_A3) {
 #if 1
 						/*  Linux booters seem to go
 						    over the edge sometimes: */
 						ch = '\0';
-						printf("[...]\n");
+						strcpy(printfbuf, "[...]\n");
 #else
-						printf("[ decstation_prom_emul(): too many arguments ]");
-						argreg = GPR_A3;	/*  This reuses the last argument,
-								which is utterly incorrect. (TODO)  */
+						printf("[ decstation_prom_emul"
+						    "(): too many arguments ]");
+						/*  This reuses the last arg,
+						    which is utterly incorrect.
+						    (TODO)  */
+						argreg = MIPS_GPR_A3;
 #endif
 					}
-					ch2 = argdata = cpu->gpr[argreg];
+
+					ch2 = argdata =
+					    cpu->cd.mips.gpr[argreg];
 
 					switch (ch) {
 					case 'c':
-						printf("%c", ch2);
+						sprintf(printfbuf, "%c", ch2);
 						break;
 					case 'd':
-						printf("%d", argdata);
+						sprintf(printfbuf, "%d",
+						    argdata);
 						break;
 					case 'x':
-						printf("%x", argdata);
+						sprintf(printfbuf, "%x",
+						    argdata);
 						break;
 					case 's':
 						/*  Print a "%s" string.  */
 						j = 0; ch3 = '\n';
 						while (ch2) {
-							ch2 = read_char_from_memory(cpu, argreg, j++);
+							ch2 = mem_readchar(cpu,
+							    argreg, j++);
 							if (ch2) {
-								printf("%c", ch2);
+								snprintf(
+								    printfbuf +
+								    strlen(
+								    printfbuf),
+								    sizeof(
+								    printfbuf)-
+								    1-strlen(
+								    printfbuf),
+								    "%c", ch2);
 								ch3 = ch2;
 							}
 						}
-						/*  TODO:  without this newline, output looks ugly  */
-						/*  printf("\n");  */
 						break;
 					}
 					argreg ++;
 					break;
 				default:
-					printf("[ unknown printf format char '%c' ]", ch);
+					printf("[ unknown printf format char"
+					    " '%c' ]", ch);
 				}
 				break;
 			case '\0':
 				break;
 			default:
-				printf("%c", ch);
+				sprintf(printfbuf, "%c", ch);
 			}
+
+			printfbuf[sizeof(printfbuf)-1] = '\0';
+
+			for (x=0; x<strlen(printfbuf); x++)
+				console_putchar(cpu->machine->
+				    main_console_handle, printfbuf[x]);
 		}
-		if (cpu->emul->register_dump || cpu->emul->instruction_trace)
+		if (cpu->machine->register_dump ||
+		    cpu->machine->instruction_trace)
 			debug("\n");
 		fflush(stdout);
-		cpu->gpr[GPR_V0] = 0;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x54:		/*  bootinit()  */
-		/*  debug("[ DEC PROM bootinit(0x%08x): TODO ]\n", (int)cpu->gpr[GPR_A0]);  */
-		cpu->gpr[GPR_V0] = 0;
+		/*  debug("[ DEC PROM bootinit(0x%08x): TODO ]\n",
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A0]);  */
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x58:		/*  bootread(int b, void *buffer, int n)  */
 		/*
@@ -392,34 +462,40 @@ void decstation_prom_emul(struct cpu *cpu)
 		 *  TODO: Return value? NetBSD thinks that 0 is ok.
 		 */
 		debug("[ DEC PROM bootread(0x%x, 0x%08x, 0x%x) ]\n",
-		    (int)cpu->gpr[GPR_A0], (int)cpu->gpr[GPR_A1], (int)cpu->gpr[GPR_A2]);
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A0],
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A1],
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A2]);
 
-		cpu->gpr[GPR_V0] = 0;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 
-		if ((int32_t)cpu->gpr[GPR_A2] > 0) {
-			int disk_id = diskimage_bootdev();
+		if ((int32_t)cpu->cd.mips.gpr[MIPS_GPR_A2] > 0) {
+			int disk_id = diskimage_bootdev(cpu->machine);
 			int res;
 			unsigned char *tmp_buf;
 
-			tmp_buf = malloc(cpu->gpr[GPR_A2]);
+			tmp_buf = malloc(cpu->cd.mips.gpr[MIPS_GPR_A2]);
 			if (tmp_buf == NULL) {
-				fprintf(stderr, "[ ***  Out of memory in dec_prom.c, allocating %i bytes ]\n", (int)cpu->gpr[GPR_A2]);
+				fprintf(stderr, "[ ***  Out of memory in "
+				    "dec_prom.c, allocating %i bytes ]\n",
+				    (int)cpu->cd.mips.gpr[MIPS_GPR_A2]);
 				break;
 			}
 
-			res = diskimage_access(disk_id, 0, cpu->gpr[GPR_A0] *
-			    512, tmp_buf, cpu->gpr[GPR_A2]);
+			res = diskimage_access(cpu->machine, disk_id, 0,
+			    cpu->cd.mips.gpr[MIPS_GPR_A0] * 512, tmp_buf,
+			    cpu->cd.mips.gpr[MIPS_GPR_A2]);
 
 			/*  If the transfer was successful, transfer the data
 			    to emulated memory:  */
 			if (res) {
-				uint64_t dst = cpu->gpr[GPR_A1];
+				uint64_t dst = cpu->cd.mips.gpr[MIPS_GPR_A1];
 				if (dst < 0x80000000ULL)
 					dst |= 0x80000000;
 
 				store_buf(cpu, dst, (char *)tmp_buf,
-				    cpu->gpr[GPR_A2]);
-				cpu->gpr[GPR_V0] = cpu->gpr[GPR_A2];
+				    cpu->cd.mips.gpr[MIPS_GPR_A2]);
+				cpu->cd.mips.gpr[MIPS_GPR_V0] =
+				    cpu->cd.mips.gpr[MIPS_GPR_A2];
 			}
 
 			free(tmp_buf);
@@ -428,7 +504,8 @@ void decstation_prom_emul(struct cpu *cpu)
 	case 0x64:		/*  getenv()  */
 		/*  Find the environment variable given by a0:  */
 		for (i=0; i<sizeof(buf); i++)
-			memory_rw(cpu, cpu->mem, cpu->gpr[GPR_A0] + i, &buf[i],
+			cpu->memory_rw(cpu, cpu->mem,
+			    cpu->cd.mips.gpr[MIPS_GPR_A0] + i, &buf[i],
 			    sizeof(char), MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
 		buf[sizeof(buf)-1] = '\0';
 		debug("[ DEC PROM getenv(\"%s\") ]\n", buf);
@@ -436,132 +513,161 @@ void decstation_prom_emul(struct cpu *cpu)
 			/*  Matching string at offset i?  */
 			int nmatches = 0;
 			for (j=0; j<strlen((char *)buf); j++) {
-				memory_rw(cpu, cpu->mem, (uint64_t)
+				cpu->memory_rw(cpu, cpu->mem, (uint64_t)
 				    (DEC_PROM_STRINGS + i + j), &ch2,
 				    sizeof(char), MEM_READ, CACHE_DATA |
 				    NO_EXCEPTIONS);
 				if (ch2 == buf[j])
 					nmatches++;
 			}
-			memory_rw(cpu, cpu->mem, (uint64_t)(DEC_PROM_STRINGS
+			cpu->memory_rw(cpu, cpu->mem,
+			    (uint64_t)(DEC_PROM_STRINGS
 			    + i + strlen((char *)buf)), &ch2, sizeof(char),
 			    MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
 			if (nmatches == strlen((char *)buf) && ch2 == '=') {
-				cpu->gpr[GPR_V0] = DEC_PROM_STRINGS + i + strlen((char *)buf) + 1;
-				return;
+				cpu->cd.mips.gpr[MIPS_GPR_V0] =
+				    DEC_PROM_STRINGS + i +
+				    strlen((char *)buf) + 1;
+				return 1;
 			}
 		}
 		/*  Return NULL if string wasn't found.  */
-		fatal("[ DEC PROM getenv(\"%s\"): WARNING: Not in environment! ]\n", buf);
-		cpu->gpr[GPR_V0] = 0;
+		fatal("[ DEC PROM getenv(\"%s\"): WARNING: Not in "
+		    "environment! ]\n", buf);
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x6c:		/*  ulong slot_address(int sn)  */
-		debug("[ DEC PROM slot_address(%i) ]\n", (int)cpu->gpr[GPR_A0]);
+		debug("[ DEC PROM slot_address(%i) ]\n",
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A0]);
 		/*  TODO:  This is too hardcoded.  */
 		/*  TODO 2:  Should these be physical or virtual addresses?  */
-		switch (cpu->emul->machine) {
-		case MACHINE_3MAX_5000:
-			slot_base = KN02_PHYS_TC_0_START;	/*  0x1e000000  */
-			slot_size = 4*1048576;		/*  4 MB  */
+		switch (cpu->machine->machine_subtype) {
+		case MACHINE_DEC_3MAX_5000:
+			slot_base = KN02_PHYS_TC_0_START;/*  0x1e000000  */
+			slot_size = 4*1048576;		 /*  4 MB  */
 			break;
-		case MACHINE_3MIN_5000:
+		case MACHINE_DEC_3MIN_5000:
 			slot_base = 0x10000000;
 			slot_size = 0x4000000;		/*  64 MB  */
 			break;
-		case MACHINE_3MAXPLUS_5000:
+		case MACHINE_DEC_3MAXPLUS_5000:
 			slot_base = 0x1e000000;
 			slot_size = 0x800000;		/*  8 MB  */
 			break;
-		case MACHINE_MAXINE_5000:
+		case MACHINE_DEC_MAXINE_5000:
 			slot_base = 0x10000000;
 			slot_size = 0x4000000;		/*  64 MB  */
 			break;
 		default:
-			fatal("warning: DEC PROM slot_address() unimplemented for this machine type\n");
+			fatal("warning: DEC PROM slot_address() "
+			    "unimplemented for this machine type\n");
 		}
-		cpu->gpr[GPR_V0] = (int64_t)(int32_t)
-		    (0x80000000 + slot_base + slot_size * cpu->gpr[GPR_A0]);
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = (int64_t)(int32_t)
+		    (0x80000000 + slot_base + slot_size *
+		    cpu->cd.mips.gpr[MIPS_GPR_A0]);
 		break;
 	case 0x70:		/*  wbflush()  */
 		debug("[ DEC PROM wbflush(): TODO ]\n");
-		cpu->gpr[GPR_V0] = 0;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x7c:		/*  clear_cache(addr, len)  */
-		debug("[ DEC PROM clear_cache(0x%x,%i) ]\n", (uint32_t)cpu->gpr[GPR_A0], (int)cpu->gpr[GPR_A1]);
+		debug("[ DEC PROM clear_cache(0x%x,%i) ]\n",
+		    (uint32_t)cpu->cd.mips.gpr[MIPS_GPR_A0],
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A1]);
 		/*  TODO  */
-		cpu->gpr[GPR_V0] = 0;	/*  ?  */
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;	/*  ?  */
 		break;
 	case 0x80:		/*  getsysid()  */
 		/*  debug("[ DEC PROM getsysid() ]\n");  */
 		/*  TODO:  why did I add the 0x82 stuff???  */
-		cpu->gpr[GPR_V0] = ((uint32_t)0x82 << 24)
-		    + (cpu->emul->machine << 16) + (0x3 << 8);
-		cpu->gpr[GPR_V0] = (int64_t)(int32_t)cpu->gpr[GPR_V0];
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = ((uint32_t)0x82 << 24)
+		    + (cpu->machine->machine_subtype << 16) + (0x3 << 8);
+		cpu->cd.mips.gpr[MIPS_GPR_V0] =
+		    (int64_t)(int32_t)cpu->cd.mips.gpr[MIPS_GPR_V0];
 		break;
 	case 0x84:		/*  getbitmap()  */
 		debug("[ DEC PROM getbitmap(0x%08x) ]\n",
-		    (int)cpu->gpr[GPR_A0]);
-		store_buf(cpu, cpu->gpr[GPR_A0],
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A0]);
+		store_buf(cpu, cpu->cd.mips.gpr[MIPS_GPR_A0],
 		    (char *)&memmap, sizeof(memmap));
-		cpu->gpr[GPR_V0] = sizeof((memmap.bitmap));
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = sizeof((memmap.bitmap));
 		break;
 	case 0x88:		/*  disableintr()  */
 		debug("[ DEC PROM disableintr(): TODO ]\n");
-		cpu->gpr[GPR_V0] = 0;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x8c:		/*  enableintr()  */
 		debug("[ DEC PROM enableintr(): TODO ]\n");
-		cpu->gpr[GPR_V0] = 0;
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0x9c:		/*  halt()  */
 		debug("[ DEC PROM halt() ]\n");
-		cpu->emul->exit_without_entering_debugger = 1;
+		cpu->machine->exit_without_entering_debugger = 1;
 		cpu->running = 0;
+		cpu->dead = 1;
 		break;
 	case 0xa4:		/*  gettcinfo()  */
-		/*  These are just bogus values...  TODO  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO +  0, 0);	/*  revision  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO +  4, 50);	/*  clock period in nano seconds  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO +  8, 4);	/*  slot size in megabytes  TODO: not same for all models!!  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO + 12, 10);	/*  I/O timeout in cycles  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO + 16, 1);	/*  DMA address range in megabytes  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO + 20, 100);	/*  maximum DMA burst length  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO + 24, 0);	/*  turbochannel parity (yes = 1)  */
-		store_32bit_word(cpu, DEC_PROM_TCINFO + 28, 0);	/*  reserved  */
-		cpu->gpr[GPR_V0] = DEC_PROM_TCINFO;
+		/*
+		 *  These are just bogus values...  TODO
+		 *
+		 *   0:  revision
+		 *   4:  clock period in nano seconds
+		 *   8:  slot size in megabytes  TODO: not same for all models!
+		 *  12:  I/O timeout in cycles
+		 *  16:  DMA address range in megabytes
+		 *  20:  maximum DMA burst length
+		 *  24:  turbochannel parity (yes = 1)
+		 *  28:  reserved
+		 */
+		store_32bit_word(cpu, DEC_PROM_TCINFO +  0, 0);
+		store_32bit_word(cpu, DEC_PROM_TCINFO +  4, 50);
+		store_32bit_word(cpu, DEC_PROM_TCINFO +  8, 4);
+		store_32bit_word(cpu, DEC_PROM_TCINFO + 12, 10);
+		store_32bit_word(cpu, DEC_PROM_TCINFO + 16, 1);
+		store_32bit_word(cpu, DEC_PROM_TCINFO + 20, 100);
+		store_32bit_word(cpu, DEC_PROM_TCINFO + 24, 0);
+		store_32bit_word(cpu, DEC_PROM_TCINFO + 28, 0);
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = DEC_PROM_TCINFO;
 		break;
 	case 0xa8:		/*  int execute_cmd(char *)  */
 		i = 0;
-		while ((ch = read_char_from_memory(cpu, GPR_A0, i++)) != '\0')
-			console_putchar(ch);
-		console_putchar('\n');
-		cpu->gpr[GPR_V0] = 0;
+		while ((ch = mem_readchar(cpu, MIPS_GPR_A0, i++)) != '\0')
+			console_putchar(cpu->machine->main_console_handle, ch);
+		console_putchar(cpu->machine->main_console_handle, '\n');
+		cpu->cd.mips.gpr[MIPS_GPR_V0] = 0;
 		break;
 	case 0xac:		/*  rex()  */
-		debug("[ DEC PROM rex('%c') ]\n", (int)cpu->gpr[GPR_A0]);
-		switch (cpu->gpr[GPR_A0]) {
+		debug("[ DEC PROM rex('%c') ]\n",
+		    (int)cpu->cd.mips.gpr[MIPS_GPR_A0]);
+		switch (cpu->cd.mips.gpr[MIPS_GPR_A0]) {
 		case 'h':
 			debug("DEC PROM: rex('h') ==> halt\n");
-			cpu->emul->exit_without_entering_debugger = 1;
+			cpu->machine->exit_without_entering_debugger = 1;
 			cpu->running = 0;
+			cpu->dead = 1;
 			break;
 		case 'b':
-			debug("DEC PROM: rex('b') ==> reboot: TODO (halting CPU instead)\n");
-			cpu->emul->exit_without_entering_debugger = 1;
+			debug("DEC PROM: rex('b') ==> reboot: TODO "
+			    "(halting CPU instead)\n");
+			cpu->machine->exit_without_entering_debugger = 1;
 			cpu->running = 0;
+			cpu->dead = 1;
 			break;
 		default:
-			fatal("DEC prom emulation: unknown rex() a0=0x%llx ('%c')\n",
-			    (long long)cpu->gpr[GPR_A0], (char)cpu->gpr[GPR_A0]);
-			exit(1);
+			fatal("DEC prom emulation: unknown rex() a0=0x%llx ("
+			    "'%c')\n", (long long)cpu->cd.mips.gpr[MIPS_GPR_A0],
+			    (char)cpu->cd.mips.gpr[MIPS_GPR_A0]);
+			cpu->running = 0;
+			cpu->dead = 1;
 		}
 		break;
 	default:
-		cpu_register_dump(cpu, 1, 0x1);
+		cpu_register_dump(cpu->machine, cpu, 1, 0x1);
 		printf("a0 points to: ");
 		for (i=0; i<40; i++) {
 			unsigned char ch = '\0';
-			memory_rw(cpu, cpu->mem, cpu->gpr[GPR_A0] + i, &ch,
+			cpu->memory_rw(cpu, cpu->mem,
+			    cpu->cd.mips.gpr[MIPS_GPR_A0] + i, &ch,
 			    sizeof(ch), MEM_READ, CACHE_DATA | NO_EXCEPTIONS);
 			if (ch >= ' ' && ch < 126)
 				printf("%c", ch);
@@ -569,8 +675,12 @@ void decstation_prom_emul(struct cpu *cpu)
 				printf("[%02x]", ch);
 		}
 		printf("\n");
-		fatal("PROM emulation: unimplemented callback vector 0x%x\n", vector);
+		fatal("PROM emulation: unimplemented callback vector 0x%x\n",
+		    vector);
 		cpu->running = 0;
+		cpu->dead = 1;
 	}
+
+	return 1;
 }
 
