@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,16 +25,19 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: memory_fast_v2h.c,v 1.5 2005/01/09 01:55:31 debug Exp $
+ *  $Id: memory_fast_v2h.c,v 1.12 2005/02/18 06:01:18 debug Exp $
  *
  *  Fast virtual memory to host address, used by binary translated code.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
-#include "misc.h"
+
 #include "bintrans.h"
+#include "cpu.h"
 #include "memory.h"
+#include "cpu_mips.h"
+#include "misc.h"
 
 
 #ifdef BINTRANS
@@ -64,7 +67,9 @@ unsigned char *fast_vaddr_to_hostaddr(struct cpu *cpu,
 	/*  printf("fast_vaddr_to_hostaddr(): cpu=%p, vaddr=%016llx, wf=%i\n",
 	    cpu, (long long)vaddr, writeflag);  */
 
-#if 1
+#if 0
+/*  Hm. This seems to work now, so this #if X can be removed. (?)  */
+
 /*
  *  TODO:
  *
@@ -77,37 +82,47 @@ if ((vaddr & 0xc0000000ULL) >= 0xc0000000ULL && writeflag) {
 
 
 	vaddr_page = vaddr & ~0xfff;
-	i = start_and_stop = cpu->bintrans_next_index;
+	i = start_and_stop = cpu->cd.mips.bintrans_next_index;
 	n = 0;
 	for (;;) {
-		if (cpu->bintrans_data_vaddr[i] == vaddr_page &&
-		    cpu->bintrans_data_hostpage[i] != NULL &&
-		    cpu->bintrans_data_writable[i] >= writeflag) {
+		if (cpu->cd.mips.bintrans_data_vaddr[i] == vaddr_page &&
+		    cpu->cd.mips.bintrans_data_hostpage[i] != NULL &&
+		    cpu->cd.mips.bintrans_data_writable[i] >= writeflag) {
 			uint64_t tmpaddr;
 			unsigned char *tmpptr;
 			int tmpwf;
 
 			if (n < 3)
-				return cpu->bintrans_data_hostpage[i]
+				return cpu->cd.mips.bintrans_data_hostpage[i]
 				    + (vaddr & 0xfff);
 
-			cpu->bintrans_next_index = start_and_stop - 1;
-			if (cpu->bintrans_next_index < 0)
-				cpu->bintrans_next_index = MAX - 1;
+			cpu->cd.mips.bintrans_next_index = start_and_stop - 1;
+			if (cpu->cd.mips.bintrans_next_index < 0)
+				cpu->cd.mips.bintrans_next_index = MAX - 1;
 
-			tmpptr  = cpu->bintrans_data_hostpage[cpu->bintrans_next_index];
-			tmpaddr = cpu->bintrans_data_vaddr[cpu->bintrans_next_index];
-			tmpwf   = cpu->bintrans_data_writable[cpu->bintrans_next_index];
+			tmpptr  = cpu->cd.mips.bintrans_data_hostpage[
+			    cpu->cd.mips.bintrans_next_index];
+			tmpaddr = cpu->cd.mips.bintrans_data_vaddr[
+			    cpu->cd.mips.bintrans_next_index];
+			tmpwf   = cpu->cd.mips.bintrans_data_writable[
+			    cpu->cd.mips.bintrans_next_index];
 
-			cpu->bintrans_data_hostpage[cpu->bintrans_next_index] = cpu->bintrans_data_hostpage[i];
-			cpu->bintrans_data_vaddr[cpu->bintrans_next_index] = cpu->bintrans_data_vaddr[i];
-			cpu->bintrans_data_writable[cpu->bintrans_next_index] = cpu->bintrans_data_writable[i];
+			cpu->cd.mips.bintrans_data_hostpage[
+			    cpu->cd.mips.bintrans_next_index] =
+			    cpu->cd.mips.bintrans_data_hostpage[i];
+			cpu->cd.mips.bintrans_data_vaddr[
+			    cpu->cd.mips.bintrans_next_index] =
+			    cpu->cd.mips.bintrans_data_vaddr[i];
+			cpu->cd.mips.bintrans_data_writable[
+			    cpu->cd.mips.bintrans_next_index] =
+			    cpu->cd.mips.bintrans_data_writable[i];
 
-			cpu->bintrans_data_hostpage[i] = tmpptr;
-			cpu->bintrans_data_vaddr[i] = tmpaddr;
-			cpu->bintrans_data_writable[i] = tmpwf;
+			cpu->cd.mips.bintrans_data_hostpage[i] = tmpptr;
+			cpu->cd.mips.bintrans_data_vaddr[i] = tmpaddr;
+			cpu->cd.mips.bintrans_data_writable[i] = tmpwf;
 
-			return cpu->bintrans_data_hostpage[cpu->bintrans_next_index] + (vaddr & 0xfff);
+			return cpu->cd.mips.bintrans_data_hostpage[
+			    cpu->cd.mips.bintrans_next_index] + (vaddr & 0xfff);
 		}
 
 		n ++;
@@ -126,28 +141,41 @@ if ((vaddr & 0xc0000000ULL) >= 0xc0000000ULL && writeflag) {
 
 	for (i=0; i<cpu->mem->n_mmapped_devices; i++)
 		if (paddr >= cpu->mem->dev_baseaddr[i] &&
-		    paddr < cpu->mem->dev_baseaddr[i] + cpu->mem->dev_length[i]) {
+		    paddr < cpu->mem->dev_baseaddr[i] +
+		    cpu->mem->dev_length[i]) {
 			if (cpu->mem->dev_flags[i] & MEM_BINTRANS_OK) {
 				paddr -= cpu->mem->dev_baseaddr[i];
 
 				if (writeflag) {
 					uint64_t low_paddr = paddr & ~0xfff;
 					uint64_t high_paddr = paddr | 0xfff;
-					if (!(cpu->mem->dev_flags[i] & MEM_BINTRANS_WRITE_OK))
+					if (!(cpu->mem->dev_flags[i] &
+					    MEM_BINTRANS_WRITE_OK))
 						return NULL;
 
-					if (low_paddr < cpu->mem->dev_bintrans_write_low[i])
-					    cpu->mem->dev_bintrans_write_low[i] = low_paddr;
-					if (high_paddr > cpu->mem->dev_bintrans_write_high[i])
-					    cpu->mem->dev_bintrans_write_high[i] = high_paddr;
+					if (low_paddr < cpu->mem->
+					    dev_bintrans_write_low[i])
+						cpu->mem->
+						    dev_bintrans_write_low[i] =
+						    low_paddr;
+					if (high_paddr > cpu->mem->
+					    dev_bintrans_write_high[i])
+						cpu->mem->
+						    dev_bintrans_write_high[i]
+						    = high_paddr;
 				}
 
-				cpu->bintrans_next_index --;
-				if (cpu->bintrans_next_index < 0)
-					cpu->bintrans_next_index = MAX - 1;
-				cpu->bintrans_data_hostpage[cpu->bintrans_next_index] = cpu->mem->dev_bintrans_data[i] + (paddr & ~0xfff);
-				cpu->bintrans_data_vaddr[cpu->bintrans_next_index] = vaddr_page;
-				cpu->bintrans_data_writable[cpu->bintrans_next_index] = writeflag;
+				cpu->cd.mips.bintrans_next_index --;
+				if (cpu->cd.mips.bintrans_next_index < 0)
+					cpu->cd.mips.bintrans_next_index =
+					    MAX - 1;
+				cpu->cd.mips.bintrans_data_hostpage[cpu->
+				    cd.mips.bintrans_next_index] = cpu->mem->
+				    dev_bintrans_data[i] + (paddr & ~0xfff);
+				cpu->cd.mips.bintrans_data_vaddr[cpu->
+				    cd.mips.bintrans_next_index] = vaddr_page;
+				cpu->cd.mips.bintrans_data_writable[cpu->
+				    cd.mips.bintrans_next_index] = writeflag;
 				return cpu->mem->dev_bintrans_data[i] + paddr;
 			} else
 				return NULL;
@@ -163,12 +191,16 @@ if ((vaddr & 0xc0000000ULL) >= 0xc0000000ULL && writeflag) {
 	if (writeflag)
 		bintrans_invalidate(cpu, paddr);
 
-	cpu->bintrans_next_index --;
-	if (cpu->bintrans_next_index < 0)
-		cpu->bintrans_next_index = MAX - 1;
-	cpu->bintrans_data_hostpage[cpu->bintrans_next_index] = memblock + (offset & ~0xfff);
-	cpu->bintrans_data_vaddr[cpu->bintrans_next_index] = vaddr_page;
-	cpu->bintrans_data_writable[cpu->bintrans_next_index] = writeflag;  /*  ok - 1;  */
+	cpu->cd.mips.bintrans_next_index --;
+	if (cpu->cd.mips.bintrans_next_index < 0)
+		cpu->cd.mips.bintrans_next_index = MAX - 1;
+	cpu->cd.mips.bintrans_data_hostpage[cpu->cd.mips.bintrans_next_index] =
+	    memblock + (offset & ~0xfff);
+	cpu->cd.mips.bintrans_data_vaddr[cpu->cd.mips.bintrans_next_index] =
+	    vaddr_page;
+	cpu->cd.mips.bintrans_data_writable[cpu->cd.mips.bintrans_next_index] =
+	    writeflag;  /*  ok - 1;  */
+
 	return memblock + offset;
 }
 
