@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: symbol.c,v 1.17 2005/01/09 01:55:31 debug Exp $
+ *  $Id: symbol.c,v 1.22 2005/02/13 12:04:42 debug Exp $
  *
  *  Address to symbol translation routines.
  *
@@ -42,11 +42,6 @@
 
 #include "symbol.h"
 
-#ifdef HACK_STRTOLL
-#define strtoll strtol
-#define strtoull strtoul
-#endif
-
 
 #define	SYMBOLBUF_MAX	100
 
@@ -202,7 +197,8 @@ void add_symbol_name(struct symbol_context *sc,
 	struct symbol *s;
 
 	if (sc->sorted_array) {
-		fprintf(stderr, "add_symbol_name(): the symbol array is already sorted\n");
+		fprintf(stderr, "add_symbol_name(): Internal error: the "
+		    "symbol array is already sorted\n");
 		exit(1);
 	}
 
@@ -215,12 +211,11 @@ void add_symbol_name(struct symbol_context *sc,
 		exit(1);
 	}
 
-	s->name = malloc(strlen(name) + 1);
+	s->name = strdup(name);
 	if (s->name == NULL) {
 		fprintf(stderr, "out of memory\n");
 		exit(1);
 	}
-	strcpy(s->name, name);
 	s->addr = addr;
 	s->len  = len;
 	s->type = type;
@@ -271,11 +266,13 @@ void symbol_readfile(struct symbol_context *sc, char *fname)
 			fscanf(f, "%s %s\n", b3,b4);
 		}
 
-		/*  printf("b1='%s' b2='%s' b3='%s' b4='%s'\n", b1,b2,b3,b4);  */
+		/*  printf("b1='%s' b2='%s' b3='%s' b4='%s'\n",
+		    b1,b2,b3,b4);  */
 		addr = strtoull(b1, NULL, 16);
 		len  = strtoull(b2, NULL, 16);
 		type = b3[0];
-		/*  printf("addr=%016llx len=%016llx type=%i\n", addr, len, type);  */
+		/*  printf("addr=%016llx len=%016llx type=%i\n",
+		    addr, len, type);  */
 
 		if (type == 't' || type == 'r' || type == 'g')
 			continue;
@@ -285,8 +282,7 @@ void symbol_readfile(struct symbol_context *sc, char *fname)
 
 	fclose(f);
 
-	debug("'%s': %i symbols\n", fname, sc->n_symbols - cur_n_symbols,
-	    fname);
+	debug("%i symbols\n", sc->n_symbols - cur_n_symbols);
 }
 
 
